//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : Roma.java
//
// Defines      : interface Roma
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.Kxc;

/**
* This interface introduces various constants used by the other
* Roma classes in the package.
*/
public interface Roma
{
  // SetSelector method options 
  
  public final static int USE_NONE = Kxc.POPT_NONE;
  
  public final static int USE_MSGID = Kxc.POPT_USEMSGID;
  
  public final static int USE_CORRELID = Kxc.POPT_USECORRELID;
  
  public final static int USE_MSGID_AND_CORRELID = USE_MSGID | USE_CORRELID;
  
  // Queue open options
  
  /**
  * Queue open option that enables both that both store() and
  * retrieve() actions to be permitted and that non-destructive
  * operations also be allowed.
  */
  public final static int OPEN_DEFAULT = Kxc.OPEN_OP_NONE;
  
  /**
  * Queue open option that enables store() to be used.
  */
  public final static int OPEN_OUTPUT = Kxc.OPEN_OP_OUTPUT;
  
  /**
  * Queue open option that enables retrieve() to be used. 
  */
  public final static int OPEN_INPUT = Kxc.OPEN_OP_INPUT;
  
  /**
  * Queue open option that permits non-destructive actions.
  */
  public final static int OPEN_PEEK = Kxc.OPEN_OP_PEEK;
  
  /**
  * Queue open option. If MSMQ is being used, the queue is created
  * as non-transactional if it does not already exist.
  */
  public final static int OPEN_NON_XACTIONAL = Kxc.OPEN_OP_NON_XACTIONAL;
  
  /**
  * Queue open option. Open for exclusive input.
  */
  public final static int OPEN_EXCLUSIVE = Kxc.OPEN_OP_EXCLUSIVE;
  
  /**
  * Queue open option. Delete the queue when closed.
  */
  public final static int OPEN_DELETE_ON_CLOSE = Kxc.OPEN_OP_DELETE_ON_CLOSE;
  
  /**
  * Queue open option. The queue must already exist prior to
  * the open call. If not, the open call will fail.
  */
  public final static int OPEN_MUST_EXIST = Kxc.OPEN_OP_MUST_EXIST;
  
  // Syncpoints 
  
  /**
  * Syncpoint type. Messages are not subject to syncpointing.
  */
  public static final int SYNC_NONE = Kxc.SYNC_NONE;
  
  /**
  * Syncpoint type. Roma controls the syncpointing.
  */
  public static final int SYNC_LOCAL = Kxc.SYNC_LOCAL;
  
  /**
  * Syncpoint type. An application outside of Roma controls
  * the syncpointing.
  */
  public static final int SYNC_GLOBAL = Kxc.SYNC_GLOBAL;
  
  /**
  * Syncpoint type. Roma uses the setting from the associated 
  * API profile or, if the profile value if not available, the
  * default of NONE.
  */
  public static final int SYNC_PROFILE = Kxc.USEDEFAULT;
  
  /**
  * Subject style for publish/subscribe. 
  */
  public static final int STYLE_TIBRENDEZVOUS = Kxc.STYLE_TIBRENDEZVOUS;
  public static final int STYLE_MQSERIES      = Kxc.STYLE_MQSERIES;
  public static final int STYLE_ROMA          = Kxc.STYLE_ROMA;
  
  // Reporting
  
  /**
  * Default action for error routine to perform. This will 
  * override any action specified in an associated error profile.
  */
  public static final int ACTION_DEFAULT = Kxc.EP_ACTION_DEFAULT;
  
  /**
  * Error analysis routine should send a notification message.
  */
  public static final int ACTION_NOTIFY = Kxc.EP_ACTION_NOTIFY;
  
  /**
  * Error analysis routine should do a put to the exception
  * message service.
  */
  public static final int ACTION_EMS = Kxc.EP_ACTION_EMS;
  
  /**
  * Error analysis routine should send an alert.
  */
  public static final int ACTION_ALERT = Kxc.EP_ACTION_ALERT;
  
  /**
  * Error analysis routine should do a write to the log.
  */
  public static final int ACTION_LOG = Kxc.EP_ACTION_LOG;
  
  /**
  * Error analysis routine should do a reset of the error and
  * carry on.
  */
  public static final int ACTION_RESET = Kxc.EP_ACTION_RESET;
  
  /**
  * Error analysis routine should take no action at all.
  */
  public static final int ACTION_NOACTION = Kxc.EP_ACTION_NOACTION;
  
  /**
  * Error analysis routine should send a raw message to 
  * the exception message service.
  */
  public static final int ACTION_EMSRAW = Kxc.EP_ACTION_EMSRAW;
  
  // Logging
  
  /**
  * An indication of the severity of the abnormality;
  * in this case, for information only.
  */
  public static final int SEV_INFO = 0;
  
  /**
  * An indication of the severity of the abnormality;
  * in this case, a warning.
  */
  public static final int SEV_WARNING = 1;
  
  /**
  * An indication of the severity of the abnormality;
  * in this case, a warning.
  */
  public static final int SEV_ERROR = 2;
  
  /**
  * An indication of the severity of the abnormality;
  * in this case, a severe abnormality.
  */
  public static final int SEV_SEVERE = 3;
  
  /**
  * Indication of the type of Roma-supplied formatting to use;
  * in this case, none.
  */
  public static final int FORMAT_NONE = 0;
  
  /**
  * Indication of the type of Roma-supplied formatting to use;
  * in this case do not print a timestamp.
  */
  public static final int FORMAT_NOTIMESTAMP = 1;
  
  /**
  * Indication of the type of Roma-supplied formatting to use;
  * in this case do not print a severity.
  */
  public static final int FORMAT_NOSEVERITY = 2;
  
  /**
  * Indication of the type of Roma-supplied formatting to use;
  * in this case do a hex dump of the user data. If this flag is
  * not specified then user data is interpreted as text. The 
  * default is to have this flag set.
  */
  public static final int FORMAT_HEXDUMP = 4;
  
  // Error status
  
  /**
  * No Roma error has occurred.
  */
  public static final int ERROR_STATUS_NONE = Kxc.ERRSTAT_NONE;
  
  /**
  * Information on a main type of error is available.
  * 
  */
  public static final int ERROR_STATUS_MAIN = Kxc.ERRSTAT_MAIN;
  
  /**
  * Information on both main and sub errors are available.
  *
  */
  public static final int ERROR_STATUS_BOTH = Kxc.ERRSTAT_MAIN|Kxc.ERRSTAT_SUB;
  
  // Application Definition
  
  /**
  * No application options set. This is the default.
  */
  public static final int APP_NONE = Kxc.APPOPT_NONE;
  
  /**
  * Indicates that the application has to have sole
  * access to the queue.  If another application is already using
  * the queue, a failure will occur when the RomaClient or
  * RomaBusinessElement is constructed.
  */
  public static final int APP_EXCLUSIVE = Kxc.APPOPT_EXCLUSIVE;
  
  /**
  * Indicates that if the queue is on MSMQ and does not already
  * exist, Roma should create the queue as a non-transactional
  * queue. Otherwise, Roma creates the queue as a transactional
  * queue.
  */
  public static final int APP_NON_XACTIONAL = Kxc.APPOPT_NON_XACTIONAL;
  
  /**
  * Indicates that Roma expects MQSeries to do the transaction
  * control. This option is only applicable when using MQSeries.
  */
  public static final int APP_MQSERIES_XACTION = Kxc.APPOPT_MQSERIES_XACTION;

  /**
  * Indicates that pending transactions must be rolled back on
  * disconnection. By default Roma commits pending transactions
  * on disconnection.
  */
  public static final int APP_ROLLBK_OS_XACT = Kxc.APPOPT_ROLLBK_OS_XACT;


  // Constants for Messages

  /**
  * Message type. Indicates that a RomaClient is sending a
  * request and does not require a reply.
  */
  public static final int DATAGRAM = Kxc.DATAGRAM;
  
  /**
  * Message type. Indicates that a RomaClient is sending a
  * request which does require a reply.
  */
  public static final int REQUEST = Kxc.REQUEST;
  
  /**
  * Message type. Sent by a RomaBusinessElement that is
  * responding to an earlier RomaClient request.
  */
  public static final int RESPONSE = Kxc.REPLY;
  
  /**
  * Message type. Indicates that the message has been sent back to
  * the client, ignoring the workflow. This can be used, for
  * example :-
  * <ul>
  * <li> to return an error (i.e. an abandonment)
  * <li> to indicate that a server has received a message
  * </ul>
  */
  public static final int NOTIFICATION    = Kxc.REPLY | Kxc.NOTIFY;
  public static final int PUBLICATION     = Kxc.PUBLICATION;
  public static final int POSITIVE_PUBSUB = Kxc.POSITIVE_PUBSUB;
  public static final int NEGATIVE_PUBSUB = Kxc.NEGATIVE_PUBSUB;
  public static final int ADVISORY        = Kxc.ADVISORY;
  public static final int POSITIVE_COA    = Kxc.MSGTYPE_POSITIVE_COA;
  public static final int NEGATIVE_COA    = Kxc.MSGTYPE_NEGATIVE_COA;
  public static final int POSITIVE_COD    = Kxc.MSGTYPE_POSITIVE_COD;
  public static final int NEGATIVE_COD    = Kxc.MSGTYPE_NEGATIVE_COD;
  
  /**
  * Default message priority is to be used.
  */
  public static final int PRIORITY_DEFAULT = -1;
  
  /**
  * Default message priority as specified by the API profile is to
  * be used.
  */
  public static final int PRIORITY_PROFILE = Kxc.USEDEFAULT;
  
  /**
  * Perisitence enabled flag
  */
  public static final int PERSISTENCE_YES = Kxc.MSGPERSIST_YES;
  
  /**
  * Persistence disabled flag.
  */
  public static final int PERSISTENCE_NO = Kxc.MSGPERSIST_NO;
  
  /**
  * Default persistence is to be used.
  */
  public static final int PERSISTENCE_PROFILE = Kxc.USEDEFAULT;
  
  /**
  * Binary message format identifier. This is the default format.
  */
  public static final int FORMAT_BINARY = Kxc.MSGFORMAT_BINARY;
  
  /**
  * String message format identifier.
  */
  static final int FORMAT_STRING = Kxc.MSGFORMAT_STRING;
  
  /**
  * Signed binary message format identifier.
  */
  static final int FORMAT_BINARY_SIGN = Kxc.MSGFORMAT_BINARY_SIGN;
  
  /**
  * Signed character string message format identifier.
  */
  static final int FORMAT_STRING_SIGN = Kxc.MSGFORMAT_STRING_SIGN;
  
  /**
  * Encrypted binary message format identifier.
  */
  static final int FORMAT_BINARY_ENC = Kxc.MSGFORMAT_BINARY_ENC;
  
  /**
  * Encrypted character string message format identifier.
  */
  static final int FORMAT_STRING_ENC = Kxc.MSGFORMAT_STRING_ENC;
  
  /**
  * Signed, encrypted binary message format identifier.
  */
  static final int FORMAT_BINARY_SIGN_ENC = Kxc.MSGFORMAT_BINARY_SIGN_ENC;
  
  /**
  * Signed, encrypted character string message format identifier.
  */
  static final int FORMAT_STRING_SIGN_ENC = Kxc.MSGFORMAT_STRING_SIGN_ENC;
  
  /**
  * User defined message format.
  */
  public static final int FORMAT_USER = Kxc.MSGFORMAT_USER;
  
  /**
  * Message format identified by a text name picked up from the
  * formatName field.
  */
  public static final int FORMAT_USENAME = Kxc.MSGFORMAT_USENAME;
  
  /**
  * Retrieve message format from Put API profile. If profile 
  * not available or if it has no message format attribute value, 
  * the default message format is FORMAT_BINARY.
  */
  public static final int FORMAT_PROFILE = Kxc.MSGFORMAT_USEDEFAULT;
  
  
  /**
  * Message never expires. This is the default message expiry
  * interval.
  */
  public static final int EXPIRY_NEVER = Kxc.MSGEXP_NEVER;
  
  /**
  * Retrieve message expiry interval from Put API profile. If profile
  * is not available or if it has no expiry attribute value, the
  * default is EXPIRY_NEVER.
  */
  public static final int EXPIRY_PROFILE = Kxc.MSGEXP_USEDEFAULT;
  
  /**
  * Message codepage value.
  */
  public static final int CODEPAGE_USER = Kxc.MSGCODEPAGE_USER;
  
  /**
  * Retrieve message codepage value from a Put API profile. If
  * the profile is not available or if it has no codepage value
  * then the default is CODEPAGE_USER.
  */
  public static final int CODEPAGE_PROFILE = Kxc.MSGCODEPAGE_USEDEFAULT;
  
  /**
  * Default message encoding used.
  */
  public static final int ENCODING_DEFAULT = Kxc.MSGENC_DEFAULT;
  
  /**
  * Normal message encoding used.
  */
  public static final int ENCODING_NORMAL = Kxc.MSGENC_NORMAL;
  
  /**
  * Reversed message encoding used.
  */
  public static final int ENCODING_REVERSED = Kxc.MSGENC_REVERSED;
  
  /**
  * Retrieve message encoding value from a Put API profile. If
  * the profile is not available or if it has no encoding value
  * then the default is ENCODING_DEFAULT.
  */
  public static final int ENCODING_PROFILE = Kxc.MSGENC_USEDEFAULT;
  
  /**
  * Message status value. In this case, anonymous message
  * authentication used.
  */
  public static final int STATUS_NONE = 0;
  
  /**
  * Message status value. Message authenticated.
  */
  public static final int STATUS_AUTHENTICATED = 1;
  
  /**
  * Message status value. Native message.
  */
  public static final int STATUS_NATIVE = 2;
  
  /**
  * Message status value. Safe to respond to this message.
  */
  public static final int STATUS_OK_TO_RESPOND = 4;
  
  /**
  * Message status value. Safe to send a notification 
  * message in reponse to this message.
  */
  public static final int STATUS_OK_TO_NOTIFY = 8;
  
  /**
  * Backout count capabilities not supported. 
  */
  public static final int BACKOUT_UNSUPPORTED = -1;
  
  /**
  * Maximum length of the message buffer.
  */
  public static final int MAX_LEN = Kxc.MSGMAX_LEN;
  
  /**
  * Maximum length of the tag buffer.
  */
  public static final int MAX_TAGLEN  = Kxc.MSGMAX_TAGLEN;
  
  // Get Options
  
  /**
  * Theme control option that specifies if theme will be closed or
  * retained by the RomaClient/RomaBusinessElement.get() method;
  * in this case, close theme.
  */
  public static final int THEME_CLOSE = Kxc.GOPT_CLOSETHEME;
  
  /**
  * Theme control option that specifies if theme will be closed or
  * retained by the RomaClient/RomaBusinessElement.get() method;
  * in this case, retain theme.
  */
  public static final int THEME_RETAIN = Kxc.GOPT_RETAINTHEME;
  
  /**
  * Theme control option that specifies if theme is to be closed or
  * retained by the method retrieving the message.
  * <ul>
  * <li>RomaClient.get() will close the theme.
  * <li>RomaBusinessElement.get() will retain the theme unless the 
  *     message is a datagram in which case the theme is closed.
  * <li>RomaBusinessElement.getExceptionm() call will close the
  *     theme.
  * </ul>
  */
  public static final int THEME_DEFAULT_CONTROL = 0;
  
  /**
  * Get selector that specifies criteria for retrieving a message;
  * in this case, get any message.
  */
  public static final int BY_NONE = Kxc.GOPT_NONE;
  
  /**
  * Get selector that specifies criteria for retrieving a message;
  * in this case, get message with matched message identifier.
  * Message identifier should be specified by setMsgId() method.
  */
  public static final int BY_MSGID = Kxc.GOPT_USEMSGID;
  
  /**
  * Get selector that specifies criteria for retrieving a message;
  * in this case, get message by correlation id. Correlation id
  * should be specified by setCorrelId() method.
  */
  public static final int BY_CORRELID = Kxc.GOPT_USECORRELID;
  
  /**
  * Get selector specifies criteria for retrieving a message;
  * in this case, get message with matched theme. Theme should be
  * specified by setTheme() method.
  */
  public static final int BY_THEME = Kxc.GOPT_USETHEME;
  
  /**
  * Get selector that specifies criteria for retrieving a message;
  * in this case, get message using a combination of
  * BY_MSGID and BY_THEME.
  */
  public static final int BY_MSGID_AND_THEME = BY_MSGID | BY_THEME;
  
  /**
  * Get selector that specifies criteria for retrieving a message;
  * in this case, get message using combination of
  * BY_MSGID and BY_CORRELID.
  */
  public static final int BY_MSGID_AND_CORRELID = BY_MSGID | BY_CORRELID;
  
  /**
  * Get selector that specifies criteria for retrieving a message;
  * in this case, get any message
  */
  public static final int ANY_THEME = Kxc.GOPT_ANYTHEME;
  
  /**
  * Get the first message.
  */
  public static final int GET_FIRST = Kxc.GOPT_NONE;
  
  /**
  * Peek the first message. Note that a peek will not remove 
  * the message from the queue.
  */
  public static final int PEEK_FIRST = Kxc.GOPT_PEEK_FIRST;
  
  /**
  * Peek the next message. Note that a peek will not remove 
  * the message from the queue.
  */
  public static final int PEEK_NEXT = Kxc.GOPT_PEEK_NEXT;
  
  /**
  * Peek the message under the current read position. Note that
  * a peek will not remove the message from the queue.
  */
  public static final int PEEK_SAME = Kxc.GOPT_PEEK_SAME;
  
  /**
  * Get the message under the current read position.
  * Note that this is not a normal peek operation. It is
  * analogous to PEEK_SAME but it <strong>does</strong>
  * remove the message from the queue.
  */
  public static final int GET_PEEKED = Kxc.GOPT_GET_PEEKED;
  
  // Put Options
  
  // selector mask
  final int SELECTOR = Kxc.POPT_USEMSGID | Kxc.POPT_USECORRELID;
  
  // theme allocation control mask
  final int THEME_USAGE = Kxc.POPT_NEWTHEME | Kxc.POPT_USETHEME;
  
  // theme retaining control mask
  final int THEME_CONTROL = Kxc.POPT_CLOSETHEME | Kxc.POPT_RETAINTHEME;
  
  // RomaExceptionMessage
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case no information provided.
  */
  public static final int ELEMENT_NONE = Kxc.EXC_NONE;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case Roma provides the timestamp.
  */
  public static final int ELEMENT_TIMESTAMP = Kxc.EXC_TIMESTAMP;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case Roma provides details of
  * its own configuration.
  */
  public static final int ELEMENT_CONFIG = Kxc.EXC_ROMADATA;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case Roma provides details of
  * the system configuration.
  */
  public static final int ELEMENT_SYSCONFIG = Kxc.EXC_SYSTEMDATA;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case error information is provided.
  */
  public static final int ELEMENT_ERROR = Kxc.EXC_ERRORDATA;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case user data is provided.
  */
  public static final int ELEMENT_USERDATA = Kxc.EXC_USERDATA;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case the base message contains
  * meaningful information.
  */
  public static final int ELEMENT_MESSAGE = Kxc.MSGPART_MESSAGE << 8;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case the workflow details.
  */
  public static final int ELEMENT_WORKFLOW = Kxc.MSGPART_WORKFLOW << 8;
  
  /**
  * Indicator of what information is included as part of the
  * exception message; in this case all information will be passed.
  */
  public static final int ELEMENT_ALL_PUT =
    ELEMENT_TIMESTAMP |
    ELEMENT_CONFIG |
    ELEMENT_SYSCONFIG |
    ELEMENT_ERROR |
    ELEMENT_USERDATA |
    ELEMENT_MESSAGE
    ;
  
  /**
  * Indicator of which information is included as part of the
  * exception message; in this case all of the information of a
  * put operation plus the workflow details if available.
  */
  public static final int ELEMENT_ALL_GET = ELEMENT_ALL_PUT | ELEMENT_WORKFLOW;
  
  // Constants representing all of the different API call
  // types in which an error can occur. 
  
  /**
  * Application call type.
  */
  public static final int CALLTYPE_APPLICATION = Kxc.CALLTYPE_APPLICATION;
  
  /**
  * Put request call type.
  */
  public static final int CALLTYPE_PUTREQUEST = Kxc.CALLTYPE_PUTREQUEST;
  
  /**
  * Get request call type.
  */
  public static final int CALLTYPE_GETREQUEST = Kxc.CALLTYPE_GETREQUEST;
  
  /**
  * Put response call type.
  */
  public static final int CALLTYPE_PUTRESPONSE = Kxc.CALLTYPE_PUTRESPONSE;
  
  /**
  * Get response call type.
  */
  public static final int CALLTYPE_GETRESPONSE = Kxc.CALLTYPE_GETRESPONSE;
  
  /**
  * Put to bridge call type.
  */
  public static final int CALLTYPE_BRIDGEPUT = Kxc.CALLTYPE_BRIDGEPUT;
  
  /**
  * get from bridge call type.
  */
  public static final int CALLTYPE_BRIDGEGET = Kxc.CALLTYPE_BRIDGEGET;
  
  /**
  * Put exception message call type.
  */
  public static final int CALLTYPE_PUTEXCEPTION = Kxc.CALLTYPE_PUTEXCEPTION;
  
  /**
  * Initialize call type.
  */
  public static final int CALLTYPE_INITIALIZE = Kxc.CALLTYPE_INITIALIZE;
  
  /**
  * Terminate call type.
  */
  public static final int CALLTYPE_TERMINATE = Kxc.CALLTYPE_TERMINATE;
  
  /**
  * Commit call type.
  */
  public static final int CALLTYPE_COMMIT = Kxc.CALLTYPE_COMMIT;
  
  /**
  * Rollback call type.
  */
  public static final int CALLTYPE_ROLLBACK = Kxc.CALLTYPE_ROLLBACK;
  
  /**
  * Free buffer call type.
  */
  public static final int CALLTYPE_FREEBUFFER = Kxc.CALLTYPE_FREEBUFFER;
  
  /**
  * Close handle call type.
  */
  public static final int CALLTYPE_CLOSEHANDLE = Kxc.CALLTYPE_CLOSEHANDLE;
  
  /**
  * Interpret message call type.
  */
  public static final int CALLTYPE_INTERPRETMSG = Kxc.CALLTYPE_INTERPRETMSG;
  
  /**
  * Write to log call type.
  */
  public static final int CALLTYPE_WRITELOG = Kxc.CALLTYPE_WRITELOG;
  
  /**
  * Send alert call type.
  */
  public static final int CALLTYPE_SENDALERT = Kxc.CALLTYPE_SENDALERT;
  
  /**
  * Open a storage queue call type.
  */
  public static final int CALLTYPE_OPENSTORAGEQ = Kxc.CALLTYPE_OPENSTORAGEQ;
  
  /**
  * Close a storage queue call type.
  */
  public static final int CALLTYPE_CLOSESTORAGEQ = Kxc.CALLTYPE_CLOSESTORAGEQ;
  
  /**
  * Delete a storage queue call type.
  */
  public static final int CALLTYPE_DELETESTORAGEQ = Kxc.CALLTYPE_DELETESTORAGEQ;
  
  /**
  * Put to a storage queue call type.
  */
  public static final int CALLTYPE_PUTTOSTORAGEQ = Kxc.CALLTYPE_PUTTOSTORAGEQ;
  
  /**
  * Get from a storage queue call type.
  */
  public static final int CALLTYPE_GETFROMSTORAGEQ = Kxc.CALLTYPE_GETFROMSTORAGEQ;
  
  /**
  * Theme query call type.
  */
  public static final int CALLTYPE_QUERYTHEME = Kxc.CALLTYPE_QUERYTHEME;
  
  /**
  * Theme build call type.
  */
  public static final int CALLTYPE_BUILDTHEME = Kxc.CALLTYPE_BUILDTHEME;
  
  /**
  * Purge queue to EMS call type.
  */
  public static final int CALLTYPE_PURGEQTOEMS = Kxc.CALLTYPE_PURGEQTOEMS;
  
  /**
  * Report error call type.
  */
  public static final int CALLTYPE_REPORTERROR = Kxc.CALLTYPE_REPORTERROR;
  
  /**
  * Context query call type.
  */
  public static final int CALLTYPE_QUERYCONTEXT = Kxc.CALLTYPE_QUERYCONTEXT;
  
  /**
  * set up LDAP service call type.
  */
  public static final int CALLTYPE_SETLDAPSERVICE = Kxc.CALLTYPE_SETLDAPSERVICE;
  
  /**
  * Set ILS data call type.
  */
  public static final int CALLTYPE_SETILSDATA = Kxc.CALLTYPE_SETILSDATA;
  
  /**
  * Get ILS data call type.
  */
  public static final int CALLTYPE_GETILSDATA = Kxc.CALLTYPE_GETILSDATA;
  
  /**
  * Directory look up call type.
  */
  public static final int CALLTYPE_DIRLOOKUP = Kxc.CALLTYPE_DIRLOOKUP;
  
  /**
  * Get directory item distinguished name call type.
  */
  public static final int CALLTYPE_DIRGETDN = Kxc.CALLTYPE_DIRGETDN;
  
  /**
  * Update directory item call type.
  */
  public static final int CALLTYPE_DIRUPDATEITEM = Kxc.CALLTYPE_DIRUPDATEITEM;
  
  /**
  * Delete directory item call type.
  */
  public static final int CALLTYPE_DIRDELETEITEM = Kxc.CALLTYPE_DIRDELETEITEM;
  
  /**
  * Get directory modification count call type.
  */
  public static final int CALLTYPE_DIRGETMODCOUNT = Kxc.CALLTYPE_DIRGETMODCOUNT;
  
  /**
  * Export data call type.
  */
  public static final int CALLTYPE_XPORTDATA = Kxc.CALLTYPE_XPORTDATA;
  
  // Directory Access
  
  /**
  * Search type.  
  */
  public static final int EXACTMATCH = Kxc.SEARCH_EXACTMATCH;
  
  /**
  * Search type. 
  */
  public static final int ONELEVEL = Kxc.SEARCH_ONELEVEL;
  
  /**
  * Search type.
  */
  public static final int SUBTREE = Kxc.SEARCH_SUBTREE;
  
  /**
  * Search option. Uses the starting point parameter. 
  */
  public static final int USE_NO_BASE = Kxc.SEARCH_OPTION_NONE;
  
  /**
  * Search option. Searches from top of tree.
  */
  public static final int USE_ROMA_BASE = Kxc.SEARCH_OPTION_BASE;
  
  /**
  * Store option. Will fail if the item does not exist.
  */
  public static final int MUST_EXIST = Kxc.STORE_MUST_EXIST;
  
  /**
  * Store option. Will fail if the item exists.
  */
  public static final int MUST_NOT_EXIST  = Kxc.STORE_MUST_NOT_EXIST;
  
  /**
  * Store option. Will succeed whether or not the item exists.
  */
  public static final int FORCE = Kxc.STORE_FORCE;
  
  // In Line Service data type option values
  public static final int  ILS_TYPE_XPORT    = Kxc.ILS_TYPE_XPORT; 
  public static final int  ILS_TYPE_USER     = Kxc.ILS_TYPE_USER;
  public static final int  ILS_TYPE_ROUTER   = Kxc.ILS_TYPE_ROUTER;
  public static final int  ILS_TYPE_SYSMGMT  = Kxc.ILS_TYPE_SYSMGMT;
  public static final int  ILS_TYPE_ERROR    = Kxc.ILS_TYPE_ERROR;
  public static final int  ILS_TYPE_WEB      = Kxc.ILS_TYPE_WEB;
  public static final int  ILS_TYPE_XPTDATA  = Kxc.ILS_TYPE_XPTDATA;
  
  // Other constants
  public static final int GOWAIT_INDEFINITE                 = -1;
  public static final int GOWAIT_PROFILE                    = -2;
  
  public final static int CC_OK                             = 0;
  public final static int CC_WARNING                        = 1;
  public final static int CC_FAILED                         = 2;
  
  public final static int OP_KXC                            = 0;
  public final static int OP_MQSERIES                       = 1;
  public final static int OP_MSMQ                           = 2;
  public final static int OP_RPC                            = 3;
  public final static int OP_OS                             = 4;
  public final static int OP_LIB                            = 5;
  public final static int OP_LDAP                           = 6;
  public final static int OP_LDIF                           = 7;
  public final static int OP_CICS                           = 8;
  public final static int OP_USER                           = 100;
  
  public final static int RC_OK                             = 0;
  
  public final static int RC_INVALID_CONTEXT                = 1;
  public final static int RC_INVALID_OPTIONS                = 2;
  public final static int RC_INVALID_ROUTE                  = 3;
  public final static int RC_INVALID_THEME                  = 4;
  public final static int RC_INVALID_MESSAGE                = 5;
  public final static int RC_INVALID_RESPONSE               = 6;
  public final static int RC_INVALID_PARAMETER              = 7;
  public final static int RC_INVALID_APPDEF                 = 8;
  public final static int RC_INVALID_SERVICE                = 9;
  public final static int RC_INVALID_HANDLE                 = 10;
  public final static int RC_CONTEXT_MISMATCH               = 11;
  public final static int RC_NONXACTIONAL                   = 12;
  public final static int RC_NON_LOCALQ                     = 13;
  public final static int RC_INVALID_SYNCTYPE               = 14;
  public final static int RC_INVALID_SECURITY               = 15;
  public final static int RC_CALL_IN_PROGRESS               = 16;
  
  public final static int RC_NO_MSG_AVAILABLE               = 20;
  public final static int RC_BUFFER_TOO_SMALL               = 21;
  public final static int RC_BUFFER_OVERFLOW                = 22;
  public final static int RC_NOT_SUPPORTED                  = 23;
  public final static int RC_VALIDATION_FAILED              = 24;
  public final static int RC_INVALID_TRAILER                = 25;
  public final static int RC_MESSAGE_SUPPRESSED             = 26;
  public final static int RC_BACKED_OUT                     = 27;
  public final static int RC_ACCESS_DENIED                  = 28;
  public final static int RC_BUFFER_ERROR                   = 29;
  public final static int RC_Q_DELETED                      = 30;
  public final static int RC_Q_FULL                         = 31;
  public final static int RC_MSG_UNAVAILABLE                = 32;
  public final static int RC_SIGNAL_ACCEPTED                = 33;
  public final static int RC_SIGNAL_INVALID                 = 34;
  public final static int RC_SIGNAL_OUTSTANDING             = 35;
  
  public final static int RC_ROUTING_FAILURE                = 40;
  public final static int RC_STORAGE_FAILURE                = 41;
  public final static int RC_TRANSPORT_FAILURE              = 42;
  public final static int RC_EXIT_FAILURE                   = 43;
  public final static int RC_EXCEPTION_CAUGHT               = 44;
  public final static int RC_NOT_ACCESSIBLE                 = 45;
  public final static int RC_RESOURCE_PROBLEM               = 46;
  public final static int RC_TRANSPORT_UNAVAILABLE          = 47;
  public final static int RC_MUTEX_FAILURE                  = 48;
  public final static int RC_INVALID_MSGTYPE                = 49;
  public final static int RC_CACHE_FAILURE                  = 50;
  public final static int RC_HANDLE_BROKEN                  = 51;
  public final static int RC_SYSCONFIG_FAILURE              = 52;
  
  public final static int RC_UNKNOWN_SERVICE                = 60;
  public final static int RC_UNKNOWN_MSGFORMAT              = 61;
  public final static int RC_UNKNOWN_CLIENT                 = 62;
  public final static int RC_UNKNOWN_SERVER                 = 63;
  public final static int RC_UNKNOWN_BRIDGE                 = 64;
  public final static int RC_INVALID_QNAME                  = 65;
  public final static int RC_CONFIG_ERROR                   = 66;
  public final static int RC_DIRECTORY_UNAVAILABLE          = 67;
  public final static int RC_BRIDGE_UNAVAILABLE             = 68;
  public final static int RC_INVALID_COMPONENT              = 69;
  public final static int RC_INVALID_ELEMENT                = 70;
  public final static int RC_INVALID_SERVERNAME             = 71;
  public final static int RC_INVALID_FILENAME               = 72;
  public final static int RC_WAIT_CANCELLED                 = 73;
  public final static int RC_UNKNOWN_ERRPROFILE             = 74;
  public final static int RC_UNKNOWN_PUTPROFILE             = 75;
  public final static int RC_UNKNOWN_GETPROFILE             = 76;
  public final static int RC_UNKNOWN_OBJECT                 = 77;
  public final static int RC_GET_ATTRIBUTE_FAILURE          = 78;
  public final static int RC_UNKNOWN_NODE                   = 79;
  
  
  public final static int RC_LDAP_GET_FAILURE               = 80;
  public final static int RC_LDAP_OPEN_FAILURE              = 81;
  public final static int RC_LDAP_BIND_FAILURE              = 82;
  public final static int RC_LDAP_SEARCH_FAILURE            = 83;
  public final static int RC_LDAP_FIRST_ENTRY_FAILURE       = 84;
  public final static int RC_LDAP_GET_DN_FAILURE            = 85;
  public final static int RC_LDAP_NEXT_ENTRY_FAILURE        = 86;
  public final static int RC_LDAP_COUNT_ENTRIES_FAILURE     = 87;
  public final static int RC_LDAP_MODIFY_FAILURE            = 88;
  public final static int RC_LDAP_DELETE_FAILURE            = 89;
  public final static int RC_LDAP_ADD_FAILURE               = 90;
  public final static int RC_LDAP_MSGFREE_FAILURE           = 91;
  public final static int RC_LDAP_UNBIND_FAILURE            = 92;
  public final static int RC_LDAP_VALUE_FREE_FAILURE        = 93;
  public final static int RC_LDAP_GET_VALUES_FAILURE        = 94;
  public final static int RC_LDAP_CONNECTION_LOST           = 95;
  public final static int RC_LDAP_INVALID_GATEWAY           = 96;
  public final static int RC_LDAP_MODRDN_FAILURE            = 97;
  public final static int RC_LDAP_IMPORT_FAILURE            = 98;
  public final static int RC_LDAP_EXPORT_FAILURE            = 99;
  public final static int RC_LDAP_MODCOUNT_FAILURE          = 100;
  
  public final static int RC_LDIF_GET_FAILURE               = 120;
  public final static int RC_LDIF_OPEN_FAILURE              = 121;
  public final static int RC_LDIF_SEARCH_FAILURE            = 123;
  public final static int RC_LDIF_GET_DN_FAILURE            = 124;
  public final static int RC_LDIF_NEXT_ENTRY_FAILURE        = 125;
  public final static int RC_LDIF_COUNT_ENTRIES_FAILURE     = 126;
  public final static int RC_LDIF_MODIFY_FAILURE            = 127;
  public final static int RC_LDIF_DELETE_FAILURE            = 128;
  public final static int RC_LDIF_ADD_FAILURE               = 129;
  public final static int RC_LDIF_GET_VALUES_FAILURE        = 130;
  public final static int RC_LDIF_INVALID_GATEWAY           = 131;
  public final static int RC_LDIF_RENAME_FAILURE            = 132;
  public final static int RC_LDIF_INVALID_ATTR_FAILURE      = 133;
  
  public final static int RC_LDIF_INVALID_PS_OPTIONS        = 140;
  public final static int RC_LDIF_INVALID_SUBJECT           = 141;
  public final static int RC_LDIF_THREAD_INIT_FAILURE       = 142;
  public final static int RC_LDIF_PERSISTENCE_NOT_SUPPORTED = 143;
  public final static int RC_LDIF_MSGID_NOT_SUPPORTED       = 144;
  public final static int RC_LDIF_CORRELID_NOT_SUPPORTED    = 145;
  
  public final static int RC_CICS_GETMAIN_FAILURE           = 1;
  public final static int RC_CICS_COMMIT_ROLLBACK           = 2;
  public final static int RC_CICS_NO_TCTLS                  = 3;
  public final static int RC_CICS_QMGR_QUIESCE              = 4;
  public final static int RC_CICS_CONN_QUIESCE              = 5;
  public final static int RC_CICS_WAITFAIL                  = 6;
  public final static int RC_CICS_COMMAND_TIMEOUT           = 7;
  public final static int RC_CICS_INVALID_POSTCODE          = 8;
  public final static int RC_CICS_EXEC_WAIT_ERROR           = 9;
  public final static int RC_CICS_GETSIG_FAIL               = 10;
  public final static int RC_CICS_CALL_ABANDONED            = 11;
  public final static int RC_CICS_KXCCTPI_UNAVAILABLE       = 12;
  public final static int RC_CICS_KXCCCDIR_UNAVAILABLE      = 13;
  public final static int RC_CICS_TRUE_INACTIVE             = 14;
  public final static int RC_CMQI_CALL_ABANDONED            = 5000;
  
  // Return code for errors in java api
  public final static int RC_MARSHALLING_ERROR              = 10000;
  
  // Error types
  public final static int ETYPE_CRITICAL                    = 1;
  public final static int ETYPE_ERROR                       = 2;
  public final static int ETYPE_TRANSIENT                   = 4;
  public final static int ETYPE_API                         = 8;
  
  // Constants error format RomaException.errorText 
  public static final int ERROR_TEXT_COMPACT                = 0;
  public static final int ERROR_TEXT_FULL                   = 1;
  public final static int PRIMARY_INFO                      = 1;
  public final static int SECONDARY_INFO                    = 2;

  // Confirmation flags
  public final static int CONFIRM_DELIVERY         = Kxc.POPT_COD;
  public final static int CONFIRM_ARRIVAL          = Kxc.POPT_COA;

  // Application types
  public final static int APPTYPE_SERVER           = 1;
  public final static int APPTYPE_BRIDGE           = 2;
  public final static int APPTYPE_CLIENT           = 3;
  public final static int APPTYPE_NATIVE_SERVER    = 4;
  public final static int APPTYPE_NATIVE_CLIENT    = 5;
  public final static int APPTYPE_UNKNOWN          = 6;

  // Environment types
  public final static int ENVTYPE_INTEL     = 0x01000000;
  public final static int ENVTYPE_UNIX      = 0x02000000;
  public final static int ENVTYPE_MVS       = 0x04000000;
  public final static int ENVTYPE_OS400     = 0x08000000;
  public final static int ENVTYPE_VMS       = 0x10000000;

  // Environment OSs
  public final static int ENVOS_AIX         = (0x00010000 | ENVTYPE_UNIX);
  public final static int ENVOS_HPUX102     = (0x00080000 | ENVTYPE_UNIX);
  public final static int ENVOS_HPUX11      = (0x00020000 | ENVTYPE_UNIX);
  public final static int ENVOS_DUX         = (0x00100000 | ENVTYPE_UNIX);
  public final static int ENVOS_MVSBATCH    = (0x00020000 | ENVTYPE_MVS);
  public final static int ENVOS_MVSCICS     = (0x00010000 | ENVTYPE_MVS);
  public final static int ENVOS_SUN         = (0x00040000 | ENVTYPE_UNIX);
  public final static int ENVOS_WIN95       = (0x00020000 | ENVTYPE_INTEL);
  public final static int ENVOS_WINNT       = (0x00010000 | ENVTYPE_INTEL);
  public final static int ENVOS_OS400       = (0x00100000 | ENVTYPE_OS400);

  // Route step flags
  public final static int ROUTEFLAG_RESTRICTED    = 1;
  public final static int ROUTEFLAG_XML_ILS       = 2;
  public final static int ROUTEFLAG_NOTRANSFORM   = 4;

  // Build theme flags
  public final static int THEMEINFO_THEMEID        = 1;
  public final static int THEMEINFO_RETRYCOUNT     = 2;
  public final static int THEMEINFO_WORKFLOW       = 4;
  public final static int THEMEINFO_ETEINFO        = 8;
  public final static int THEMEINFO_TRAILERS       = 16;
  public final static int THEMEINFO_CONFIRMFLAGS   = 32;
  public final static int THEMEINFO_ALL            = (THEMEINFO_THEMEID |
                                                      THEMEINFO_RETRYCOUNT |
                                                      THEMEINFO_WORKFLOW |
                                                      THEMEINFO_ETEINFO |
                                                      THEMEINFO_TRAILERS |
                                                      THEMEINFO_CONFIRMFLAGS);

  // Interpret message flags
  public final static int INTMSG_THEME = 1;
}

