//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaBusinessFlow.java,v 1.1.2.1 2003/10/16 18:47:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/Attic/RomaBusinessFlow.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaBusinessFlow.java
//
// Defines      : class RomaBusinessFlow
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the business flow information. It is used with
* {@link RomaBusinessElement#setBusinessFlow(RomaBusinessFlow)}
* and {@link RomaClient#setBusinessFlow(RomaBusinessFlow)}
* methods.
*/
public class RomaBusinessFlow implements Cloneable
{
    /**
    * Constructs the object with the specified data.
    *
    * @param flow Business Flow name
    */
    public RomaBusinessFlow(String flow)
    {
        if (flow != null)
        {
            m_flow = flow;
        }
    }

    /**
    * Constructs the object with the specified data.
    *
    * @param flow Business Flow name
    * @param oper Operation name
    */
    public RomaBusinessFlow(String flow, String oper)
    {
        if (flow != null)
        {
            m_flow = flow;
        }

        if (oper != null)
        {
            m_oper = oper;
        }
    }

    /**
    * Returns the business flow name.
    */
    public String getFlowName()
    {
        return m_flow;
    }

    /**
    * Sets the business flow name.
    *
    * @param name Business Flow name
    */
    public void setFlowName(String name)
    {
        m_flow = (name == null ? "" : name);
    }

    /**
    * Returns the operation name.
    */
    public String getOperationName()
    {
        return m_oper;
    }

    /**
    * Sets the operation name.
    *
    * @param name Operation name
    */
    public void setOperationName(String name)
    {
        m_oper = (name == null ? "" : name);
    }

    /**
    * Returns a copy of the object.
    */
    public Object clone()
    {
        RomaBusinessFlow copy = null;

        try
        {
            copy = (RomaBusinessFlow)super.clone();
            copy.m_flow = new String(m_flow);
            copy.m_oper = new String(m_oper);
        }
        catch (CloneNotSupportedException e)
        {
        }

        return copy;
    }

    private String m_flow = "";
    private String m_oper = "";
}

