//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaExceptionInfo.java,v 1.1.2.2 2003/10/17 14:04:38 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/Attic/RomaExceptionInfo.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaExceptionInfo.java
//
// Defines      : class RomaExceptionInfo
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import java.util.Date;


/**
* This class defines the message exception information.
*
* @see RomaExcErrorData
*/
public final class RomaExceptionInfo
{
  /**
  * @return true if the timestamp is available.
  */
  public boolean hasTimestamp()
  {
     return (m_ts != null);
  }

  /**
  * @return true if the Roma configuration is available.
  */
  public boolean hasRomaConfig()
  {
     return (m_rc != null);
  }

  /**
  * @return true if the system configuration is available.
  */
  public boolean hasSystemConfig()
  {
     return (m_sc != null);
  }

  /**
  * @return true if the error data is available.
  */
  public boolean hasErrorData()
  {
     return (m_edata != null);
  }

  /**
  * @return true if the user data is available.
  */
  public boolean hasUserData()
  {
     return (m_udata != null);
  }

  /**
  * Returns the timestamp. This is time when the message was sent to the
  * exception message handler.
  */
  public Date getTimestamp()
  {
     return m_ts;
  }

  /**
  * Returns the Roma configuration of the reporting application.
  */
  public RomaConfig getRomaConfig()
  {
     return m_rc;
  }

  /**
  * Returns the configuration of the system where the reporting application
  * has been running.
  */
  public RomaSystemConfig getSystemConfig()
  {
     return m_sc;
  }

  /**
  * Returns the error data set by the reporting application.
  */
  public RomaExcErrorData getErrorData()
  {
     return m_edata;
  }

  /**
  * Returns the user data.
  */
  public byte[] getUserData()
  {
     return m_udata;
  }


  RomaExceptionInfo()
  {
  }

  RomaExceptionInfo(long sec, int msec, RomaConfig rc, RomaSystemConfig sc,
                    RomaExcErrorData edata, byte[] udata)
  {
    if (sec != 0 || msec != 0)
    {
        m_ts = new Date(sec * 1000 + msec);
    }

    m_rc = rc;
    m_sc = sc;
    m_edata = edata;
    m_udata = udata;
  }

  private byte[] m_udata = null;
  private RomaExcErrorData m_edata = null;
  private Date m_ts = null;
  private RomaConfig m_rc = null;
  private RomaSystemConfig m_sc = null;
}


