//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaILSData.java
//
// Defines      : class RomaILSData
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsalates Roma Inline Service Data 
*/

public class RomaILSData
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constructor
  *     
  */
  public RomaILSData( ) 
  {
    m_option        = Kxc.ILS_OPTION_NONE;
    m_ilsDataHandle = Kxc.NULL_ILS_DATA_HANDLE;
  }
  
  /**
  * Constructor taking an ils handle.
  * For received ILS data to be accessed by 
  * getBuffer and getNextBuffer.			 
  * Package visibility    
  */
  RomaILSData( long ilsHandle ) 
  {
    m_option        = Kxc.ILS_OPTION_NONE;
    m_ilsDataHandle = ilsHandle;
  }
  
  /**
  * Set option to replace.
  * 
  */
  public void setReplace( boolean bReplace )
  {
    if ( bReplace) 
      m_option = Kxc.ILS_OPTION_REPLACE;
    else
      m_option = Kxc.ILS_OPTION_NONE;
  }
  
  /**
  * Set an ILS buffer. 
  * 
  */
  public void setBuffer( byte[] buffer,
                         int    ILSNumber,
                         int    type)
  {
    m_ilsDataHandle = KxcContext.setILSBuffer( m_ilsDataHandle,
                                               buffer,  
                                               ILSNumber,
                                               type,
                                               m_option );			
  }
  
  /**
  * Get specified in-line service buffer using the 
  * default type.
  *
  * @param ilsId  inline service identity
  */
  public byte[] getBuffer(int ILSNumber)
  {
    return KxcContext.getILSBuffer( m_ilsDataHandle,
                                    ILSNumber,
                                    Roma.ILS_TYPE_USER );			
  }		
  
  /**
  * Get specified in-line service buffer
  *
  * @param ilsId  inline service identity
  * @param type   type of service 
  */
  public byte[] getBuffer(int ILSNumber,
                          int type)
  {
    return KxcContext.getILSBuffer( m_ilsDataHandle,
                                    ILSNumber,
                                    type );			
  }		
  
  /**
  * Get next in-line service buffer
  *
  * @param ilsId returned inline service id
  * @param type  returned inline service type
  * @return the buffer   
  */
  public byte[] getNextBuffer(RomaInteger ILSNumber,
                              RomaInteger type)
  {
    return KxcContext.getILSBuffer( m_ilsDataHandle,
                                    ILSNumber,
                                    type );	
  }
  
  /**
  * Get ILS data handle. 
  * 
  */
  long getILSDataHandle( )
  {
    return m_ilsDataHandle;
  }	  
  
  private long m_ilsDataHandle;
  
  private int m_option;
}
