//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaRoute.java,v 1.1.2.3 2003/10/16 18:47:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/Attic/RomaRoute.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaRoute.java
//
// Defines      : class RomaRoute
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;


/**
* This class defines the workflow route step.
*
* @see RomaWorkflow
*/
public class RomaRoute implements Cloneable
{
  /**
  * Constructs the empty object.
  */
  public RomaRoute()
  {
  }

  /**
  * Returns the route type. This is the type of the target element
  * in the route step.
  *
  * @return one of
  * <br><ul>
  * <li><code>Roma.APPTYPE_SERVER</code> - the element is a server</li>
  * <li><code>Roma.APPTYPE_CLIENT</code> - the element is a client</li>
  * </ul>
  */
  public int getType()
  {
     return m_type;
  }

  /**
  * Sets the route type.
  *
  * @param type Target element type, one of
  * <br><ul>
  * <li><code>Roma.APPTYPE_SERVER</code> - the element is a server</li>
  * <li><code>Roma.APPTYPE_CLIENT</code> - the element is a client</li>
  * </ul>
  */
  public void setType(int type)
  {
      m_type = type;
  }

  /**
  * Returns the route flags.
  *
  * @return zero or a combination of
  * <br><ul>
  * <li><code>Roma.ROUTEFLAG_RESTRICTED</code> - the target element supports only a subset of message formats</li>
  * <li><code>Roma.ROUTEFLAG_XML_ILS</code> - the XML inline service is enabled</li>
  * <li><code>Roma.ROUTEFLAG_NOTRANSFORM</code> - message transformation is prohibited</li>
  * </ul>
  */
  public int getFlags()
  {
     return m_flags;
  }

  /**
  * Sets the route flags.
  *
  * @param flags Route flags. Should be zero or a combination of
  * <br><ul>
  * <li><code>Roma.ROUTEFLAG_RESTRICTED</code> - the target element supports only a subset of message formats</li>
  * <li><code>Roma.ROUTEFLAG_XML_ILS</code> - the XML inline service is enabled</li>
  * <li><code>Roma.ROUTEFLAG_NOTRANSFORM</code> - message transformation is prohibited</li>
  * </ul>
  */
  public void setFlags(int flags)
  {
      m_flags = flags;
  }

  /**
  * Returns the target queue name.
  * The format of the queue name is:<br><code>
  * &lt;QueueNameLo&gt;@&lt;QueueNameHi&gt@&lt;SubNet&gt@&lt;Transport&gt
  * </code><br>For example:<br><code>KXCSREPLY@FFLIN@CamMq@MQSeries</code>
  */
  public String getTargetQueue()
  {
     return m_queue;
  }

  /**
  * Sets the target queue name.
  *
  * @param queue Queue name
  */
  public void setTargetQueue(String queue)
  {
     m_queue = queue;
  }

  /**
  * Returns the target business element name. The target element can be a
  * client or server.
  */
  public String getTargetElement()
  {
     return m_be;
  }

  /**
  * Sets the target business element name. The target element can be a
  * client or server.
  *
  * @param elem Target element name
  */
  public void setTargetElement(String elem)
  {
     m_be = elem;
  }

  /**
  * Returns the target business component name.
  * For a client, this is <code>$$client</code>.
  */
  public String getTargetComponent()
  {
     return m_bc;
  }

  /**
  * Sets the target business component name.
  *
  * @param comp Target component name
  */
  public void setTargetComponent(String comp)
  {
     m_bc = comp;
  }

  /**
  * Returns the target code page.
  */
  public String getTargetCodePage()
  {
     return m_cp;
  }

  /**
  * Sets the target code page.
  *
  * @param cp Code page name
  */
  public void setTargetCodePage(String cp)
  {
     m_cp = cp;
  }

  /**
  * Returns the name of the business service that has created or updated
  * the route step.
  * This is the business service specified on a message put call
  * unless an application has changed it using
  * {@link RomaBusinessElement#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * or {@link RomaClient#buildTheme(RomaTheme,RomaThemeInfo,int,boolean)}
  * somewhere on the route,
  * in which case this is the business service that has changed the workflow.
  */
  public String getSourceService()
  {
     return m_bs;
  }

  /**
  * Sets the name of the business service created the route step.
  *
  * @param bs Business Service name
  */
  public void setSourceService(String bs)
  {
     m_bs = bs;
  }

  /**
  * Returns the name of the business flow that has updated the route step.
  */
  public String getSourceFlow()
  {
     return m_bf;
  }

  /**
  * Sets the name of the business flow that has updated the route step.
  *
  * @param bf Business Flow name
  */
  public void setSourceFlow(String bf)
  {
     m_bf = bf;
  }

  /**
  * Returns the name of the operation that has updated the route step.
  */
  public String getSourceOperation()
  {
     return m_op;
  }

  /**
  * Sets the name of the operation that has updated the route step.
  *
  * @param op Operation name
  */
  public void setSourceOperation(String op)
  {
     m_op = op;
  }

  /**
  * Returns a copy of the object.
  */
  public Object clone()
  {
     RomaRoute copy = null;

     try
     {
        copy = (RomaRoute)super.clone();
        copy.m_op = new String(m_op);
        copy.m_bf = new String(m_bf);
        copy.m_bs = new String(m_bs);
        copy.m_be = new String(m_be);
        copy.m_bc = new String(m_bc);
        copy.m_cp = new String(m_cp);
        copy.m_queue = new String(m_queue);
     }
     catch (CloneNotSupportedException e)
     {
     }

     return copy;
  }

  RomaRoute(int type, String q, String be, String bc, int flags,
                    String cp, String bs, String bf, String op)
  {
     m_op = op;
     m_bf = bf;
     m_bs = bs;
     m_be = be;
     m_bc = bc;
     m_cp = cp;
     m_queue = q;
     m_type = type;
     m_flags = flags;
  }

  private String m_op = "";
  private String m_bf = "";
  private String m_bs = "";
  private String m_be = "";
  private String m_bc = "";
  private String m_cp = "";
  private String m_queue = "";
  private int m_type = 0;
  private int m_flags = 0;
}

