//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaSubject.java
//
// Defines      : class RomaSubject
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class is used by applications to define the topic(s) for
* publish and subscribe operations.
* The format of a subject depends on the underlying transport conventions
*/
public final class RomaSubject
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Initialize a new Subject, specifying subject name
  */
  public RomaSubject(String subject)
  {
    setSubject( subject );
  }
  
  /**
  * Initialize a new Subject, specifying subject and style
  */
  public RomaSubject(String subject, int style)
  {
    setSubject( subject );
    setSubjectStyle( style );
  }
  
  /**
  * Set the subject style
  */
  public void setSubjectStyle( int style )
  {
    m_style = style;
  }
  
  /**
  * Set the subject string
  */
  public void setSubject( String subject )
  {
    m_subject = subject;
  }
  
  /**
  * Get the subject
  */
  String subject()
  {
    return m_subject;
  }
  
  /**
  * Get the subject style
  */
  int style()
  {
    return m_style;
  }
  
  private int    m_style   = Roma.STYLE_ROMA;
  private String m_subject;
}
