//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaThemeInfo.java,v 1.1.2.1 2003/10/16 18:47:26 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/Attic/RomaThemeInfo.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaThemeInfo.java
//
// Defines      : class RomaThemeInfo
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import java.util.*;


/**
* This class defines the theme details extracted with the
* {@link RomaBusinessElement#queryTheme(RomaTheme)}
* or {@link RomaClient#queryTheme(RomaTheme)}
* method.
*
* @see RomaWorkflow
* @see RomaTrailer
* @see RomaEteInfo
*/
public class RomaThemeInfo implements Cloneable
{
   /**
   * Constructs the empty object.
   */
   public RomaThemeInfo()
   {
        m_tid = new byte[24];
        m_wflow = new RomaWorkflow();
        m_trailers = new LinkedList();
   }

   /**
   * Sets the workflow.
   *
   * @param wf Workflow information
   */
   public void setWorkflow(RomaWorkflow wf)
   {
      m_wflow = wf;
   }

   /**
   * Returns the associated workflow.
   */
   public RomaWorkflow getWorkflow()
   {
      return m_wflow;
   }

   /**
   * Sets the retry count.
   * Note that applications should enable the retry counter specifying
   * true for the
   * {@link RomaBeResponse#setIncrementRetryCount(boolean)}
   * method on any subsequent put operation to prevent
   * Roma from setting the retry count to 0.
   *
   * @param count Retry count
   */
   public void setRetryCount(int count)
   {
      m_rc = count;
   }

   /**
   * Returns the retry count. This is the number of times the message
   * has been sent to the exception message queue since the last successful
   * put operation. For the first retrieval of a failed message,
   * the retry count will be 0. If the put operation fails a second time
   * and the message is again sent to the EMQ, the retry count will be 1.
   * This lets your EMQ application know how many times a put attempt has
   * failed, so it can choose to take different a action when the count
   * reaches some threshold. Once a client or server operation succeeds,
   * your EMQ application should turn the retry incrementation off so
   * the retry count can be reset to 0. This can be done specifying false
   * for the 
   * {@link RomaBeResponse#setIncrementRetryCount(boolean)}
   * method.
   */
   public int getRetryCount()
   {
      return m_rc;
   }

   /**
   * Sets the theme ID.
   *
   * @param id Theme ID
   */
   public void setThemeId(byte[] id)
   {
      m_tid = id;
   }

   /**
   * Returns the theme ID.
   */
   public byte[] getThemeId()
   {
      return m_tid;
   }

   /**
   * Sets the confirmation flags.
   *
   * @param flags Confirmation flags. Should be zero or a combination of
   * <br><ul>
   * <li><code>Roma.CONFIRM_DELIVERY</code> - confirmation of delivery</li>
   * <li><code>Roma.CONFIRM_ARRIVAL</code> - confirmation of arrival</li>
   * </ul>
   */
   public void setConfirmFlags(int flags)
   {
      m_cf = flags;
   }

   /**
   * Returns the confirmation flags.
   *
   * @return zero if the confirmation is disabled or a combination of
   * <br><ul>
   * <li><code>Roma.CONFIRM_DELIVERY</code> - confirmation of delivery</li>
   * <li><code>Roma.CONFIRM_ARRIVAL</code> - confirmation of arrival</li>
   * </ul>
   */
   public int getConfirmFlags()
   {
      return m_cf;
   }

   /**
   * Sets the persistent trailers overwriting whatever persistent
   * trailers already exist.
   *
   * @param col Collection of RomaTrailer objects
   */
   public void setTrailers(Collection col)
   {
      m_trailers = (col == null) ? null : new LinkedList(col);
   }

   /**
   * Returns the persistent trailers. The trailers are added to the message
   * by various ILSs involved in the message transformation and delivery.
   *
   * @return List of RomaTrailer objects
   */
   public LinkedList getTrailers()
   {
      return (m_trailers == null ? new LinkedList() : m_trailers);
   }

   /**
   * Sets the end-to-end and in-flight information.
   *
   * @param ete End-to-end and in-flight information.
   */
   public void setEteInfo(RomaEteInfo ete)
   {
      m_ete = ete;
   }

   /**
   * Returns the end-to-end and in-flight information.
   */
   public RomaEteInfo getEteInfo()
   {
      return m_ete;
   }

   /**
   * Returns a copy of the object.
   */
   public Object clone()
   {
      RomaThemeInfo copy = null;
      
      try
      {
        copy = (RomaThemeInfo)super.clone();

        if (m_wflow != null)
        {
          copy.m_wflow = (RomaWorkflow)m_wflow.clone();
        }

        if (m_trailers != null && !m_trailers.isEmpty())
        {
            copy.m_trailers = new LinkedList();
            ListIterator iter = m_trailers.listIterator(0);

            while (iter.hasNext())
            {
                RomaTrailer tr = (RomaTrailer)iter.next();
                copy.m_trailers.add(tr.clone());
            }
        }
      }
      catch (CloneNotSupportedException e)
      {
      }

      return copy;
   }


   RomaThemeInfo(byte[] id, int rc, RomaWorkflow wf, int cf,
                 LinkedList trs, byte[] ete)
   {
      m_tid = id;
      m_rc = rc;
      m_cf = cf;
      m_wflow = wf;
      m_trailers = trs;
      m_ete = new RomaEteInfo(ete);
   }

   Object[] getTrailersArray()
   {
        if (m_trailers == null || m_trailers.isEmpty())
        {
            return null;
        }

        return m_trailers.toArray();
   }

   byte[] getEteData()
   {
       return (m_ete == null ? null : m_ete.getData());
   }

   private byte[] m_tid = null;
   private RomaWorkflow m_wflow = null;
   private int m_rc = 0;
   private int m_cf = 0;
   private LinkedList m_trailers = null;
   private RomaEteInfo m_ete = null;
}

