//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaWorkflow.java,v 1.1.2.3 2003/10/16 18:47:26 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/Attic/RomaWorkflow.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaWorkflow.java
//
// Defines      : class RomaWorkflow
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import java.util.*;


/**
* This class defines the workflow.
*
* @see RomaWorkflowService
* @see RomaRoute
*/
public class RomaWorkflow implements Cloneable
{
  /**
  * Constructs the empty object.
  */
  public RomaWorkflow()
  {
    m_wserv = new RomaWorkflowService();
    m_routes = new LinkedList();
  }

  /**
  * Returns the associated workflow service.
  */
  public RomaWorkflowService getWorkflowService()
  {
     return m_wserv;
  }

  /**
  * Associates the workflow service with the workflow.
  *
  * @param ws Workflow Service information
  */
  public void setWorkflowService(RomaWorkflowService ws)
  {
     m_wserv = ws;
  }

  /**
  * Returns the route steps of the workflow.
  *
  * @return List of RomaRoute objects
  */
  public LinkedList getRoutes()
  {
     return m_routes;
  }

  /**
  * Sets the route steps of the workflow.
  *
  * @param col Collection of RomaRoute objects
  */
  public void setRoutes(Collection col)
  {
     m_routes = (col == null) ? new LinkedList() : new LinkedList(col);
  }

  /**
  * Returns a copy of the object.
  */
  public Object clone()
  {
     RomaWorkflow copy = null;
     
     try
     {
        copy = (RomaWorkflow)super.clone();

        if (m_wserv != null)
        {
            copy.m_wserv = (RomaWorkflowService)m_wserv.clone();
        }

        if (m_routes != null && !m_routes.isEmpty())
        {
            copy.m_routes = new LinkedList();
            ListIterator iter = m_routes.listIterator(0);

            while (iter.hasNext())
            {
                RomaRoute rr = (RomaRoute)iter.next();
                copy.m_routes.add(rr.clone());
            }
        }
     }
     catch (CloneNotSupportedException e)
     {
     }

     return copy;
  }


  RomaWorkflow(RomaWorkflowService ws, LinkedList rl)
  {
     m_wserv = ws;
     m_routes = rl;
  }

  Object[] getRoutesArray()
  {
    if (m_routes == null || m_routes.isEmpty())
    {
        return null;
    }

    return m_routes.toArray();
  }


  private RomaWorkflowService m_wserv = null;
  private LinkedList m_routes = null;
}

