//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : Kxc.java
//
// Defines      : interface Kxc
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Gather together various Roma C API constants used in the raw
* interface. The values for these constants are lifted directly
* from the Roma C layer. 
*/
public interface Kxc
{
  //------------------------------------------------------------------
  // Signifies that default value to be used in a given field.
  // cf. KXCMSG_USEDEFAULT_VALUE in kxchkxca.h
  //------------------------------------------------------------------
  public static final int USEDEFAULT = -2;
  
  //------------------------------------------------------------------
  // Constants to identify the message type in the KXCMSG block.
  // cf. KXCMSGTYPE values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int DATAGRAM              = 0;
  public static final int REQUEST               = 1;
  public static final int REPLY                 = 2;
  public static final int NOTIFY                = 4;
  public static final int MSGTYPE_POSITIVE      = 8;
  public static final int MSGTYPE_NEGATIVE      = 0;
  public static final int MSGTYPE_COA           = 16;
  public static final int MSGTYPE_COD           = 0;
  public static final int MSGTYPE_CONFIRMATION  = 32;
  public static final int MSGTYPE_MASK          = 63;
  
  public static final int PUBLICATION           = 64;
  public static final int POSITIVE_PUBSUB       = 128;
  public static final int NEGATIVE_PUBSUB       = 256;
  public static final int ADVISORY              = 512;
  public static final int MSGTYPE_POSITIVE_COA = 
    ( MSGTYPE_CONFIRMATION | MSGTYPE_POSITIVE | MSGTYPE_COA );
  public static final int MSGTYPE_NEGATIVE_COA = 
    ( MSGTYPE_CONFIRMATION | MSGTYPE_NEGATIVE | MSGTYPE_COA );
  public static final int MSGTYPE_POSITIVE_COD =
    ( MSGTYPE_CONFIRMATION | MSGTYPE_POSITIVE | MSGTYPE_COD );
  public static final int MSGTYPE_NEGATIVE_COD = 
    ( MSGTYPE_CONFIRMATION | MSGTYPE_NEGATIVE | MSGTYPE_COD );
  
  //------------------------------------------------------------------
  // Constants for message persistence value in KXCMSG block.
  // cf. KXCMSGPERSIST values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGPERSIST_YES        = 1;
  public static final int MSGPERSIST_NO         = 0;
  public static final int MSGPERSIST_USEDEFAULT = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants for message format value in KXCMSG block.
  // cf. KXCMSGFORMAT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGFORMAT_BINARY      = 1;
  public static final int MSGFORMAT_STRING      = 2;
  public static final int MSGFORMAT_BINARY_SIGN = 3;
  public static final int MSGFORMAT_STRING_SIGN = 4;
  public static final int MSGFORMAT_BINARY_ENC  = 5;
  public static final int MSGFORMAT_STRING_ENC  = 6;
  public static final int MSGFORMAT_BINARY_SIGN_ENC = 7;
  public static final int MSGFORMAT_STRING_SIGN_ENC = 8;
  public static final int MSGFORMAT_USER        = 65536;
  public static final int MSGFORMAT_USENAME     = -1;
  public static final int MSGFORMAT_USEDEFAULT  = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants for message feedback value in KXCMSG block.
  // cf. KXCMSGFBCK values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGFBCK_NONE          = 0;
  public static final int MSGFBCK_ACK           = 1;
  public static final int MSGFBCK_NACK          = 64;
  public static final int MSGFBCK_BAD_DEST      = 65;
  public static final int MSGFBCK_Q_FULL        = 66;
  public static final int MSGFBCK_EXPIRED       = 67;
  public static final int MSGFBCK_ALLOC_ERR     = 68;
  public static final int MSGFBCK_USEDEFAULT    = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants for message expiry value in KXCMSG block.
  // cf. KXCMSGEXP values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGEXP_NEVER          = -1;
  public static final int MSGEXP_USEDEFAULT     = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants for message codepage value in KXCMSG block.
  // cf. KXCMSGCODEPAGE values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGCODEPAGE_USER        = -1;
  public static final int MSGCODEPAGE_USEDEFAULT  = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants for message encoding value in KXCMSG block.
  // cf. KXCMSGENC values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGENC_NORMAL       = 0;
  public static final int MSGENC_REVERSED     = 1;
  public static final int MSGENC_DEFAULT      = MSGENC_NORMAL;
  public static final int MSGENC_USEDEFAULT   = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants for message status value in KXCMSG block.
  // cf. KXCMSGSTAT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int MSGSTAT_NONE          = 0;
  public static final int MSGSTAT_ANONYMOUS     = MSGSTAT_NONE;
  public static final int MSGSTAT_AUTHENTICATED = 1;
  public static final int MSGSTAT_NATIVE        = 2;
  public static final int MSGSTAT_OK_TO_RESPOND = 4;
  public static final int MSGSTAT_OK_TO_NOTIFY  = 8;
  
  //------------------------------------------------------------------
  // Constants for max message and tag buffer lengths in KXCMSG block.
  //------------------------------------------------------------------
  public static final int MSGMAX_LEN    = 0x3C0000;
  public static final int MSGMAX_TAGLEN = 0xFFFF;
  
  //------------------------------------------------------------------
  // Constants that identify the error status in the KXCERROR block.
  // cf KXCERRSTAT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int ERRSTAT_NONE  = 0;
  public static final int ERRSTAT_MAIN  = 1;
  public static final int ERRSTAT_SUB   = 2;
  
  //------------------------------------------------------------------
  // Constants used to hold the buffer status in the KXCBUFFER block.
  // cf. KXCBUFFSTAT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int BUFSTAT_USER  = 0;
  public static final int BUFSTAT_KXC   = 1;
  
  //------------------------------------------------------------------
  // Constants used to hold the receive options in the KXCGETOPT 
  // block.
  // cf. KXCGOPT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int GOPT_NONE           = 0x00000000;
  public static final int GOPT_USEMSGID       = 0x00000001;
  public static final int GOPT_USECORRELID    = 0x00000002;
  public static final int GOPT_WAIT           = 0x00000008;
  public static final int GOPT_USETHEME       = 0x00000010;
  public static final int GOPT_ANYTHEME       = 0x00000020;
  public static final int GOPT_RETAINTHEME    = 0x00000040;
  public static final int GOPT_CLOSETHEME     = 0x00000080;
  public static final int GOPT_AUTOALLOC      = 0x00000100;
  public static final int GOPT_PEEK_FIRST     = 0x00000200;
  public static final int GOPT_PEEK_NEXT      = 0x00000400;
  public static final int GOPT_PEEK_SAME      = 0x00000800;
  public static final int GOPT_GET_PEEKED     = 0x00001000;
  public static final int GOPT_ALLOWNATIVE    = 0x00002000;
  public static final int GOPT_USESIGNAL      = 0x00004000;
  public static final int GOPT_NOEMS          = 0x00008000;
  public static final int GOPT_NONOTIFY       = 0x00010000;
  public static final int GOPT_ALLOWTERMINATE = 0x00020000;
  public static final int GOPT_SUPP_ERROR     = 0x00040000;
  public static final int GOPT_USEDEFAULT     = 0x40000000;
  
  
  // KXCGETOPT Status field values
  public static final int GOPT_STAT_NONE      = 0x00000000;
  public static final int GOPT_STAT_TERMINATE = 0x00000001;
  
  
  //------------------------------------------------------------------
  // Constants used to hold the wait length value in the KXCGETOPT 
  // block.
  //------------------------------------------------------------------
  public static final int GOWAIT_INDEFINITE = -1;
  public static final int GOWAIT_USEDEFAULT = -2;
  
  //------------------------------------------------------------------
  // Constants used to hold the request options in the KXCPUTOPT
  // block.
  // cf. KXCPOPT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int POPT_NONE           = 0x00000000;
  public static final int POPT_USEMSGID       = 0x00000001;
  public static final int POPT_USECORRELID    = 0x00000002;
  public static final int POPT_USETHEME       = 0x00000010;
  public static final int POPT_NEWTHEME       = 0x00000020;
  public static final int POPT_RETAINTHEME    = 0x00000040;
  public static final int POPT_CLOSETHEME     = 0x00000080;
  public static final int POPT_FREEBUFFER     = 0x00000100;
  public static final int POPT_DATAGRAM       = 0x00000200;
  public static final int POPT_ALLOWNOSYNC    = 0x00000400;
  public static final int POPT_NOTIFY         = 0x00000800;
  public static final int POPT_EXCTIMESTAMP   = 0x00001000;
  public static final int POPT_EXCROMADATA    = 0x00002000;
  public static final int POPT_EXCSYSDATA     = 0x00004000;
  public static final int POPT_EXCERRDATA     = 0x00008000;
  public static final int POPT_EXCUSERDATA    = 0x00010000;
  public static final int POPT_INCRETRYCOUNT  = 0x00020000;
  public static final int POPT_NOEMS          = 0x00040000;
  public static final int POPT_NONOTIFY       = 0x00080000;
  public static final int POPT_NOXFORM        = 0x00100000;
  public static final int POPT_ALLOWTERMINATE = 0x00200000;
  public static final int POPT_COA            = 0x00400000;
  public static final int POPT_COD            = 0x00800000;
  public static final int POPT_NOERRORPROF    = 0x01000000;
  
  public static final int POPT_USEDEFAULT     = 0x40000000;
  
  // KXCPUTOPT Status field values
  public static final int POPT_STAT_NONE      = 0x00000000;
  public static final int POPT_STAT_TERMINATE = 0x00000001;
  
  //------------------------------------------------------------------
  // Constants used to hold the transaction type value in the 
  // options in the KXCSYNC block
  // cf. KXCSYNC values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int SYNC_NONE       = 0;
  public static final int SYNC_LOCAL      = 1;
  public static final int SYNC_GLOBAL     = 2;
  public static final int SYNC_USEDEFAULT = USEDEFAULT;
  
  //------------------------------------------------------------------
  // Constants used to hold the options value in the KXCAPPDEF block
  // cf. KXCAPPOPT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int APPOPT_NONE             = 0;
  public static final int APPOPT_EXCLUSIVE        = 1;
  public static final int APPOPT_NON_XACTIONAL    = 2;
  public static final int APPOPT_MQSERIES_XACTION = 4;
  public static final int APPOPT_ROLLBK_OS_XACT   = 16;

  //------------------------------------------------------------------
  // Constants used to hold the exception flags in the KXCEXCEPTION
  // block.
  // cf. KXCEXC values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int EXC_NONE        = 0;
  public static final int EXC_TIMESTAMP   = 1;
  public static final int EXC_ROMADATA    = 2;
  public static final int EXC_SYSTEMDATA  = 4;
  public static final int EXC_ERRORDATA   = 8;
  public static final int EXC_USERDATA    = 16;
  
  //------------------------------------------------------------------
  // Constants used to hold the error options in the KXCEPOPT block.
  // cf. KXCEPOPT values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int EPOPT_NONE        = 0;
  public static final int EPOPT_USEEPNAME   = 1;
  public static final int EPOPT_USEDEFAULT  = 2;
  public static final int EPOPT_NONOTIFY    = 4;
  public static final int EPOPT_NOEMS       = 8;
  public static final int EPOPT_NOALERT     = 16;
  public static final int EPOPT_NOXFORM     = 32;
  
  //------------------------------------------------------------------
  // Constants used to hold the actions value in the KXCEPOPT block.
  // cf. KXCEP_ACTION values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int EP_ACTION_DEFAULT   = 0;
  public static final int EP_ACTION_NOTIFY    = 1;
  public static final int EP_ACTION_EMS       = 2;
  public static final int EP_ACTION_ALERT     = 4;
  public static final int EP_ACTION_LOG       = 8;
  public static final int EP_ACTION_RESET     = 16;
  public static final int EP_ACTION_NOACTION  = 32;
  public static final int EP_ACTION_EMSRAW    = 64;
  
  //------------------------------------------------------------------
  // Constants used for the status value when interpreting a message.
  // cf. KXCMSGPART values in kxchutla.h
  //------------------------------------------------------------------
  public static final int MSGPART_MESSAGE   = 1;
  public static final int MSGPART_WORKFLOW  = 2;
  public static final int MSGPART_EXCEPTION = 4;
  public static final int MSGPART_HTHEME    = 8;
  
  //------------------------------------------------------------------
  // Constants used for the flag values when interpreting a message.
  // cf. KXCMOPT values in kxchutla.h
  //------------------------------------------------------------------
  public static final int MOPT_NORMAL = 0;
  public static final int MOPT_HTHEME = 1;
  
  //------------------------------------------------------------------
  // Constants used for the different versions of the KXCWFLOW blocks.
  // See kxchxita.h.
  //------------------------------------------------------------------
  public static final int WFLOW_VERSION_1 = 1;
  public static final int WFLOW_VERSION_2 = 2;
  public static final int WFLOW_VERSION_3 = 3;
  public static final int WFLOW_VERSION_4 = 4;
  
  //------------------------------------------------------------------
  // NULL constant for handles
  //------------------------------------------------------------------
  public static final int NULL_HANDLE = 0;
  
  //------------------------------------------------------------------
  // The size of various data types.
  //------------------------------------------------------------------
  public static final int OBJECT_NAME_LENGTH  = 132;
  public static final int SUBNET_NAME_LENGTH  = 20;
  public static final int QCOMP_NAME_LENGTH   = 48;
  public static final int MSG_ID_LENGTH       = 24;
  
  //------------------------------------------------------------------
  // Encodings for Binary Integers 
  //------------------------------------------------------------------
  public static final int KXCENC_INTEGER_UNDEFINED = USEDEFAULT;
  public static final int KXCENC_INTEGER_NORMAL    = MSGENC_NORMAL;
  public static final int KXCENC_INTEGER_REVERSED  = MSGENC_REVERSED;
  
  //------------------------------------------------------------------
  // Encodings for Packed-Decimal Integers 
  //------------------------------------------------------------------
  public static final int KXCENC_DECIMAL_UNDEFINED = USEDEFAULT;
  public static final int KXCENC_DECIMAL_NORMAL    = MSGENC_NORMAL;
  public static final int KXCENC_DECIMAL_REVERSED  = MSGENC_REVERSED;
  
  //------------------------------------------------------------------
  // Encodings for Floating-Point Numbers
  //------------------------------------------------------------------
  public static final int KXCENC_FLOAT_UNDEFINED     = USEDEFAULT;
  public static final int KXCENC_FLOAT_IEEE_NORMAL   = MSGENC_NORMAL;
  public static final int KXCENC_FLOAT_IEEE_REVERSED = MSGENC_REVERSED;
  
  //------------------------------------------------------------------
  // Constants for publish-subscribe sibject styles ...
  // cf. KXCSUBJECTSTYLE values in kxchkxca.h
  //------------------------------------------------------------------
  public static final int STYLE_NONE          = 0;
  public static final int STYLE_TIBRENDEZVOUS = 1;
  public static final int STYLE_MQSERIES      = 2;
  public static final int STYLE_ROMA          = 3;
  
  //------------------------------------------------------------------
  // Constants for directory access
  // cf. KXCDOPT values in kxc.h
  //------------------------------------------------------------------
  public static final int SEARCH_EXACTMATCH	  = 0;
  public static final int SEARCH_ONELEVEL     = 1;
  public static final int SEARCH_SUBTREE      = 2;
  
  public static final int SEARCH_OPTION_NONE  = 0;
  public static final int SEARCH_OPTION_BASE  = 1;
  
  //------------------------------------------------------------------
  // Constants for directory access
  // cf. KXCUPDOPT values in kxc.h
  //------------------------------------------------------------------
  public static final int STORE_FORCE           = 0;
  public static final int STORE_MUST_EXIST      = 1;
  public static final int STORE_MUST_NOT_EXIST  = 2;
  
  //------------------------------------------------------------------
  // Constants for in line service options
  // cf. KXCILSOPT values in kxc.h
  //------------------------------------------------------------------
  public static final long NULL_ILS_DATA_HANDLE = 0;
  
  public static final int ILS_OPTION_NONE       = 0;
  public static final int ILS_OPTION_REPLACE    = 1;
  public static final int ILS_OPTION_GETNEXT    = 2;
  
  //------------------------------------------------------------------
  // Constants for in line service data type 
  // cf. KXCXIT values in kxexit.h
  //------------------------------------------------------------------
  
  public static final int  ILS_TYPE_XPORT    = 1;
  public static final int  ILS_TYPE_USER     = 2;
  public static final int  ILS_TYPE_ROUTER   = 3;
  public static final int  ILS_TYPE_SYSMGMT  = 4;
  public static final int  ILS_TYPE_ERROR    = 5;
  public static final int  ILS_TYPE_XPTDATA  = 6;
  public static final int  ILS_TYPE_WEB      = 7;
  
  //------------------------------------------------------------------
  // Constants error format kxcFormatError 
  //------------------------------------------------------------------
  
  public static final int ERROR_TEXT_COMPACT  = 0;
  public static final int ERROR_TEXT_FULL     = 1;
  public final static int PRIMARY_INFO        = 1;
  public final static int SECONDARY_INFO      = 2;
  
  //------------------------------------------------------------------
  // Constants for the storage queue open options from kxc.h
  //------------------------------------------------------------------
  
  public final static int OPEN_OP_NONE            = 0;
  public final static int OPEN_OP_OUTPUT          = 1;
  public final static int OPEN_OP_INPUT           = 2;
  public final static int OPEN_OP_PEEK            = 4;
  public final static int OPEN_OP_NON_XACTIONAL   = 8;
  public final static int OPEN_OP_EXCLUSIVE       = 16;
  public final static int OPEN_OP_DELETE_ON_CLOSE = 32;
  public final static int OPEN_OP_MUST_EXIST      = 64;
  
  //------------------------------------------------------------------
  // Constants for the call types in kxc.h
  //------------------------------------------------------------------
  
  public final static int CALLTYPE_APPLICATION      = 0;
  public final static int CALLTYPE_PUTREQUEST       = 1;
  public final static int CALLTYPE_GETREQUEST       = 2;
  public final static int CALLTYPE_PUTRESPONSE      = 3;
  public final static int CALLTYPE_GETRESPONSE      = 4;
  public final static int CALLTYPE_BRIDGEPUT        = 5;
  public final static int CALLTYPE_BRIDGEGET        = 6;
  public final static int CALLTYPE_PUTEXCEPTION     = 7;
  public final static int CALLTYPE_INITIALIZE       = 8;
  public final static int CALLTYPE_TERMINATE        = 9;
  public final static int CALLTYPE_COMMIT           = 10; 
  public final static int CALLTYPE_ROLLBACK         = 11;
  public final static int CALLTYPE_FREEBUFFER       = 12;
  public final static int CALLTYPE_CLOSEHANDLE      = 13;
  public final static int CALLTYPE_INTERPRETMSG     = 14;
  public final static int CALLTYPE_WRITELOG         = 15;
  public final static int CALLTYPE_SENDALERT        = 16;
  public final static int CALLTYPE_OPENSTORAGEQ     = 17;
  public final static int CALLTYPE_CLOSESTORAGEQ    = 18;
  public final static int CALLTYPE_DELETESTORAGEQ   = 19;
  public final static int CALLTYPE_PUTTOSTORAGEQ    = 20;
  public final static int CALLTYPE_GETFROMSTORAGEQ  = 21;
  public final static int CALLTYPE_QUERYTHEME       = 22;
  public final static int CALLTYPE_BUILDTHEME       = 23;
  public final static int CALLTYPE_PURGEQTOEMS      = 24;
  public final static int CALLTYPE_REPORTERROR      = 25;
  public final static int CALLTYPE_QUERYCONTEXT     = 26;
  public final static int CALLTYPE_SETLDAPSERVICE   = 27;
  public final static int CALLTYPE_SETILSDATA       = 28;
  public final static int CALLTYPE_GETILSDATA       = 29;
  public final static int CALLTYPE_DIRLOOKUP        = 30;
  public final static int CALLTYPE_DIRGETDN         = 31;
  public final static int CALLTYPE_DIRUPDATEITEM    = 32;
  public final static int CALLTYPE_DIRDELETEITEM    = 33;
  public final static int CALLTYPE_DIRGETMODCOUNT   = 34;
  public final static int CALLTYPE_XPORTDATA        = 35;
}
