//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcException.java
//
// Defines      : class KxcException
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Abstracts KXCEXCPETION C API structure as object in the KxcOper parameters
* pool.
*/

public class KxcException extends KxcStruct
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  KxcException( KxcOper oper )
  {
    super( oper );
    m_userData  = new KxcBuffer( oper, EXC_USERDATA, EXC_USERDATA_LENGTH );
    m_romaConf  = new KxcExcRomaConf( oper );
    m_sysConf   = new KxcExcSysConf( oper );
    m_errData   = new KxcExcErrData( oper );
  }
  
  public int excFlags()
  {
    return m_ip[EXC_FLAGS];
  }

  public void setExcFlags( int flags )
  {
    m_ip[EXC_FLAGS] = flags;
  }
  
  public void setUserData( byte[] userData )
  {
    m_userData.setData( userData ); 
    m_userData.setDataLength( userData.length ); 
  }
  
  public long timeStamp()
  {
    long secs = m_ip[EXC_SECS];
    return secs*1000+m_ip[EXC_MSECS];
  }

  public KxcExcRomaConf romaConf()
  {
    return m_romaConf;
  }

  public KxcExcSysConf sysConf()
  {
    return m_sysConf;
  }
  
  public KxcExcErrData errData()
  {
    return m_errData;
  }
  
  public KxcBuffer userData()
  {
    return m_userData;
  }
  
  KxcExcRomaConf  m_romaConf;
  KxcExcSysConf   m_sysConf;
  KxcExcErrData   m_errData;
  KxcBuffer       m_userData;
}
