//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcMsg.java
//
// Defines      : class KxcMsg
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Abstracts KXCMSG C API structure as object in the KxcOper parameters
* pool.
*/

public class KxcMsg extends KxcStruct
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  KxcMsg( KxcOper oper )
  {
    super( oper );
    m_msgBuffer = new KxcBuffer( oper, MSGBUF, MSG_LENGTH );
    m_tagBuffer = new KxcBuffer( oper, TAGBUF, TAG_LENGTH );
  }
  
  public int type()
  {
    return m_ip[MSG_TYPE];
  }
  
  public void setType( int type )
  {
    m_ip[MSG_TYPE] = type;
  }
  
  public int priority()
  {
    return m_ip[PRIORITY];
  }
  
  public void setPriority( int prior )
  {
    m_ip[PRIORITY] = prior;
  }
  
  public int persistence()
  {
    return m_ip[PERSISTENCE];
  }
  
  public void setPersistence( int persist )
  {
    m_ip[PERSISTENCE] = persist;
  }
  
  public int format()
  {
    return m_ip[FORMAT];
  }
  
  public void setFormat( int fmt )
  {
    m_ip[FORMAT] = fmt;
  }
  
  public int feedback()
  {
    return m_ip[FEEDBACK];
  }
  
  public void setFeedback( int fback )
  {
    m_ip[FEEDBACK] = fback;
  }
  
  public int expiry()
  {
    return m_ip[EXPIRY];
  }
  
  public void setExpiry( int expiry )
  {
    m_ip[EXPIRY] = expiry;
  }
  
  public int codepage()
  {
    return m_ip[CODEPAGE];
  }
  
  public void setCodepage( int codepage )
  {
    m_ip[CODEPAGE] = codepage;
  }
  
  public int encoding()
  {
    return m_ip[ENCODING];
  }
  
  public void setEncoding( int encoding )
  {
    m_ip[ENCODING] = encoding;
  }
  
  public byte[] msgId()
  {
    return (byte[])m_op[MSGID];
  }
  
  public void setMsgId( byte b[] )
  {
    m_op[MSGID] = b;
  }
  
  public byte[] correlId()
  {
    return (byte[])m_op[CORRELID];
  }
  
  public void setCorrelId( byte b[] )
  {
    m_op[CORRELID] = b;
  }
  
  public KxcBuffer msgBuffer()
  {
    return m_msgBuffer;
  }
  
  public int msgLength()
  {
    return m_ip[MSG_LENGTH];
  }
  
  public void setMsgLength( int msgLength )
  {
    m_ip[MSG_LENGTH] = msgLength;
  }
  
  public KxcBuffer tagBuffer()
  {
    return m_tagBuffer;
  }
  
  public int tagLength()
  {
    return m_ip[TAG_LENGTH];
  }
  
  public void setTagLength( int tagLength )
  {
    m_ip[TAG_LENGTH] = tagLength;
  }
  
  public int status()
  {
    return m_ip[SEC_STATUS];
  }
  
  public void setStatus( int st )
  {
    m_ip[SEC_STATUS] = st;
  }
  
  public int backoutCount()
  {
    return m_ip[BACKOUT_COUNT];
  }
  
  public void setBackoutCount( int bCnt )
  {
    m_ip[BACKOUT_COUNT] = bCnt;
  }
  
  public String formatName()
  {
    return (String)m_op[FORMAT_NAME];
  }
  
  public String messageName()
  {
    return (String)m_op[MESSAGE_NAME];
  }
  
  public String subject()
  {
    return (String)m_op[SUBJECT]; 
  }
  
  public void setFormatName( String fmtName )
  {
    m_op[FORMAT_NAME] = fmtName;
  }
  
  public void setMessageName( String msgName )
  {
    m_op[MESSAGE_NAME] = msgName;
  }
  
  public long timeStamp()
  {
    long secs = m_ip[TIMESTAMP_SECS];
    int msecs = m_ip[TIMESTAMP_MSECS];
    return secs * 1000 + msecs;
  }
  
  public void setTimeStamp( long v )
  {
    int i = (int)(v / 1000);
    m_ip[TIMESTAMP_SECS] = i;
    i = (int)(v % 1000);
    m_ip[TIMESTAMP_MSECS] = i;
  }
  
  private KxcBuffer m_msgBuffer, m_tagBuffer;
}
