//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcMsgCtrl.java
//
// Defines      : class KxcMsgCtrl
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Abstracts common base between KXCGETOPT and KXCPUTOPT
* C API structures and maps it to KxcOper parameters pool.
*/

public class KxcMsgCtrl extends KxcStruct
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  KxcMsgCtrl( KxcOper oper )
  {
    super( oper );
  }
  
  public void setSecurity( byte sec[] )
  {
    m_op[SECBUF] = sec;
  }
  
  public void setSyncpoint( int trType, 
                            int trCtx )
  {
    m_ip[TR_TYPE] = trType;
    m_ip[TR_CTX]  = trCtx;
  }
  
  public long themeHandle()
  {
    return m_lp[HOBJ];
  }
  
  public void setThemeHandle( long hTheme )
  {
    m_lp[HOBJ] = hTheme;
  }
  
  public void setOpts( int opt )
  {
    m_ip[OPT] = opt;
  }
  
  public void maskOpts( int mask, 
                        int opt )
  {
    m_ip[OPT] &= ~mask;
    m_ip[OPT] |= (mask & opt);
  }
  
  public int opts()
  {
    return m_ip[OPT];
  }
  
  public void setMsgId( byte msgId[] )
  {
    m_op[MSGID] = msgId;
  }
  
  public void setCorrelId( byte correlId[] )
  {
    m_op[CORRELID] = correlId;
  }
  
  public void setApiProfileName( String s )
  {
    m_op[API_PROFILE] = s;
  }
  
  public void setErrorProfileName( String s )
  {
    m_op[ERROR_PROFILE] = s;
  }
  
  public void setILSDataHandle( long ilsHandle )
  {
    m_lp[ILS_DATA_HANDLE] = ilsHandle;
  }
  
  public long ilsDataHandle( )
  {
    return m_lp[ILS_DATA_HANDLE];
  }
  
  public void setPutGetStatus( int status )
  {
    m_ip[PUTGET_STATUS] = status;
  }
  
  public int putgetStatus( )
  {
    return m_ip[PUTGET_STATUS];
  }
}
