//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcNativeInterface.java
//
// Defines      : interface KxcNativeInterface
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Introduces various constants employed by the
* Roma raw native interface.
* This constants includes indexes in the parameter pool.
*/
interface KxcNativeInterface
{
  //------------------------------------------------------------------
  // Native operation codes
  //------------------------------------------------------------------
  static final int NIOP_CLNINIT             = 0;
  static final int NIOP_SVRINIT             = 1;
  static final int NIOP_TERMINATE           = 2;
  static final int NIOP_PUTREQ              = 3;
  static final int NIOP_GETREQ              = 4;
  static final int NIOP_PUTRESP             = 5;
  static final int NIOP_GETRESP             = 6;
  static final int NIOP_COMMIT              = 7;
  static final int NIOP_ROLLBACK            = 8;
  static final int NIOP_GET_LAST_ERROR      = 9;
  static final int NIOP_FREEBUF             = 10;
  static final int NIOP_CLOSE_HANDLE        = 11;
  static final int NIOP_FORMAT_ERROR        = 12;
  static final int NIOP_OPEN_STORAGEQ       = 13;
  static final int NIOP_CLOSE_STORAGEQ      = 14;
  static final int NIOP_GET_STORAGEQ        = 15;
  static final int NIOP_PUT_STORAGEQ        = 16;
  static final int NIOP_DELETEQ             = 17;
  static final int NIOP_INTERPRETMSG        = 18;
  static final int NIOP_PUTEXCEPTION        = 19;
  static final int NIOP_REPORTERROR         = 20;
  static final int NIOP_WRITELOG            = 21;
  static final int NIOP_SENDALERT           = 22;
  static final int NIOP_GETEXCEPTION        = 23;
  static final int NIOP_PUBLISH             = 24;
  static final int NIOP_SUBSCRIBE           = 25;
  static final int NIOP_UNSUBSCRIBE         = 26;
  static final int NIOP_DIRLOOKUP           = 27;
  static final int NIOP_DIRGETITEM          = 28;
  static final int NIOP_DIRGETDN            = 29;
  static final int NIOP_DIRCNTATTRIB        = 30;
  static final int NIOP_GETATTRIBAT         = 31;
  static final int NIOP_GETATTRIBVALUE      = 32;
  static final int NIOP_GETMODCNT_FROM_ITEM = 33;
  static final int NIOP_GETMODCNT_FROM_DIR  = 34;
  static final int NIOP_DIRUPDATE           = 35;
  static final int NIOP_DIRDELETE           = 36;
  static final int NIOP_DIRSETDN            = 37;
  static final int NIOP_DIRADDATTR          = 38;
  static final int NIOP_SETSECURITYBUFFER   = 39;
  
  static final int NIOP_GETILSDATA_SPEC     = 40;
  static final int NIOP_GETILSDATA_NEXT     = 41;
  static final int NIOP_SETILSBUFFER        = 42;
  static final int NIOP_QUERY_CONTEXT       = 43;
  
  //------------------------------------------------------------------
  // Native return codes
  //------------------------------------------------------------------
  static final int NIRC_OK          = 0;
  static final int NIRC_KXC_ERROR   = 1;
  static final int NIRC_UNKNOWN_OP  = 2;
  static final int NIRC_GEN_FAIL    = 3;
  
  //------------------------------------------------------------------
  // Common parameters to the native layer. These can be of three
  // types : either integers, Java objects or longs.
  //------------------------------------------------------------------
  
  //------------------------------------------------------------------
  // Base integer parameter id.
  //------------------------------------------------------------------
  static final int IP_COMM = 0;
  //------------------------------------------------------------------
  // Operation code id
  //------------------------------------------------------------------
  static final int OP_CODE = IP_COMM+0;
  //------------------------------------------------------------------
  // Options id ( eg, application, storage, put or get )
  //------------------------------------------------------------------
  static final int OPT = IP_COMM+4;
  //------------------------------------------------------------------
  // Transaction type
  //------------------------------------------------------------------
  static final int TR_TYPE = IP_COMM+5;
  //------------------------------------------------------------------
  // Transaction context
  //------------------------------------------------------------------
  static final int TR_CTX = IP_COMM+7;
  //------------------------------------------------------------------
  // Signal
  //------------------------------------------------------------------
  static final int WAIT_LEN = IP_COMM+8;
  
  //------------------------------------------------------------------
  // Integer parameters for error handling
  //------------------------------------------------------------------
  static final int DEFAULT_ACTIONS      = IP_COMM+9;
  static final int ACTIONS              = IP_COMM+10;
  static final int EXC_FLAGS            = IP_COMM+11;
  static final int EXC_SECS             = IP_COMM+12;
  static final int EXC_MSECS            = IP_COMM+13;
  static final int EXC_USERDATA_LENGTH  = IP_COMM+14;
  static final int EXC_ERRCALL_TYPE     = IP_COMM+15;
  static final int EXC_SYSENV_TYPE      = IP_COMM+16;
  static final int EXC_SYSPROCESS_ID    = IP_COMM+17;
  static final int EXC_SYSTHREAD_ID     = IP_COMM+18;
  static final int EXC_ROMA_ELTYPE      = IP_COMM+19;
  static final int REPORT_MSG           = IP_COMM+20;
  
  //------------------------------------------------------------------
  // Integer parameters for message manipulation. All of the
  // constants counting from IP_MSG map to members of the
  // KXCMSG block in the C API.
  //------------------------------------------------------------------
  //------------------------------------------------------------------
  // Base message parameter id : the message type.
  //------------------------------------------------------------------
  static final int IP_MSG       = 20;
  static final int MSG_TYPE     = IP_MSG+1;
  static final int PRIORITY     = IP_MSG+2;
  static final int PERSISTENCE  = IP_MSG+3;
  static final int FORMAT       = IP_MSG+4;
  static final int FEEDBACK     = IP_MSG+5;
  static final int EXPIRY       = IP_MSG+6;
  static final int CODEPAGE     = IP_MSG+7;
  static final int ENCODING     = IP_MSG+8;
  
  //------------------------------------------------------------------
  // Message buffer handle.
  //------------------------------------------------------------------
  static final int MSGBUF_HANDLE = IP_MSG+11;
  //------------------------------------------------------------------
  // Message buffer size.
  //------------------------------------------------------------------
  static final int MSGBUF_SIZE = IP_MSG+12;
  //------------------------------------------------------------------
  // Message length.
  //------------------------------------------------------------------
  static final int MSG_LENGTH = IP_MSG+13;
  //------------------------------------------------------------------
  // Tag buffer handle.
  //------------------------------------------------------------------
  static final int TAGBUF_HANDLE = IP_MSG+14;
  //------------------------------------------------------------------
  // Tag buffer size.
  //------------------------------------------------------------------
  static final int TAGBUF_SIZE = IP_MSG+15;
  //------------------------------------------------------------------
  // Length of tag data.
  //------------------------------------------------------------------
  static final int TAG_LENGTH = IP_MSG+16;
  //------------------------------------------------------------------
  // Security status.
  //------------------------------------------------------------------
  static final int SEC_STATUS = IP_MSG+17;
  //------------------------------------------------------------------
  // Backout count.
  //------------------------------------------------------------------
  static final int BACKOUT_COUNT = IP_MSG+18;
  //------------------------------------------------------------------
  // Seconds component of the message timestamp.
  //------------------------------------------------------------------
  static final int TIMESTAMP_SECS = IP_MSG+19;
  //------------------------------------------------------------------
  // Milliseconds component of the message timestamp.
  //------------------------------------------------------------------
  static final int TIMESTAMP_MSECS = IP_MSG+20;
  
  //------------------------------------------------------------------
  // All of the constants counting from IP_ERR map to members of the
  // KXCERROR structure. At the C-API level a KXCERROR structure
  // has two KXCSUBERROR blocks as members : one for the main error
  // and one for the sub-error. The below constants interleave the
  // members of the two separate KXCSUBERROR structures.
  //------------------------------------------------------------------
  static final int IP_ERR = 40;
  //------------------------------------------------------------------
  // Status of the error.
  //------------------------------------------------------------------
  static final int ERR_STATUS = IP_ERR+1;
  //------------------------------------------------------------------
  // Completion code of the result (ie, main) suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_CC = IP_ERR+2;
  //------------------------------------------------------------------
  // Completion code of the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_CC = IP_ERR+3;
  //------------------------------------------------------------------
  // Return code of the result (ie, main) suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_RC = IP_ERR+4;
  //------------------------------------------------------------------
  // Return code of the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_RC = IP_ERR+5;
  //------------------------------------------------------------------
  // Op code of the result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_OP = IP_ERR+6;
  //------------------------------------------------------------------
  // Op code of the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_OP = IP_ERR+7;
  //------------------------------------------------------------------
  // Reason code of the result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_REASON = IP_ERR+8;
  //------------------------------------------------------------------
  // Reason code of the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_REASON = IP_ERR+9;
  //------------------------------------------------------------------
  // Position identifier from the result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_POS = IP_ERR+10;
  //------------------------------------------------------------------
  // Position identifier from the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_POS = IP_ERR+11;
  //------------------------------------------------------------------
  // Error type identifier from the result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_TYPE = IP_ERR+12;
  //------------------------------------------------------------------
  // Error type identifier from the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_TYPE = IP_ERR+13;
  
  //------------------------------------------------------------------
  // Despite the fact that the following integer constants count from
  // IP_ERR, they are NOT related to error handling. 
  // The following three constants map to members of the KXCMSGINFO
  // strcture.
  //------------------------------------------------------------------
  //------------------------------------------------------------------
  // Status value of a KXCMSGINFO control block.
  //------------------------------------------------------------------
  static final int MSGINFO_STATUS = IP_ERR+14;
  //------------------------------------------------------------------
  // Length of the workflow buffer member of a KXCMSGINFO block.
  //------------------------------------------------------------------
  static final int WORKFLOW_LENGTH = IP_ERR+15;
  //------------------------------------------------------------------
  // Despite the fact that the following integer constant hangs off
  // IP_ERR, it is not related to error handling. 
  //------------------------------------------------------------------
  //------------------------------------------------------------------
  // Not clear what how this is used.
  //------------------------------------------------------------------
  static final int INTERPRET_OPT = IP_ERR+16;
  
  //------------------------------------------------------------------
  // Despite the fact that the following integer constants count from
  // IP_ERR, they are NOT related to error handling.
  //------------------------------------------------------------------
  //------------------------------------------------------------------
  // DataLen member of the KXCLOGDATA structure.
  //------------------------------------------------------------------
  static final int LOGDATA_LEN = IP_ERR+17;
  //------------------------------------------------------------------
  // Severity member of the KXCLOGDATA structure.
  //------------------------------------------------------------------
  static final int LOG_SEVERITY = IP_ERR+18;
  //------------------------------------------------------------------
  // Format member of the KXCLOGDATA structure.
  //------------------------------------------------------------------
  static final int LOG_FORMAT = IP_ERR+19;
  
  //------------------------------------------------------------------
  // Subject stuff
  //------------------------------------------------------------------
  static final int IP_SUB = 60;
  //------------------------------------------------------------------
  // xx
  //------------------------------------------------------------------
  static final int STYLE = IP_SUB+1;
  
  //------------------------------------------------------------------
  // Directory Access
  //------------------------------------------------------------------ 
  static final int IP_DIR                 = 61;
  static final int DIR_SRC_OPT            = 62;
  static final int DIR_SRC_TYPE           = 63;
  static final int DIR_SRC_MATCHES        = 64;
  static final int DIR_SRC_COUNT          = 65;
  static final int DIR_ITEM_DETAIL_LENGTH = 66;
  static final int DIR_ITEM_NBR           = 67;
  
  //------------------------------------------------------------------
  // In-line service data params 
  //------------------------------------------------------------------
  static final int ILS_IDENTITY   = 69;
  static final int ILS_TYPE       = 70;
  static final int ILS_OPTION     = 71;
  static final int PUTGET_STATUS  = 72;
  
  //------------------------------------------------------------------
  // Pub/Sub subject style
  //------------------------------------------------------------------ 
  static final int SUBJECT_STYLE = 73;
  
  //------------------------------------------------------------------
  // Last of the integer parameter id's.
  //------------------------------------------------------------------
  static final int IP_LAST = 80;
  
  //------------------------------------------------------------------
  // The remaining parameter indices are for object parameters passed
  // between the native layers.
  //------------------------------------------------------------------
  //------------------------------------------------------------------
  // Base object parameter id.
  //------------------------------------------------------------------
  static final int OBP_FIRST = 0;
  
  //------------------------------------------------------------------
  // String. The name of the client, business element etc.
  //------------------------------------------------------------------
  static final int NAME = OBP_FIRST+0;
  //------------------------------------------------------------------
  // Byte array. Security buffer object.
  //------------------------------------------------------------------
  static final int SECBUF = OBP_FIRST+1;
  //------------------------------------------------------------------
  // Byte array. Message id of a message.
  //------------------------------------------------------------------
  static final int MSGID = OBP_FIRST+3;
  //------------------------------------------------------------------
  // Byte array. The correl id of a message.
  //------------------------------------------------------------------
  static final int CORRELID = OBP_FIRST+4;
  //------------------------------------------------------------------
  // KxcBuffer. Message buffer of a message.
  //------------------------------------------------------------------
  static final int MSGBUF = OBP_FIRST+5;
  //------------------------------------------------------------------
  // KxcBuffer. Tag buffer of a message.
  //------------------------------------------------------------------
  static final int TAGBUF = OBP_FIRST+6;
  //------------------------------------------------------------------
  // String. Format name of a message.
  //------------------------------------------------------------------
  static final int FORMAT_NAME = OBP_FIRST+7;
  //------------------------------------------------------------------
  // String. Location value of a main result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_LOC = OBP_FIRST+8;
  //------------------------------------------------------------------
  // String. Location value of a sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_LOC = OBP_FIRST+9;
  //------------------------------------------------------------------
  // String. Full text info from the main result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_FULL_TEXT = OBP_FIRST+10;
  //------------------------------------------------------------------
  // String. Full text info from the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_FULL_TEXT = OBP_FIRST+11;
  //------------------------------------------------------------------
  // String. Abbreviated text info from the main result suberror.
  //------------------------------------------------------------------
  static final int ERR_MAIN_SHORT_TEXT = OBP_FIRST+12;
  //------------------------------------------------------------------
  // String. Abbreviated text info from the sub-result suberror.
  //------------------------------------------------------------------
  static final int ERR_SUB_SHORT_TEXT = OBP_FIRST+13;
  //------------------------------------------------------------------
  // String. Error profile name of a KxcReportCtrl.
  //------------------------------------------------------------------
  static final int PROFILE_NAME = OBP_FIRST+14;
  //------------------------------------------------------------------
  // Byte array. User data buffer part of an exception.
  //------------------------------------------------------------------
  static final int EXC_USERDATA = OBP_FIRST+15;
  //------------------------------------------------------------------
  // String. Domain name in a system config exception.
  //------------------------------------------------------------------
  static final int EXC_SYSDOMAIN_NAME = OBP_FIRST+16;
  //------------------------------------------------------------------
  // String. Userid part of a system config exception.
  //------------------------------------------------------------------
  static final int EXC_SYSUSER_ID = OBP_FIRST+17;
  //------------------------------------------------------------------
  // String. Program name from a system config exception.
  //------------------------------------------------------------------
  static final int EXC_SYSPROG_NAME = OBP_FIRST+18;
  //------------------------------------------------------------------
  // String. The element name from a Roma config exception.
  //------------------------------------------------------------------
  static final int EXC_ROMA_ELNAME = OBP_FIRST+19;
  //------------------------------------------------------------------
  // String. The queue name from a Roma config exception.
  //------------------------------------------------------------------
  static final int EXC_ROMA_QNAME = OBP_FIRST+20;
  //------------------------------------------------------------------
  // String. Business component name from a Roma config exception.
  //------------------------------------------------------------------
  static final int EXC_ROMA_COMPNAME = OBP_FIRST+21;
  //------------------------------------------------------------------
  // String.Business service name from a Roma config exception.
  //------------------------------------------------------------------
  static final int EXC_ROMA_SVCNAME = OBP_FIRST+22;
  //------------------------------------------------------------------
  // String. Node name from a Roma config exception.
  //------------------------------------------------------------------
  static final int EXC_ROMA_NODENAME = OBP_FIRST+23;
  //------------------------------------------------------------------
  // Byte array. The workflow data buffer of a message info.
  //------------------------------------------------------------------
  static final int WORKFLOW_DATA = OBP_FIRST+24;
  //------------------------------------------------------------------
  // Byte array. Data buffer of a log data structure.
  //------------------------------------------------------------------
  static final int LOGDATA = OBP_FIRST+25;
  //------------------------------------------------------------------
  // String. User text part of a log data structure. 
  //------------------------------------------------------------------
  static final int LOG_USERTEXT = OBP_FIRST+26;
  //------------------------------------------------------------------
  // String. Name of the API profile associated with a get or put
  // options structure. 
  //------------------------------------------------------------------
  static final int API_PROFILE = OBP_FIRST+27;
  //------------------------------------------------------------------
  // String. Name of the error profile associated with a get or put
  // options structure. 
  //------------------------------------------------------------------
  static final int ERROR_PROFILE = OBP_FIRST+28;
  //------------------------------------------------------------------
  // Pub-Sub subject
  //------------------------------------------------------------------
  static final int SUBJECT = OBP_FIRST+29;
  //------------------------------------------------------------------
  // String. Format name of a message.
  //------------------------------------------------------------------
  static final int MESSAGE_NAME = OBP_FIRST+30;
  //------------------------------------------------------------------
  // Directory Access
  //------------------------------------------------------------------
  
  //------------------------------------------------------------------
  // String. Name of record for starting point of search 
  //------------------------------------------------------------------
  static final int DIR_START = OBP_FIRST+31;
  
  //------------------------------------------------------------------
  // String. Name and attribute to search for in directory
  //------------------------------------------------------------------
  static final int DIR_NAME = OBP_FIRST+32;
  
  //------------------------------------------------------------------
  // Byte array. Item (i.e. directory entry) 
  //------------------------------------------------------------------
  static final int DIR_ITEM_DETAIL = OBP_FIRST+33;
  
  //------------------------------------------------------------------
  // String. Distinguished name of an item 
  //------------------------------------------------------------------
  static final int DIR_DIST_NAME = OBP_FIRST+34;
  
  //------------------------------------------------------------------
  // String. Attribute Name 
  //------------------------------------------------------------------
  static final int DIR_ATTR_NAME = OBP_FIRST+35;
  
  //------------------------------------------------------------------
  // String. Attribute Value 
  //------------------------------------------------------------------
  static final int DIR_ATTR_VALUE = OBP_FIRST+36;
  
  //------------------------------------------------------------------
  // String. Identity for validating update and delete 
  //------------------------------------------------------------------
  static final int DIR_IDENTITY = OBP_FIRST+37;
  
  //------------------------------------------------------------------
  // String. Password for validating update and delete 
  //------------------------------------------------------------------
  static final int DIR_PASSWORD = OBP_FIRST+38;
  
  //------------------------------------------------------------------
  // Byte array. Inline service buffer
  //------------------------------------------------------------------
  static final int  ILS_BUFFER = OBP_FIRST+39;
  
  //------------------------------------------------------------------
  // String. Model Queue Name
  //------------------------------------------------------------------
  static final int  MODEL_QUEUE = OBP_FIRST+40;
  
  // last object parameter
  static final int OBP_LAST = 41;
  
  //------------------------------------------------------------------
  // Base long parameter id.
  //------------------------------------------------------------------
  static final int LP_COMM = 0;
  //------------------------------------------------------------------
  // Roma context handle id
  //------------------------------------------------------------------
  static final int HCTX = LP_COMM+0;
  //------------------------------------------------------------------
  // Queue handle id
  //------------------------------------------------------------------
  static final int HQUEUE = LP_COMM+1;
  //------------------------------------------------------------------
  // Theme handle id
  //------------------------------------------------------------------
  static final int HOBJ = LP_COMM+2;
  //------------------------------------------------------------------
  // KXCMSGINFO block's hTheme handle.
  //------------------------------------------------------------------
  static final int MSGINFO_HTHEME = HOBJ;
  //------------------------------------------------------------------
  // Inline service data handle
  //------------------------------------------------------------------
  static final int ILS_DATA_HANDLE = LP_COMM+3;
  
  // last object parameter (actually size of m_lp[]
  static final int LP_LAST = 4;
}
