//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcOper.java
//
// Defines      : class KxcOper
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Native interface abstraction in the raw interface.
* Maintains pool of parameters which are passed between
* native interface and Java.
* Introduces 2-stage operation invocation.
* In first stage operation is intialized and returned from
* one of KxcContext, KxcClient or KxcServer calls.
* Then the user should call invoke() method on the operation.
* This allows to simplify exception mapping in the client code,
* because exceptions arise only in a single place.
*
* Also, by abstracting doOper() method, this class makes it
* possible to add remote call capability in the model.
*/

public abstract class KxcOper implements KxcNativeInterface
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // Used to notify Roma C API error exception
  // Provided with a reference to KxcError object,
  // which maintains exception details.
  public static class ApiException extends Exception
  {
    private static final String copyrightNotice =
      "Copyright Candle Corporation 1999,2000,2001";
    
    ApiException( KxcError err )
    {
      m_err = err;
    }
    
    public KxcError error()
    {
      return m_err;
    }
    
    KxcError m_err;
  }
  
  // General exception
  public static class GenException extends Exception
  {
    private static final String copyrightNotice =
      "Copyright Candle Corporation 1999,2000,2001";
    
    GenException( String s )
    {
      super( s );
    }
  }
  
  // the invocation interface. May throw any exceptions.
  public void invoke()
    throws Exception
  {
    int nirc = doOper();
    if ( nirc != NIRC_OK )
    {
      if ( nirc == NIRC_KXC_ERROR )
        throw new ApiException( new KxcError( this ) );

      // process native code error
      String s = "native error: "+nirc;
      throw new GenException( s );
    }
  }
  
  protected abstract int doOper()
    throws Exception;
  
  int     m_ip[]  = new int [IP_LAST];
  Object  m_op[]  = new Object [OBP_LAST];
  long    m_lp[]  = new long[LP_LAST];
}
