//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcSubError.java
//
// Defines      : class KxcSubError
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Abstracts KXCSUBERROR C API structure as object in the KxcOper parameters
* pool.
*/

public class KxcSubError extends KxcStruct
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  KxcSubError( KxcOper oper, int off )
  {
    super( oper );
    m_off = off;
  }
  
  public int compCode()
  {
    return m_ip[ERR_MAIN_CC+m_off];
  }
  
  public int returnCode()
  {
    return m_ip[ERR_MAIN_RC+m_off];
  }
  
  public int opType()
  {
    return m_ip[ERR_MAIN_OP+m_off];
  }
  
  public int reason()
  {
    return m_ip[ERR_MAIN_REASON+m_off];
  }
  
  public int position()
  {
    return m_ip[ERR_MAIN_POS+m_off];
  }
  
  public int type()
  {
    return m_ip[ERR_MAIN_TYPE+m_off];
  }
  
  public String location()
  {
    return (String)m_op[ERR_MAIN_LOC+m_off];
  }
  
  public String fullText()
  {
    return (String)m_op[ERR_MAIN_FULL_TEXT+m_off];
  }
  
  public String shortText()
  {
    return (String)m_op[ERR_MAIN_SHORT_TEXT+m_off];
  }
  
  public void setCompCode( int v )
  {
    m_ip[ERR_MAIN_CC+m_off] = v;
  }
  
  public void setReturnCode( int v )
  {
    m_ip[ERR_MAIN_RC+m_off] = v;
  }
  
  public void setOpType( int v )
  {
    m_ip[ERR_MAIN_OP+m_off] = v;
  }
  
  public void setReason( int v )
  {
    m_ip[ERR_MAIN_REASON+m_off] = v;
  }
  
  public void setPosition( int v )
  {
    m_ip[ERR_MAIN_POS+m_off] = v;
  }
  
  public void setType( int v )
  {
    m_ip[ERR_MAIN_TYPE+m_off] = v;
  }
  
  public void setLocation( String loc )
  {
    m_op[ERR_MAIN_LOC+m_off] = loc;
  }
  
  private final int m_off;
}
