//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcjbnxa.cpp
//
// Description  : Java native library
//
// Implements   : class RomaJxOper
//
//--------------------------------------------------------------------

// Standard includes
#include <stdio.h>
#include <string.h>

// KXC includes
#include <kxcenv.h>
#include <kxcutl.h>

// KXCJNI includes
#include "kxcjbnxa.hpp"


#define TRACE2_HEX( s, x ) s << "=" << HEX << (x) << DEC << "; "

#define TRACE2( s, x ) s << "=" << (x) << "; "

#define TRACE_SYNC( sync )                                           \
  TRACE_EVENT( << "KXCSYNC details: "                                \
               << TRACE2_HEX( "Type", kxcGetSyncType( &sync ))       \
               << TRACE2_HEX( "Context", kxcGetSyncContext( &sync )) \
             );

#define PREP_SYNC( sync )                                  \
  kxcSetSyncDefault( &sync );                              \
  kxcSetSyncType( &sync, intParam( TR_TYPE ));             \
  kxcSetSyncContext( &sync, (PKXCVOID)intParam( TR_CTX )); \
  TRACE_SYNC(sync);

#ifdef KXC_HPUX
inline int max(int iValue1,int iValue2)
{
  return (iValue1 > iValue2) ? iValue1 : iValue2;
}
#endif

#define PREP_SEC( sec )                                                        \
  JByteArray secBuf;                                                           \
  byteArrayParam( SECBUF, secBuf );                                            \
  sec = (PKXCSECDEF)secBuf.mem();                                              \
  TRACE_EVENT( << TRACE2_HEX( "Security", sec ));                              \
  if (sec)                                                                     \
  {                                                                            \
    TRACE_DUMP( sec, max( ((PKXCSECDEF)sec)->TotalLen, 16 ), << "Security:" ); \
  }


#define TRACE_ERROR( err )                                                                     \
  TRACE_EVENT( << "KXCERROR content: " << TRACE_HEX(err.Status) );                             \
  TRACE_EVENT( << "Result:" << TRACE2("LastCompCode", err.Result.LastCompCode)                 \
                            << TRACE2("LastReturnCode", err.Result.LastReturnCode)             \
                            << TRACE2("LastOpType", err.Result.LastOpType)                     \
                            << TRACE2("LastReason", err.Result.LastReason)                     \
                            << TRACE2("LastPosition", err.Result.LastPosition)                 \
                            << TRACE2("LastLocation", err.Result.LastLocation)                 \
                            << TRACE2("LastErrorType", err.Result.LastErrorType)               \
             );                                                                                \
  if ( err.Status & KXCERRSTAT_SUB )                                                           \
  {                                                                                            \
    TRACE_EVENT( << "SubResult:" << TRACE2("Sub-LastCompCode", err.SubResult.LastCompCode)     \
                                 << TRACE2("Sub-LastReturnCode", err.SubResult.LastReturnCode) \
                                 << TRACE2("Sub-LastOpType", err.SubResult.LastOpType)         \
                                 << TRACE2("Sub-LastReason", err.SubResult.LastReason)         \
                                 << TRACE2("Sub-LastPosition", err.SubResult.LastPosition)     \
                                 << TRACE2("Sub-LastLocation", err.SubResult.LastLocation)     \
                                 << TRACE2("Sub-LastErrorType", err.SubResult.LastErrorType)   \
               );                                                                              \
  }

#define TRACE_BUFFER( buf, text )                 \
  TRACE_DUMP( buf.pBuffer, min( buf.BufLen, 256), \
              << text                             \
              << TRACE_HEX(buf.Status)            \
              << TRACE(buf.BufLen) );

#define TRACE_MSG( msg )                                                   \
  TRACE_EVENT( << "KXCMSG content: " << TRACE( msg.MsgType )               \
                                     << TRACE( msg.Priority )              \
                                     << TRACE( msg.Persistence )           \
                                     << TRACE( msg.Format )                \
                                     << TRACE( msg.Feedback )              \
                                     << TRACE( msg.Expiry )                \
                                     << TRACE_HEX( msg.Encoding )          \
                                     << TRACE( msg.CodePage )              \
                                     << TRACE( msg.Status )                \
                                     << TRACE( msg.BackoutCount )          \
                                     << TRACE_OBJNAME( msg.FormatName )    \
                                     << TRACE_OBJNAME( msg.MessageName )   \
                                     << TRACE_HEX(msg.TimeStamp.Seconds)   \
                                     << TRACE_HEX(msg.TimeStamp.Millisecs) \
             );                                                            \
  TRACE_DUMP( msg.MsgId, KXCMSG_ID_LENGTH, << "MsgId:" );                  \
  TRACE_DUMP( msg.CorrelId, KXCMSG_ID_LENGTH, << "CorrelId:" );            \
  TRACE_BUFFER( msg.MsgBuffer, "MsgBuffer: " << TRACE(msg.MsgLen) );       \
  TRACE_BUFFER( msg.TagBuffer, "TagBuffer: " << TRACE(msg.TagLen) );

#define GET_STR_PARAM( id, dest ) getStrParam( id, dest, sizeof(dest) )

#define SET_STR_PARAM( id, src )                          \
  {                                                       \
    KXCCHAR buf[sizeof(src)+1];                           \
    strncpy( (char*)buf, (const char*)src, sizeof(src) ); \
    buf[sizeof(src)] = '\0';                              \
    setStrParam( id, (char*)buf );                        \
  }

#define TRACE_EXCEPTION( exc )                                                                 \
  TRACE_EVENT( << "KXCEXCEPTION details:"   << TRACE_HEX( exc.ExceptionFlags )                 \
                                            << TRACE_HEX( exc.TimeStamp.Seconds )              \
                                            << TRACE_HEX( exc.TimeStamp.Millisecs )            \
             );                                                                                \
  TRACE_EVENT( << "KXCEXCROMACONF details:" << TRACE_OBJNAME( exc.RomaConfData.ElementName )   \
                                            << TRACE_HEX( exc.RomaConfData.ElementType )       \
                                            << TRACE_OBJNAME( exc.RomaConfData.QueueName )     \
                                            << TRACE_OBJNAME( exc.RomaConfData.ComponentName ) \
                                            << TRACE_OBJNAME( exc.RomaConfData.ServiceName )   \
                                            << TRACE_OBJNAME( exc.RomaConfData.NodeName )      \
             );                                                                                \
  TRACE_EVENT( << "KXCEXCSYSCONF details:"  << TRACE_HEX( exc.SystemConfData.EnvironmentType ) \
                                            << TRACE( exc.SystemConfData.DomainName )          \
                                            << TRACE( exc.SystemConfData.UserId )              \
                                            << TRACE( exc.SystemConfData.ProgramName )         \
                                            << TRACE_HEX( exc.SystemConfData.ProcessId )       \
                                            << TRACE_HEX( exc.SystemConfData.ThreadId )        \
             );

#define TRACE_EXCEPTION_USERINFO( exc )                                                            \
  TRACE_EVENT( << "KXCEXCERRDATA details: " << TRACE(exc.ErrorData.CallType) );                    \
  TRACE_ERROR( exc.ErrorData.Error );                                                              \
  TRACE_BUFFER( exc.UserData.UserBuf, "KXCEXCUSERDATA details: " << TRACE(exc.UserData.UserBufLen) );


//---------------------------------------------------------------------
// Class name : ObjNameTracer
//---------------------------------------------------------------------
class ObjNameTracer
{
public:
  ObjNameTracer( KXCCHAR* objName) : m_objName(objName)
  {
    m_lastCh = m_objName[KXCOBJECT_NAME_LENGTH-1];
    m_objName[KXCOBJECT_NAME_LENGTH-1] = '\0';
  }
  ~ObjNameTracer()
  {
    m_objName[KXCOBJECT_NAME_LENGTH-1] = m_lastCh;
  }
  operator const KXCCHAR* () const
  {
    return m_objName;
  }
private:
  KXCCHAR* m_objName;
  KXCCHAR m_lastCh;
};

#define TRACE_OBJNAME(name) ObjNameTracer(name)

TRACE_MODULE_VERSION($Revision: 1.1.1.1.6.3.2.2 $);


//=====================================================================
// Method name : RomaJxOper::processKxcError
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::processKxcError()
{
  TRACE_ENTRY( RomaJxOper::processKxcError, << "" );
  KXCERROR err;
  if  ( kxcGetLastError( &err ) )
  {
    TRACE_ERROR( err );
    setError( err );
  }
  else
  {
    TRACE_INT_RETURN( JXRC_GEN_FAIL );
  }

  TRACE_INT_RETURN( JXRC_KXC_ERROR );
}

//=====================================================================
// Method name : RomaJxOper::doOper
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::doOper()
{
  int opCode = intParam( OP_CODE ), retCode;
  TRACE_ENTRY( RomaJxOper::doOper, << TRACE(opCode) );

  switch (opCode)
  {
  case JXOP_CLJXJXT:
    retCode = init( CLIENT );
    break;
  
  case JXOP_SVRIJXT:
    retCode = init( SERVER );
    break;
  
  case JXOP_PUTREQ:
    retCode = put( CLIENT );
    break;
  
  case JXOP_PUBLISH:
    retCode = put( PUBLISH );
    break;
  
  case JXOP_SUBSCRIBE:
    retCode = put( SUBSCRIBE );
    break;
  
  case JXOP_UNSUBSCRIBE:
    retCode = put( UNSUBSCRIBE );
    break;
  
  case JXOP_GETREQ:
    TRACE_EVENT(<< "get server");
    retCode = get( SERVER );
    TRACE_EVENT(<<"return from get");
    break;
  
  case JXOP_PUTRESP:
    retCode = put( SERVER );
    break;
  
  case JXOP_GETRESP:
    retCode = get( CLIENT );
    break;
  
  case JXOP_CLOSE_HANDLE:
    retCode = closeHandle();
    break;
  
  case JXOP_COMMIT:
  case JXOP_ROLLBACK:
    retCode = txControl( opCode );
    break;
  
  case JXOP_OPEN_STORAGEQ:
    retCode = openStorageQ();
    break;
  
  case JXOP_PUT_STORAGEQ:
    retCode = put( STORAGEQ );
    break;
  
  case JXOP_GET_STORAGEQ:
    retCode = get( STORAGEQ );
    break;
  
  case JXOP_CLOSE_STORAGEQ:
    retCode = closeStorageQ();
    break;
  
  case JXOP_DELETEQ:
    retCode = deleteStorageQ();
    break;
  
  case JXOP_TERMINATE:
    retCode = terminate();
    break;
  
  case JXOP_INTERPRETMSG:
    retCode = interpretMsg();
    break;
  
  case JXOP_PUTEXCEPTION:
    retCode = put( EXCEPT );
    break;
  
  case JXOP_GETEXCEPTION:
    retCode = get( EXCEPT );
    break;
  
  case JXOP_REPORTERROR:
    retCode = reportError();
    break;
  
  case JXOP_WRITELOG:
    retCode = writeLog(false);
    break;
  
  case JXOP_SENDALERT:
    retCode = writeLog(true);
    break;

  case JXOP_DIR_LOOKUP:
    retCode = dirLookup();
    break;

  case JXOP_DIR_GETMODCNT_FROM_DIR:
    retCode = dirGetModCountFromDir();
    break;
  
  case JXOP_DIR_GETMODCNT_FROM_ITEM:
    retCode = dirGetModCountFromItem();
    break;
  
  case JXOP_DIR_GETITEM:
    retCode = dirGetItem();
    break;
  
  case JXOP_DIR_GETDNAME:
    retCode = dirGetDN();
    break;
  
  case JXOP_DIR_CNTATTRIB:
    retCode = dirCountAttribs();
    break;
  
  case JXOP_DIR_GETATTRAT:
    retCode = dirGetAttributeAt();
    break;
  
  case JXOP_DIR_GETVALUE:
    retCode = dirDirGetValue();
    break;
  
  case JXOP_DIR_UPDATE:
    retCode = dirUpdateItem();
    break;
  
  case JXOP_DIR_DELETE:
    retCode = dirDeleteItem();
    break;
  
  case JXOP_DIR_SETDN:
    retCode = dirSetDN();
    break;
  
  case JXOP_DIR_ADDATTR:
    retCode = dirAddAttr();
    break;
  
  case JXOP_SETSECURITYBUFFER:
    retCode = setSecurityBuffer();
    break;
  
  case JXOP_GETILSBUFFER_SPEC:
    retCode = getILSBufferSpec();
    break;
  
  case JXOP_GETILSBUFFER_NEXT:
    retCode = getILSBufferNext();
    break;
  
  case JXOP_SETILSBUFFER:
    retCode = setILSBuffer();
    break;
  
  case JXOP_QUERY_CONTEXT:
    retCode = queryContext();
    break;
  
  default:
    TRACE_INT_RETURN( JXRC_UNKNOWN_OP );
  }

  // process error if any
  if (retCode == JXRC_KXC_ERROR)
    int retCode = processKxcError();

  TRACE_EVENT(<<"return from doOper");
  TRACE_INT_RETURN( retCode );
}


//=====================================================================
// Method name : RomaJxOper::init
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::init( CtxType ctype )
{
  TRACE_ENTRY( RomaJxOper::init, << TRACE(ctype) );
  
  KXCAPPDEF appDef;
  
#if defined(AIX)
  putenv("KBB_SIG1=-dumpoff -asyncoff -syncoff");
#endif
  
  kxcSetAppDefDefault( &appDef );
#if defined(AIX)
  getObjName( NAME, appDef.ElementName );
  getObjName( MODEL_QUEUE, appDef.ModelQName );
#else
  getObjName( NAME, &appDef.ElementName );
  getObjName( MODEL_QUEUE, &appDef.ModelQName );
#endif
  kxcSetAppOptions( &appDef, intParam(OPT) );
  
  TRACE_EVENT( << TRACE_OBJNAME(appDef.ElementName)
               << TRACE_HEX(appDef.Options)
             );
  
  PREP_SEC( appDef.pSecurity );
  HKXC hKxc;
  KXCBOOL b;
  
  switch (ctype)
  {
  case CLIENT:
    TRACE_EVENT( << "About to call kxcClientInitialize" );
    b = kxcClientInitialize( &appDef, &hKxc );
    break;

  case SERVER:
    TRACE_EVENT( << "About to call kxcServerInitialize" );
    b = kxcServerInitialize( &appDef, &hKxc );
    break;
  
  default:
    TRACE_INT_RETURN( JXRC_UNKNOWN_OP );
  }
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  // store context handle
  longParam(HCTX) = hKxc;
  
  TRACE_EVENT( << "Success:" << TRACE_HEX(hKxc) );
  TRACE_INT_RETURN( JXRC_OK );
}


//=====================================================================
// Method name : RomaJxOper::terminate
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::terminate()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::terminate, << TRACE_HEX(hKxc) );
  KXCBOOL b = kxcTerminate( &hKxc );
  longParam( HCTX ) = hKxc;
  
  if ( !b )
    TRACE_INT_RETURN( JXRC_KXC_ERROR );

  TRACE_EVENT( << "Success: " << TRACE_HEX(hKxc) );
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::getMsgId
// Description : 
// Return type : void 
//---------------------------------------------------------------------
inline
void RomaJxOper::getMsgId( KXCMSGID& msgId, int i )
{
  JByteArray jba;
  byteArrayParam( i, jba  );

  if (jba.mem())
    jba.copyTo( msgId, KXCMSG_ID_LENGTH );
}

//=====================================================================
// Method name : RomaJxOper::setMsgId
// Description : 
// Return type : void 
//---------------------------------------------------------------------
inline
void RomaJxOper::setMsgId( const KXCMSGID& msgId, int i )
{
  JByteArray jba;
  newByteArrayParam( i, KXCMSG_ID_LENGTH, jba );
  jba.copyFrom( msgId, KXCMSG_ID_LENGTH );
}

//=====================================================================
// Method name : RomaJxOper::put
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::put( CtxType ctype )
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::put, << TRACE(ctype) 
                                << TRACE_HEX(hKxc) );
  
  // prepare message, put options, pub-sub list
  KXCPUTOPT opt;
  KXCMSG msg;
  HKXCTHEME theme;
  KXCPSLIST PubSubList;
  
  // prepare options/theme
  kxcSetPutOptDefault( &opt );
  kxcSetPutOptions( &opt, intParam( OPT ));
  getMsgId( opt.MsgId, MSGID );
  getMsgId( opt.CorrelId, CORRELID );
  PREP_SEC( opt.pSecurity );
#if defined(AIX)
  getObjName( API_PROFILE, opt.APIProfile );
  getObjName( ERROR_PROFILE, opt.ErrorProfile );
#else
  getObjName( API_PROFILE, &opt.APIProfile );
  getObjName( ERROR_PROFILE, &opt.ErrorProfile );
#endif
  theme = (HKXCTHEME)longParam( HOBJ );
  
  HKXCILS ilsDataHandle = (HKXCILS)longParam( ILS_DATA_HANDLE );
  kxcSetPutILSData(&opt, ilsDataHandle);
  
  // trace options/theme
  TRACE_EVENT( << TRACE2_HEX( "PutOptions", opt.RequestOptions )
               << TRACE2_HEX( "PutSecurity", opt.pSecurity )
               << TRACE2_HEX( "PutTheme", theme )
               << TRACE( opt.APIProfile )
               << TRACE( opt.ErrorProfile )
               << TRACE( ilsDataHandle )
             );
  TRACE_DUMP( opt.MsgId, KXCMSG_ID_LENGTH, << "MsgId:" )
  TRACE_DUMP( opt.CorrelId, KXCMSG_ID_LENGTH, << "CorrelId:" )
    
  // prepare KXCMSG structure
  kxcSetMsgDefault( &msg );
  msg.Priority    = intParam( PRIORITY );
  msg.Persistence = intParam( PERSISTENCE );
  msg.Format      = intParam( FORMAT );
  msg.Feedback    = intParam( FEEDBACK );
  msg.Expiry      = intParam( EXPIRY );
  msg.CodePage    = intParam( CODEPAGE );
  msg.Encoding    = intParam( ENCODING );

#if defined(AIX)
  getObjName( FORMAT_NAME, msg.FormatName );
  getObjName( MESSAGE_NAME, msg.MessageName );
#else
  getObjName( FORMAT_NAME, &msg.FormatName ); 
  getObjName( MESSAGE_NAME, &msg.MessageName );
#endif

  TRACE_EVENT( << TRACE( msg.Priority )
               << TRACE( msg.Persistence )
               << TRACE( msg.Format )
               << TRACE( msg.Feedback )
               << TRACE( msg.Expiry )
               << TRACE_HEX( msg.Encoding )
               << TRACE( msg.CodePage )
               << TRACE_OBJNAME( msg.FormatName )
               << TRACE_OBJNAME( msg.MessageName )
             );
  
  // prepare put buffers
  JByteArray jmsgBuf;
  byteArrayParam( MSGBUF, jmsgBuf );
  JByteArray jtagBuf;
  byteArrayParam( TAGBUF, jtagBuf );
  msg.MsgBuffer.Status = KXCBUFSTAT_USER;
  msg.MsgBuffer.pBuffer = jmsgBuf.mem();
  
  int intMsgLen = intParam( MSG_LENGTH );
  if (intMsgLen == -1)
    msg.MsgLen = 0;
  else
    msg.MsgLen = intMsgLen;

  msg.MsgBuffer.BufLen = msg.MsgLen;
  
  TRACE_EVENT( << TRACE2( "Status", msg.MsgBuffer.Status )
               << TRACE2_HEX( "pBuffer", msg.MsgBuffer.pBuffer )
               << TRACE2( "BufSize", msg.MsgBuffer.BufLen )
               << TRACE2( "DataLength", msg.MsgLen )
             );

  msg.TagBuffer.Status = KXCBUFSTAT_USER;
  msg.TagBuffer.pBuffer = jtagBuf.mem();
  
  int intTagLen = intParam( TAG_LENGTH );
  if (intTagLen == -1)
    msg.TagLen = 0;
  else
    msg.TagLen = intTagLen;

  msg.TagBuffer.BufLen = msg.TagLen;
  
  TRACE_EVENT( << TRACE2_HEX( "Status", msg.TagBuffer.Status )
               << TRACE2_HEX( "pBuffer", msg.TagBuffer.pBuffer )
               << TRACE2( "BufSize", msg.TagBuffer.BufLen )
               << TRACE2( "DataLength", msg.TagLen )
             );
  
  // prepare syncpoint
  KXCSYNC sync;
  PREP_SYNC( sync );
  
  // Hold the subject string passed in from Java ...
  JString Subject;
  
  // Sort out any special options settings for the PutRequest ...
  switch (ctype)
  {
  case PUBLISH:
    kxcSetPutPSOptions(&opt, KXCPSOPT_PUBLISH);
    break;
  
  case SUBSCRIBE:
    kxcSetPutPSOptions(&opt, KXCPSOPT_SUBSCRIBE);
    break;
  
  case UNSUBSCRIBE:
    kxcSetPutPSOptions(&opt, KXCPSOPT_UNSUBSCRIBE);
    break;
  
  default:
    break;
  }
  
  // prepare service definition
  KXCBOOL b;
  switch (ctype)
  {
  case PUBLISH:
  case SUBSCRIBE:
  case UNSUBSCRIBE:
    {
      // prepare  pub-sub list
      kxcSetPSListDefault(&PubSubList);
    
      // pick up style and subject from JNI array ...
      PubSubList.aSubject.Style = intParam( STYLE );
    
      strParam( SUBJECT, Subject );
      PubSubList.aSubject.pSubject  = (PKXCCHAR)Subject();
      PubSubList.aSubject.DataLen   = strlen((char *)PubSubList.aSubject.pSubject);
    
      // for now, limited to one subject in 'list'
      PubSubList.NumSubjects = 1;
      PubSubList.TotalLen += sizeof(KXCSUBJECT);
    
      kxcSetPutPSbuffer(&opt, &PubSubList);
    }
    // deliberately fall through to the client.
  case CLIENT:
    {
      KXCSERVICE svc;
      getService( svc );
      TRACE_EVENT( << "About to call kxcPutRequest(): " << TRACE_OBJNAME(svc.ServiceName) 
                                                        << TRACE_HEX(svc.Options) );
      b = kxcPutRequest( hKxc, &svc, &msg, &opt, &sync, &theme );
    }
    break;
    
  case SERVER:
    {
      TRACE_EVENT( << "About to call kxcPutResponse()" );
      b = kxcPutResponse( hKxc, &msg, &opt, &sync, &theme );
    }
    break;
  
  case EXCEPT:
    {
      // process exception details
      KXCEXCEPTION exc;
      getException( exc );
      TRACE_EXCEPTION_USERINFO( exc );

      // process service details
      KXCSERVICE svc;
      PKXCSERVICE pSvc = getService( svc );
      if (pSvc) 
      {
        TRACE_EVENT( << "About to call kxcPutException(): " << TRACE2("SvcName", svc.ServiceName) 
                                                            << TRACE2("SvcOptions", svc.Options) );
      }
      else 
      {
        TRACE_EVENT( << "About to call kxcPutException()" );
      }

      b = kxcPutException( hKxc, pSvc, &msg, &exc, &opt, &sync, &theme );
      if (b)
      {
        TRACE_EXCEPTION( exc );
        setException( exc, false );
      }
    }
    break;
  
  case STORAGEQ:
    {
      HKXCSTORQ hStore = (HKXCSTORQ)longParam( HQUEUE );
      TRACE_EVENT( << "About to call kxcPutToStorageQueue(): " << TRACE_HEX(hStore));
      b = kxcPutToStorageQueue( hKxc, hStore, &msg, &opt, &sync, &theme );
    }
    break;
  
  default:
    // unknown context type
    TRACE_INT_RETURN( JXRC_UNKNOWN_OP );
  }
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  // process successful put
  TRACE_EVENT( << "Success: " << TRACE_HEX(theme)
                              << TRACE(msg.MsgType)
                              << TRACE(msg.Expiry)
                              << TRACE_HEX(msg.TimeStamp.Seconds)
                              << TRACE_HEX(msg.TimeStamp.Millisecs)
             );
  TRACE_DUMP( msg.MsgId, KXCMSG_ID_LENGTH, << "MsgId:" )
  TRACE_DUMP( msg.CorrelId, KXCMSG_ID_LENGTH, << "CorrelId:" )
  
  setMsgId( msg.MsgId, MSGID );
  setMsgId( msg.CorrelId, CORRELID );
  longParam( HOBJ )         = theme;
  intParam(MSG_TYPE)        = msg.MsgType;
  intParam(EXPIRY)          = msg.Expiry;
  intParam(TIMESTAMP_SECS)  = msg.TimeStamp.Seconds;
  intParam(TIMESTAMP_MSECS) = msg.TimeStamp.Millisecs;
  intParam(PUTGET_STATUS)   = opt.Status;
  
  ilsDataHandle = kxcGetPutILSData( &opt );
  longParam( ILS_DATA_HANDLE ) = (HKXCILS)ilsDataHandle;
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::get
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::get( CtxType ctype )
{
  JByteArray UserMsgBuf;	// for extracting user buffer
  JByteArray UserTagBuf;	// for extracting tag buffer
  
  int msgBufSize = intParam( MSG_LENGTH );
  
  // prepare hKXC
  HKXC hKxc = (HKXC)longParam( HCTX );

  // trace entry
  TRACE_ENTRY( RomaJxOper::get, << TRACE(ctype) << TRACE_HEX(hKxc) );
  
  // prepare message, get options
  KXCGETOPT opt;
  KXCMSG msg;
  HKXCTHEME theme;
  kxcSetMsgDefault( &msg );
  kxcSetGetOptDefault( &opt );
  opt.ReceiveOptions = intParam(OPT);
  getMsgId( opt.MsgId, MSGID );
  getMsgId( opt.CorrelId, CORRELID );
  PREP_SEC( opt.pSecurity );
  kxcSetGetWaitLength( &opt, intParam(WAIT_LEN) );

#if defined(AIX)
  getObjName( API_PROFILE, opt.APIProfile );
  getObjName( ERROR_PROFILE,opt.ErrorProfile );
#else
  getObjName( API_PROFILE, &opt.APIProfile );	 
  getObjName( ERROR_PROFILE, &opt.ErrorProfile ); 
#endif

  opt.SubjectStyle = intParam( SUBJECT_STYLE );
  theme = (HKXCTHEME)longParam( HOBJ );
  
  HKXCILS ilsDataHandle = (HKXCILS)longParam( ILS_DATA_HANDLE );
  kxcSetGetILSData(&opt, ilsDataHandle);
  
  
  if ( opt.ReceiveOptions & KXCGOPT_USEDEFAULT )
    opt.ReceiveOptions = KXCGOPT_USEDEFAULT;
  else
  {
    if (ctype == EXCEPT)
      opt.ReceiveOptions |= KXCGOPT_ALLOWNATIVE;
    
    if (msgBufSize == -1)
      opt.ReceiveOptions |= KXCGOPT_AUTO_ALLOC;
    else
    {
      byteArrayParam( MSGBUF, UserMsgBuf ); 
      kxcSetMsgBuffer(&msg, msgBufSize, UserMsgBuf.mem());
      
      // Set up user-defined tag buffer
      int tagLength = intParam(TAG_LENGTH);
      if ( tagLength != -1)
      {
        byteArrayParam( TAGBUF, UserTagBuf ); 
        kxcSetTagBuffer(&msg, tagLength, UserTagBuf.mem());
      }
    }
  }

  // trace options/theme
  TRACE_EVENT( << TRACE_HEX( opt.ReceiveOptions )
               << TRACE_HEX( opt.pSecurity )
               << TRACE( opt.WaitLen )
               << TRACE_HEX( theme )
               << TRACE( opt.APIProfile )
               << TRACE( opt.ErrorProfile )
               << TRACE( ilsDataHandle )
             );
  TRACE_DUMP( opt.MsgId, KXCMSG_ID_LENGTH, << "GetMsgId:" );
  TRACE_DUMP( opt.CorrelId, KXCMSG_ID_LENGTH, << "GetCorrelId:" );

  // prepare syncpoint
  KXCSYNC sync;
  PREP_SYNC( sync );
  KXCBOOL b;
  switch( ctype )
  {
  case CLIENT:
    TRACE_EVENT( << "About to call kxcGetResponse()" );
    b = kxcGetResponse( hKxc, &msg, &opt, &sync, &theme );
    break;
  
  case EXCEPT:
  case SERVER:
    TRACE_EVENT( << "About to call kxcGetRequest()" );
    b = kxcGetRequest( hKxc, &msg, &opt, &sync, &theme );
    if (ctype == EXCEPT)
    {
      if (!b)
        TRACE_INT_RETURN( JXRC_KXC_ERROR );
      
      // process exception
      TRACE_MSG( msg );
      KXCMSGINFO msgInfo;
      kxcSetMsgInfoDefault( &msgInfo );
      KXCULONG options = intParam(INTERPRET_OPT);
      TRACE_EVENT( << "About to call kxcInterpretMessage(): " << TRACE_HEX(options) );
      KXCBOOL b = kxcInterpretMessage( hKxc, options, &msg, &msgInfo );
      if (!b)
        TRACE_INT_RETURN( JXRC_KXC_ERROR );
 
      TRACE_EVENT( << "Success" << TRACE_HEX(msgInfo.Status) );
      intParam( MSGINFO_STATUS ) = msgInfo.Status;
      if (msgInfo.Status & KXCMSGPART_MESSAGE)
      {
        KXCMSG& msg = msgInfo.Message;
        TRACE_MSG( msg );
        setMsg( hKxc, msg );
      }
      if (msgInfo.Status & KXCMSGPART_WORKFLOW)
      {
        TRACE_BUFFER( msgInfo.WorkFlow, "Workflow: " );
        setBuffer( msgInfo.WorkFlow, WORKFLOW_DATA, WORKFLOW_LENGTH );
      }
      if (msgInfo.Status & KXCMSGPART_EXCEPTION)
      {
        TRACE_EXCEPTION( msgInfo.Exception );
        TRACE_EXCEPTION_USERINFO( msgInfo.Exception );
        setException( msgInfo.Exception, true );
      }
      if (msgInfo.Status & KXCMSGPART_HTHEME)
      {
        TRACE_EVENT( << TRACE_HEX(msgInfo.hTheme) );
        intParam( MSGINFO_HTHEME ) = msgInfo.hTheme;
      }
      
      // close theme
      kxcCloseHandle( hKxc, &theme );
      
      TRACE_INT_RETURN( JXRC_OK );
      
    }
    break;
  
  case STORAGEQ:
    {
      HKXCSTORQ hStore = (HKXCSTORQ)longParam( HQUEUE );
      TRACE_EVENT( << "About to call kxcGetFromStorageQueue(): " << TRACE_HEX(hStore));
      b = kxcGetFromStorageQueue( hKxc, hStore, &msg, &opt, &sync, &theme );

      // Roma returns reveresed Encoding from kxcGetFromStroageQueue
      msg.Encoding = ~msg.Encoding;
    }
    break;
  
  default:
    // unknown context type
    TRACE_INT_RETURN( JXRC_UNKNOWN_OP );
  };
  
  // process error if any
  if (!b)
  {    
    // Message and tag length are required for 'buffer to small' error case
    intParam(MSG_LENGTH) = msg.MsgLen;
    intParam(TAG_LENGTH) = msg.TagLen;
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  }	
  
  //  trace: theme, message
  TRACE_EVENT( << "Success: " << TRACE_HEX(theme) );
  TRACE_MSG( msg );

  // flag whether auto-allocated
  setMsg( hKxc, msg,  msgBufSize == -1);

  // process theme
  TRACE_EVENT( << "process theme "  );
  longParam(HOBJ) = theme;

  intParam(PUTGET_STATUS) = opt.Status;
  
  ilsDataHandle = kxcGetGetILSData( &opt );
  longParam( ILS_DATA_HANDLE ) = (HKXCILS)ilsDataHandle;
  
  // return
  TRACE_EVENT( << "Returning "  );
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::setException
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::setException( const KXCEXCEPTION& exc, bool setUserInfo )
{
  intParam( EXC_FLAGS ) = exc.ExceptionFlags;
  
  if (exc.ExceptionFlags & KXCEXC_TIMESTAMP)
  {
    intParam( EXC_SECS ) = exc.TimeStamp.Seconds;
    intParam( EXC_MSECS ) = exc.TimeStamp.Millisecs;
  }
  
  // set KXCEXCROMACONF details
  if (exc.ExceptionFlags & KXCEXC_ROMADATA)
  {
#if defined(AIX)
    setObjName( EXC_ROMA_ELNAME,    exc.RomaConfData.ElementName );
    intParam( EXC_ROMA_ELTYPE )   = exc.RomaConfData.ElementType;
    setObjName( EXC_ROMA_QNAME,     exc.RomaConfData.QueueName );
    setObjName( EXC_ROMA_COMPNAME,  exc.RomaConfData.ComponentName );
    setObjName( EXC_ROMA_SVCNAME,   exc.RomaConfData.ServiceName );
#else
    setObjName( EXC_ROMA_ELNAME,   &exc.RomaConfData.ElementName );
    intParam( EXC_ROMA_ELTYPE )   = exc.RomaConfData.ElementType;
    setObjName( EXC_ROMA_QNAME,    &exc.RomaConfData.QueueName );
    setObjName( EXC_ROMA_COMPNAME, &exc.RomaConfData.ComponentName );
    setObjName( EXC_ROMA_SVCNAME,  &exc.RomaConfData.ServiceName );
#endif
    
    SET_STR_PARAM( EXC_ROMA_NODENAME, exc.RomaConfData.NodeName );
  }

  // set KXCEXCSYSCONF details
  if (exc.ExceptionFlags & KXCEXC_SYSTEMDATA)
  {
    intParam( EXC_SYSENV_TYPE ) = exc.SystemConfData.EnvironmentType;
    
#if defined(AIX)
    setObjName( EXC_SYSDOMAIN_NAME, exc.SystemConfData.DomainName );
    setObjName( EXC_SYSUSER_ID,     exc.SystemConfData.UserId );
    setObjName( EXC_SYSPROG_NAME,   exc.SystemConfData.ProgramName );
#else
    setObjName( EXC_SYSDOMAIN_NAME, &exc.SystemConfData.DomainName );
    setObjName( EXC_SYSUSER_ID,     &exc.SystemConfData.UserId );
    setObjName( EXC_SYSPROG_NAME,   &exc.SystemConfData.ProgramName );
#endif
    
    intParam( EXC_SYSPROCESS_ID ) = exc.SystemConfData.ProcessId;
    intParam( EXC_SYSTHREAD_ID )  = exc.SystemConfData.ThreadId;
  }
  
  if ( setUserInfo )
  {
    // set KXCEXCERRDATA details
    if (exc.ExceptionFlags & KXCEXC_ERRORDATA)
    {
      intParam( EXC_ERRCALL_TYPE ) = exc.ErrorData.CallType;
      setError( exc.ErrorData.Error );
    }
    if (exc.ExceptionFlags & KXCEXC_USERDATA)
    {
      intParam( EXC_USERDATA_LEN ) = exc.UserData.UserBufLen;
      JByteArray jb;
      newByteArrayParam( EXC_USERDATA, exc.UserData.UserBufLen, jb);
      
      jb.copyFrom( exc.UserData.UserBuf.pBuffer, exc.UserData.UserBufLen );
      kxcFreeBuffer( (HKXC)longParam( HCTX ), (PKXCBUFFER)&exc.UserData.UserBuf );
    }
  }
}

//=====================================================================
// Method name : RomaJxOper::getException
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::getException( KXCEXCEPTION& exc )
{
  kxcSetExceptionDefault( &exc );
  exc.ExceptionFlags = intParam( EXC_FLAGS );
  exc.ErrorData.CallType = intParam( EXC_ERRCALL_TYPE );
  getError( exc.ErrorData.Error );
  if (exc.ExceptionFlags & KXCEXC_USERDATA)
  {
    exc.UserData.UserBufLen = exc.UserData.UserBuf.BufLen = intParam( EXC_USERDATA_LEN );
    byteArrayParam( EXC_USERDATA, m_userData );
    exc.UserData.UserBuf.pBuffer = m_userData.mem();
    exc.UserData.UserBuf.Status  = KXCBUFSTAT_USER;    
  }
}

//=====================================================================
// Method name : RomaJxOper::getService
// Description : 
// Return type : PKXCSERVICE 
//---------------------------------------------------------------------
PKXCSERVICE RomaJxOper::getService( KXCSERVICE& svc )
{
  if ( hasObjParam( NAME ) )
  {
    kxcSetServiceDefault( &svc );
#if defined(AIX)
    getObjName( NAME, svc.ServiceName );
#else
    getObjName( NAME, &svc.ServiceName );
#endif

    return &svc;
  }
  return NULL;
}

//=====================================================================
// Method name : RomaJxOper::closeHandle
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::closeHandle()
{
  // prepare hKXC
  HKXC hKxc = (HKXC)longParam( HCTX );
  KXCHANDLE hObj = (KXCHANDLE)longParam( HOBJ );

  // trace entry
  TRACE_ENTRY( RomaJxOper::closeHandle, << TRACE_HEX( hKxc ) 
                                        << TRACE_HEX( hObj ));
  TRACE_EVENT( << "About to call kxcCloseHandle()" );

  if ( kxcCloseHandle( hKxc, &hObj ) )
  {
    TRACE_EVENT( << "Success: " << TRACE_HEX( hObj ));
  }
  else
  {
    TRACE_EVENT( << "Failure: " << TRACE_HEX( hObj ));
  }

  longParam( HOBJ ) = (KXCHANDLE)hObj;
  
  // return
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::txControl
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::txControl( int opCode )
{
  // prepare hKXC
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::txControl, << TRACE_HEX( hKxc ) 
                                      << TRACE( opCode ));
  KXCBOOL b;
  KXCSYNC sync;
  PREP_SYNC( sync );

  if (opCode == JXOP_COMMIT)
  {
    TRACE_EVENT( << "About to call kxcCommit()" );
    b = kxcCommit( hKxc, &sync );
  }
  else
  {
    TRACE_EVENT( << "About to call kxcRollback()" );
    b = kxcRollback( hKxc, &sync );
  }

  if (b)
  {
    TRACE_INT_RETURN( JXRC_OK );
  }
  else
  {
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  }
}

//=====================================================================
// Method name : RomaJxOper::openStorageQ
// Description : 
// Return type : int
//---------------------------------------------------------------------
int RomaJxOper::openStorageQ()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::openStorageQ, << TRACE_HEX(hKxc) );

  KXCSTORQUEUEDEF qDef;
  kxcSetStoreQDefault( &qDef );

#if defined(AIX)
  getObjName( NAME, qDef.QueueName );
  getObjName( MODEL_QUEUE, qDef.ModelQName );
#else
  getObjName( NAME, &qDef.QueueName ); 
  getObjName( MODEL_QUEUE, &qDef.ModelQName );
#endif

  qDef.Options = intParam(OPT);
  TRACE_EVENT( << TRACE(qDef.QueueName)
               << TRACE_HEX(qDef.Options)
             );
  
  HKXCSTORQ hQue;
  TRACE_EVENT( << "About to call kxcOpenStorageQueue" );
  KXCBOOL b = kxcOpenStorageQueue( hKxc, &qDef, &hQue );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  // store context handle
  longParam(HQUEUE) = hQue;

  TRACE_EVENT( << "Success: " << TRACE_HEX(hQue) );
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::closeStorageQ
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::closeStorageQ()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  HKXCSTORQ hQue = (HKXCSTORQ)longParam( HQUEUE );
  TRACE_ENTRY( RomaJxOper::closeStorageQ, << TRACE_HEX(hKxc) 
                                          << TRACE_HEX(hQue));
  
  KXCBOOL b = kxcCloseStorageQueue( hKxc, &hQue );
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  // store context handle
  longParam(HQUEUE) = hQue;

  TRACE_EVENT( << "Success:" << TRACE_HEX(hQue) );
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::deleteStorageQ
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::deleteStorageQ()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::deleteStorageQ, << TRACE_HEX(hKxc) );
  KXCSTORQUEUEDEF qDef;
  kxcSetStoreQDefault( &qDef );

#if defined(AIX)
  getObjName( NAME, qDef.QueueName );
#else
  getObjName( NAME, &qDef.QueueName );
#endif

  TRACE_EVENT( << TRACE_OBJNAME(qDef.QueueName)
               << TRACE_HEX(qDef.Options)
             );

  TRACE_EVENT( << "About to call kxcDeleteStorageQueue" );
  KXCBOOL b = kxcDeleteStorageQueue( hKxc, &qDef );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::interpretMsg
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::interpretMsg()
{
  return JXRC_UNKNOWN_OP;
}

//=====================================================================
// Method name : RomaJxOper::reportError
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::reportError()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::reportError, << TRACE_HEX(hKxc) );
  
  KXCEXCEPTION exc;
  getException( exc );
  
  KXCMSG msg = { KXCMSG_DEFAULT };
  PKXCMSG pMsg = & msg;
  if (pMsg)
  {
    getMsg( msg );
    TRACE_MSG( msg );
  }
  
  KXCEPOPT opt = { KXCEPOPT_DEFAULT };
  opt.ErrorOptions = intParam(OPT);
  opt.DefaultAction = intParam(DEFAULT_ACTIONS);

#if defined(AIX)
  getObjName( PROFILE_NAME, opt.ErrorProfile );
#else
  getObjName( PROFILE_NAME, &opt.ErrorProfile ); 
#endif

  KXCSYNC sync;
  PREP_SYNC( sync );

  HKXCTHEME theme = (HKXCTHEME)longParam(HOBJ);
  
  TRACE_EVENT( << "About to call kxcReportError" << TRACE_HEX(opt.ErrorOptions)
                                                 << TRACE(opt.ErrorProfile)
                                                 << TRACE_HEX(opt.DefaultAction)
                                                 << TRACE_HEX(theme)
             );
  
  if (pMsg)
  {
    getMsg( msg );
    TRACE_MSG( msg );
  }
  
  TRACE_EXCEPTION_USERINFO( exc );
  TRACE_SYNC( sync );
  
  KXCBOOL b = kxcReportError( hKxc, &opt, pMsg, &exc, &sync, &theme );

  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_EVENT( << "Success: " << TRACE_HEX(opt.DefaultAction) 
                              << TRACE_HEX(opt.Actions) 
                              << TRACE_HEX(theme)
             );
  TRACE_EXCEPTION( exc );

  intParam(DEFAULT_ACTIONS) = opt.DefaultAction;
  intParam(ACTIONS) = opt.Actions;
  if (pMsg)
  {
    TRACE_MSG( msg );
    setMsg( hKxc, msg );
  }

  longParam( HOBJ ) = theme;
  setException( exc, false );

  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::writeLog
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::writeLog( bool sendAlert )
{
  TRACE_ENTRY( RomaJxOper::writeLog, << TRACE(sendAlert) );
  KXCLOGDATA alert = { KXCLOGDATA_DEFAULT };

  alert.Severity  = intParam(LOG_SEVERITY);
  alert.Format    = intParam(LOG_FORMAT);
  
  JString userText;
  strParam( LOG_USERTEXT, userText );
  alert.pUserText = (PKXCCHAR)userText();
  
  JByteArray logData;
  byteArrayParam(LOGDATA, logData);
  if (logData.mem())
  {
    alert.DataBuf.pBuffer = logData.mem();
    alert.DataBuf.BufLen  = alert.DataLen = intParam(LOGDATA_LEN);
  }
  
  TRACE_EVENT( << TRACE_HEX(alert.Severity)
               << TRACE_HEX(alert.Format)
               << TRACE(alert.pUserText)
             );
  TRACE_BUFFER( alert.DataBuf, TRACE(alert.DataLen) );
  
  KXCBOOL b;
  if ( sendAlert ) 
  {
    HKXC hKxc = (HKXC)longParam( HCTX );
    TRACE_EVENT( << "About to call kxcSendAlert(): " << TRACE_HEX(hKxc) );
    b = kxcSendAlert( hKxc, &alert );
  } 
  else 
  {
    TRACE_EVENT( << "About to call kxcWriteLog()" );
    b = kxcWriteLog( &alert );
  }

  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );

  TRACE_EVENT( << "Success" );
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirLookup
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirLookup()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirLookup, << TRACE_HEX(hKxc) );
  
  KXCDIRSEARCHREQ dsr = { KXCDIRSEARCHREQ_DEFAULT };
  
  dsr.Options = intParam( DIR_SRC_OPT );
  
  dsr.SearchType = intParam( DIR_SRC_TYPE );
  
  JString jsStartingPoint;
  strParam( DIR_START, jsStartingPoint );
  dsr.StartingPoint = (PKXCCHAR)jsStartingPoint();
  
  JString jsSearchName;
  strParam( DIR_NAME, jsSearchName );
  dsr.SearchName = (PKXCCHAR)jsSearchName();
  
  TRACE_EVENT( << TRACE_HEX(dsr.Options)
               << TRACE_OBJNAME(dsr.StartingPoint)
               << TRACE_HEX(dsr.SearchType)
               << TRACE_OBJNAME(dsr.SearchName)
             );
  
  KXCDIRSEARCHRES dst = { KXCDIRSEARCHRES_DEFAULT };
  
  TRACE_EVENT( << "About to call kxcDirLookUp" );
  KXCBOOL b = kxcDirLookUp(hKxc, &dsr, &dst);
  
  if (b)
  {
    // set return parameters
    intParam( DIR_SRC_MATCHES ) = dst.Matches;
    intParam( DIR_SRC_COUNT )   = dst.ModCount;
  }
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirGetItem
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirGetItem()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirGetItem, << TRACE_HEX(hKxc) );
  
  KXCDIRITEMREQ dgi = { KXCDIRITEMREQ_DEFAULT };
  
  dgi.EntryNumber = intParam( DIR_ITEM_NBR );
  
  TRACE_EVENT( << TRACE_HEX(dgi.EntryNumber)
             );
  
  KXCDIRSEARCHRES DirSrcRes = { KXCDIRSEARCHRES_DEFAULT };
  
  // Provide a KXCBUFFER for result.
  // Empty to force buffer allocation by kxcDirGetItem()
  PKXCBUFFER pkxcbuf = new KXCBUFFER();

  pkxcbuf->BufLen   = 0;
  pkxcbuf->pBuffer  = NULL;
  
  TRACE_EVENT( << "About to call dirGetItem" );
  KXCBOOL b = kxcDirGetItem(hKxc, &dgi, pkxcbuf);
  
  if (b)
    setBuffer( *pkxcbuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirGetModCountFromDir
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirGetModCountFromDir()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirGetModCount, << TRACE_HEX(hKxc) );
  
  KXCULONG lModCount;
  
  TRACE_EVENT( << "About to call kxcDirGetModCount" );
  KXCBOOL b = kxcDirGetModCount(hKxc, &lModCount);
  
  if (b)
    intParam( DIR_SRC_COUNT ) = lModCount;
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirUpdateItem
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirUpdateItem()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirUpdateItem, << TRACE_HEX(hKxc) );
  
  KXCDIRUPDATEOPT duo = { KXCDIRUPDATEOPT_DEFAULT };
  
  duo.Options = intParam( DIR_SRC_OPT );
  
#if defined(AIX)	
  getObjName( DIR_IDENTITY, duo.Id );
  getObjName( DIR_PASSWORD, duo.Password );
#else
  getObjName( DIR_IDENTITY, &duo.Id );
  getObjName( DIR_PASSWORD, &duo.Password );
#endif
  
  KXCBUFFER kxcBuf;
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << TRACE_HEX(duo.Options)
               << TRACE_OBJNAME(duo.Id)
               << TRACE_OBJNAME(duo.Password)
             );
  
  TRACE_EVENT( << "About to call kxcDirUpdateItem" );
  KXCBOOL b = kxcDirUpdateItem(hKxc, &duo, &kxcBuf);
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirDeleteItem
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirDeleteItem()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirDeleteItem, << TRACE_HEX(hKxc) );
  
  KXCDIRUPDATEOPT duo = { KXCDIRUPDATEOPT_DEFAULT };
  
  duo.Options = intParam( DIR_SRC_OPT );
  
#if defined(AIX)	
  getObjName( DIR_IDENTITY, duo.Id );
  getObjName( DIR_PASSWORD, duo.Password );
#else
  getObjName( DIR_IDENTITY, &duo.Id );
  getObjName( DIR_PASSWORD, &duo.Password );
#endif
  
  KXCBUFFER kxcBuf;
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << TRACE_HEX(duo.Options)
               << TRACE_OBJNAME(duo.Id)
               << TRACE_OBJNAME(duo.Password)
             );
  
  TRACE_EVENT( << "About to call kxcDirDeleteItem" );
  KXCBOOL b = kxcDirDeleteItem(hKxc, &duo, &kxcBuf);
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::queryContext
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::queryContext()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::queryContext, << TRACE_HEX(hKxc) );
  
  //valid options are a conbination of
  // KXCCID_SUBNET
  // KXCCID_QNAMEHI
  // KXCCID_QNAMELO
  // KXCCID_APPDEF
  // KXCCID_ELEMENTTYPE
  // KXCCID_DIRHANDLE
  // KXCCID_DIREXITDEF
  KXCULONG  options = KXCCID_ELEMENTTYPE;
  KXCBUFFER info = {KXCBUFFER_DEFAULT};
  
  TRACE_EVENT( << "About to call kxcQueryContext" );
  KXCBOOL b = kxcQueryContext(hKxc, options, &info);
  
  if ( b == TRUE && hKxc != NULL_HKXCSTORQ )
    longParam( HCTX ) = hKxc;
  else
    longParam( HCTX ) = 0;	
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirGetModCountFromItem
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirGetModCountFromItem()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirGetModCountFromItem, << TRACE_HEX(hKxc) );
  
  KXCBOOL b = FALSE;
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_KXC_ERROR );
}

//=====================================================================
// Method name : RomaJxOper::dirCountAttribs
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirCountAttribs()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirCountAttribs, << TRACE_HEX(hKxc) );
  
  KXCBUFFER kxcBuf;
  
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  KXCULONG Count;
  
  TRACE_EVENT( << "About to call kxcDirCountAttributes" );
  KXCBOOL b;
  kxcDirCountAttributes(&kxcBuf, Count, b);
  
  if (b)
    intParam( DIR_SRC_COUNT ) = Count;
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirGetDN
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirGetDN()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirGetDN, << TRACE_HEX(hKxc) );
  
  KXCBUFFER kxcBuf;
  
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << "About to call kxcDirGetDn" );
  
  PKXCCHAR pchar = NULL;
  
  KXCBOOL b;
  kxcDirGetDn(&kxcBuf, pchar, b);
  
  if (b)
    setStrParam( DIR_DIST_NAME, (const char*)pchar );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirGetAttributeAt
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirGetAttributeAt()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirGetAttributeAt, << TRACE_HEX(hKxc) );
  
  KXCULONG nNumAttr = intParam( DIR_ITEM_NBR );
  KXCBUFFER kxcBuf;
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << "About to call dirGetAttributeAt" );
  
  PKXCCHAR pName = NULL;
  PKXCCHAR pValue = NULL;
  KXCBOOL b;
  kxcDirGetAttributeAt(&kxcBuf, nNumAttr,  pName, pValue, b);
  
  if (b)
  {
    setStrParam( DIR_ATTR_NAME, (const char*)pName );
    setStrParam( DIR_ATTR_VALUE, (const char*)pValue );
  }
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirDirGetValue
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirDirGetValue()
{
  
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirDirGetValue, << TRACE_HEX(hKxc) );
  
  KXCULONG nNumAttr = intParam( DIR_ITEM_NBR );
  
  JString jsAttrname;
  strParam( DIR_ATTR_NAME, jsAttrname );
  PKXCCHAR szAttrname = (PKXCCHAR)jsAttrname();
  
  KXCBUFFER kxcBuf;
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << "About to call kxcDirGetValue" );
  
  PKXCCHAR pValue = NULL;
  KXCBOOL b;
  kxcDirGetValue(&kxcBuf, szAttrname, nNumAttr, pValue, b);
  
  if (b)
    setStrParam( DIR_ATTR_VALUE, (const char*)pValue );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirSetDN
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirSetDN()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirSetItemDN, << TRACE_HEX(hKxc) );
  
  JString jsDistName;
  strParam( DIR_DIST_NAME, jsDistName );
  PKXCCHAR szDistName = (PKXCCHAR)jsDistName();
  
  TRACE_EVENT( << TRACE_OBJNAME(szDistName)
             );
  
  KXCBUFFER kxcBuf;
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << "About to call kxcSetDirDn" );
  
  KXCBOOL b;
  kxcSetDirDn(&kxcBuf, szDistName, b);
  
  if (b)
    setBuffer( kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::dirAddAttr
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::dirAddAttr()
{
  HKXC hKxc = (HKXC)longParam( HCTX );
  TRACE_ENTRY( RomaJxOper::dirSetDirItem, << TRACE_HEX(hKxc) );
  
  JString jsAttrName;
  strParam( DIR_ATTR_NAME, jsAttrName );
  PKXCCHAR szAttrName = (PKXCCHAR)jsAttrName();
  
  JString jsAttrValue;
  strParam( DIR_ATTR_VALUE, jsAttrValue );
  PKXCCHAR szAttrValue = (PKXCCHAR)jsAttrValue();
  
  TRACE_EVENT( << TRACE_OBJNAME(szAttrName)
               << TRACE_OBJNAME(szAttrValue)
             );
  
  KXCBUFFER kxcBuf;
  JByteArray jItem;
  getBuffer(jItem, kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  TRACE_EVENT( << "About to call kxcSetDirItem" );
  
  KXCBOOL b;
  kxcSetDirItem(&kxcBuf, szAttrName, szAttrValue, b);
  
  if (b)
    setBuffer( kxcBuf, DIR_ITEM_DETAIL, DIR_ITEM_DETAIL_LENGTH );
  
  // process error if any
  if (!b)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::setSecurityBuffer
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::setSecurityBuffer()
{
  TRACE_ENTRY( RomaJxOper::setSecurityBuffer, << TRACE_HEX(0) );
  
  // For each ILS, a separate data buffer can be defined. The buffers
  // are combined into a single array-like structure (KXCSECDEF) which
  // is then ready to be passed to the KXC API.
  
  // set up initial length - the standard SECDEF structure
  int length = sizeof(KXCSECDEF);
  
  // made this public in RomaJxEnv class so we can use
  int nItems = (m_ipSize - 1) / 2;
  
  // add a KXCSECHDR and relevant data length for each buffer defined
  for (int i = 0; i < nItems; i++)
    length += sizeof(KXCSECHDR) + intParam( i + 1);
  
  // create new security structure and clear it down ...
  KXCSECDEF* psecdef = (KXCSECDEF*) new char[length];
  
  bool success = (psecdef != 0);
  
  if (success) 
  {
    memset(psecdef, 0, length);
    
    // initialise the KXC data structure ...
    // the length will be stored in psecdef->TotalLen
    kxcInitSecDef(psecdef, nItems, length);
    
    // initialize individual security structures. These are effectively
    // array entries (indexed from 0) and appended to KXCSECHDR ...
    for (int i = 0; i < nItems; i++) 
    {
      int ILSNumber = intParam( i + nItems + 1);
      int dataLength = intParam( i + 1);
      
      JByteArray jItem;
      byteArrayParam( i + 1, jItem ); 
      char * pdata = (char *)jItem.mem();
      
      kxcInitSecHdr(psecdef, i, ILSNumber, dataLength, pdata);
    }
  }
  
  if (success)
  {
    JByteArray jb;
    newByteArrayParam( 1, length, jb );
    jb.copyFrom( psecdef, length );
  }
  
  // process error if any
  if (!success)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::getILSBufferSpec
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::getILSBufferSpec()
{
  TRACE_ENTRY( RomaJxOper::getILSBufferSpec, << TRACE_HEX(0) );
  
  HKXCILS handle = (HKXCILS) longParam( ILS_DATA_HANDLE );
  
  int id = intParam( ILS_IDENTITY );
  int type = intParam( ILS_TYPE );
  
  KXCILSOPT opt  = { KXCILSOPT_DEFAULT };
  kxcSetILSType ( &opt, type );
  kxcSetILSMethod( &opt, id );
  
  // retrieve any data for this selection ...
  bool success = (kxcGetILSData( &opt, handle ) != 0 );
  
  int length;
  void *buffer;
  
  if ( success )
  {
    length = kxcGetILSLength( &opt );
    buffer = kxcGetILSBuffer( &opt );
    
    JByteArray jb;
    newByteArrayParam( ILS_BUFFER, length, jb );
    jb.copyFrom( buffer, length );
  }
  
  // process error if any
  if (!success)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::getILSBufferNext
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::getILSBufferNext()
{
  TRACE_ENTRY( RomaJxOper::getILSBufferNext, << TRACE_HEX(0) );
  
  HKXCILS handle = (HKXCILS) longParam( ILS_DATA_HANDLE );
  
  
  KXCILSOPT opt  = { KXCILSOPT_DEFAULT };
  opt.Options    = KXCIOPT_GETNEXT;
  
  // retrieve any data for this selection ...
  bool success = (kxcGetILSData( &opt, handle ) != 0 );
  
  int length, ilsid, type;
  void *buffer;
  
  if ( success )
  {
    length = kxcGetILSLength( &opt );
    buffer = kxcGetILSBuffer( &opt );
    ilsid  = kxcGetILSMethod( &opt );
    type   = kxcGetILSType  ( &opt );
    
    JByteArray jb;
    newByteArrayParam( ILS_BUFFER, length, jb );
    jb.copyFrom( buffer, length );
    
    intParam( ILS_IDENTITY ) = ilsid;
    intParam( ILS_TYPE) = type;
  }
  
  // process error if any
  if (!success)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::setILSBuffer
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::setILSBuffer()
{
  TRACE_ENTRY( RomaJxOper::setILSBuffer, << TRACE_HEX(0) );
  
  HKXCILS handle = (HKXCILS) longParam( ILS_DATA_HANDLE );
  int id = intParam( ILS_IDENTITY );
  int type = intParam( ILS_TYPE );
  int options = intParam( ILS_OPTION );
  
  JByteArray jItem;
  byteArrayParam( ILS_BUFFER, jItem ); 

  char *buffer  = (char *)jItem.mem();
  int   length  = jItem.size();
  
  
  KXCILSOPT opt;
  kxcSetILSDefault(&opt);
  kxcSetILSOptions(&opt, options);
  
  kxcSetILSMethod(&opt,id);
  kxcSetILSType  (&opt,type);
  kxcSetILSBuffer(&opt,length, buffer);
  kxcSetILSLength(&opt,length);
  
  // build up the data definition ...
  bool success = ( kxcSetILSData(&opt, &handle) != 0 );
  
  // may have a new handle value so return
  if ( success )
    longParam( ILS_DATA_HANDLE ) = (HKXCILS)handle;
  
  // process error if any
  if (!success)
    TRACE_INT_RETURN( JXRC_KXC_ERROR );
  
  TRACE_INT_RETURN( JXRC_OK );
}

//=====================================================================
// Method name : RomaJxOper::encoding
// Description : 
// Return type : int 
//---------------------------------------------------------------------
int RomaJxOper::encoding()
{
  return KXCMSGENC_PLATFORM;
}

//=====================================================================
// Method name : RomaJxOper::init
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::init(JNIEnv *env, 
                      jintArray jip, 
                      jobjectArray jop,
                      jlongArray jlp )
{
  RomaJxEnv::init( env, jip, jop, jlp );
}

//=====================================================================
// Method name : RomaJxOper::getMsg
// Description : 
// Return type : void 
// Argument    : KXCMSG& msg
//---------------------------------------------------------------------
void RomaJxOper::getMsg( KXCMSG& msg )
{
  kxcSetMsgDefault( &msg );
  msg.MsgType           = intParam( MSG_TYPE );
  msg.Priority          = intParam( PRIORITY );
  msg.Persistence       = intParam( PERSISTENCE );
  msg.Format            = intParam( FORMAT );
  msg.Feedback          = intParam( FEEDBACK );
  msg.Expiry            = intParam( EXPIRY );
  msg.CodePage          = intParam( CODEPAGE );
  msg.Encoding          = intParam( ENCODING );
  getMsgId( msg.MsgId, MSGID );
  getMsgId( msg.CorrelId, CORRELID );
  
  byteArrayParam( MSGBUF, m_msgBuf );
  msg.MsgBuffer.pBuffer = m_msgBuf.mem();
  msg.MsgLen            = intParam( MSG_LENGTH );
  msg.MsgBuffer.BufLen  = intParam( MSGBUF_SIZE );
  msg.MsgBuffer.BufLen  = msg.MsgLen;
  
  byteArrayParam( TAGBUF, m_tagBuf );
  msg.TagBuffer.pBuffer = m_tagBuf.mem();
  msg.TagLen = intParam( TAG_LENGTH );
  msg.TagBuffer.BufLen  = intParam( TAGBUF_SIZE );
  msg.TagBuffer.BufLen  = msg.TagLen;
  
  msg.Status = intParam( SEC_STATUS );
  msg.BackoutCount      = intParam( BACKOUT_COUNT );

#if defined(AIX)
  getObjName( FORMAT_NAME,  msg.FormatName );
  getObjName( MESSAGE_NAME, msg.MessageName );
#else
  getObjName( FORMAT_NAME,  &msg.FormatName );
  getObjName( MESSAGE_NAME, &msg.MessageName );
#endif

  msg.TimeStamp.Seconds   = intParam(TIMESTAMP_SECS);
  msg.TimeStamp.Millisecs = intParam(TIMESTAMP_MSECS);
}

//=====================================================================
// Method name : RomaJxOper::setMsg
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::setMsg(HKXC hKxc, 
                        const KXCMSG& msg, 
                        bool bAutoAlloc )
{
  TRACE_ENTRY( RomaJxOper::setMsg , << TRACE_HEX(hKxc));
  TRACE_EVENT(<< " setMsg start ");

  intParam(MSG_TYPE)        = msg.MsgType;
  intParam(PERSISTENCE)     = msg.Persistence;
  intParam(PRIORITY)        = msg.Priority;
  intParam(FORMAT)          = msg.Format;
  intParam(FEEDBACK)        = msg.Feedback;
  intParam(EXPIRY)          = msg.Expiry;
  intParam(CODEPAGE)        = msg.CodePage;
  intParam(ENCODING)        = msg.Encoding;
  intParam(MSG_LENGTH)      = msg.MsgLen;
  intParam(TAG_LENGTH)      = msg.TagLen;
  intParam(SEC_STATUS)      = msg.Status;
  intParam(BACKOUT_COUNT)   = msg.BackoutCount;

#if defined(AIX)
  setObjName( FORMAT_NAME,  msg.FormatName );
  setObjName( MESSAGE_NAME, msg.MessageName );
#else
  setObjName( FORMAT_NAME,  &msg.FormatName );
  setObjName( MESSAGE_NAME, &msg.MessageName );
#endif

  intParam(TIMESTAMP_SECS)  = msg.TimeStamp.Seconds;
  intParam(TIMESTAMP_MSECS) = msg.TimeStamp.Millisecs;
  setMsgId( msg.MsgId, MSGID );
  setMsgId( msg.CorrelId, CORRELID );

  TRACE_EVENT(<< "MSG len" << msg.MsgLen);
  TRACE_EVENT(<< "TAG len" << msg.TagLen);
  
  JByteArray jb; 
  // only copy data if buffer was auto-allocated
  if (bAutoAlloc)
  {
    newByteArrayParam( MSGBUF, msg.MsgLen, jb );
    jb.copyFrom( msg.MsgBuffer.pBuffer, msg.MsgLen );
    
    newByteArrayParam( TAGBUF, msg.TagLen, jb );
    jb.copyFrom( msg.TagBuffer.pBuffer, msg.TagLen );
  }
  else
  {
    // without this code the user defined buffer does not
    // receive the returned data
    JByteArray jmsgBuf; 
    byteArrayParam( MSGBUF, jmsgBuf ); 
    jmsgBuf.copyFrom( msg.MsgBuffer.pBuffer, msg.MsgLen );
  }
  
  // only free if auto-allocated
  if (bAutoAlloc)
  {
    kxcFreeBuffer( hKxc, (PKXCBUFFER)&msg.MsgBuffer );
    kxcFreeBuffer( hKxc, (PKXCBUFFER)&msg.TagBuffer );
  }
  
  setStrParam( SUBJECT, (const char*)msg.SubjectName );
  
  TRACE_EVENT(<< " setMsg end ");
}

//=====================================================================
// Method name : RomaJxOper::setError
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::setError( const KXCERROR& err )
{
  // process main result
  intParam( ERR_STATUS )      = err.Status;
  intParam( ERR_MAIN_CC )     = err.Result.LastCompCode;
  intParam( ERR_MAIN_RC )     = err.Result.LastReturnCode;
  intParam( ERR_MAIN_OP )     = err.Result.LastOpType;
  intParam( ERR_MAIN_REASON ) = err.Result.LastReason;
  intParam( ERR_MAIN_POS )    = err.Result.LastPosition;
  intParam( ERR_MAIN_TYPE )   = err.Result.LastErrorType;
  setStrParam( ERR_MAIN_LOC, (const char*)err.Result.LastLocation );

  const int TEXT_LEN = 1024;
  char text[TEXT_LEN];
  KXCULONG textLen;
  kxcFormatError( (PKXCSUBERROR)&err.Result, KXCERROR_FULL, text, &(textLen = TEXT_LEN) );
  setStrParam( ERR_MAIN_FULL_TEXT, text );
  kxcFormatError( (PKXCSUBERROR)&err.Result, KXCERROR_COMPACT, text, &(textLen = TEXT_LEN) );
  setStrParam( ERR_MAIN_SHORT_TEXT, text );
  
  // process sub-result
  if  ( err.Status & KXCERRSTAT_SUB )
  {
    intParam( ERR_SUB_CC )      = err.SubResult.LastCompCode;
    intParam( ERR_SUB_RC )      = err.SubResult.LastReturnCode;
    intParam( ERR_SUB_OP )      = err.SubResult.LastOpType;
    intParam( ERR_SUB_REASON )  = err.SubResult.LastReason;
    intParam( ERR_SUB_POS )     = err.SubResult.LastPosition;
    intParam( ERR_SUB_TYPE )    = err.SubResult.LastErrorType;
    setStrParam( ERR_SUB_LOC, (const char*)err.SubResult.LastLocation );
    kxcFormatError( (PKXCSUBERROR)&err.SubResult, KXCERROR_FULL, text, &(textLen = TEXT_LEN) );
    setStrParam( ERR_SUB_FULL_TEXT, text );
    kxcFormatError( (PKXCSUBERROR)&err.SubResult, KXCERROR_COMPACT, text, &(textLen = TEXT_LEN) );
    setStrParam( ERR_SUB_SHORT_TEXT, text );
  }
}

//=====================================================================
// Method name : RomaJxOper::getError
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::getError( KXCERROR& err )
{
  // process main result
  err.Status = intParam( ERR_STATUS );
  if ( err.Status & KXCERRSTAT_MAIN )
  {
    err.Result.LastCompCode   = intParam( ERR_MAIN_CC );
    err.Result.LastReturnCode = intParam( ERR_MAIN_RC );
    err.Result.LastOpType     = intParam( ERR_MAIN_OP );
    err.Result.LastReason     = intParam( ERR_MAIN_REASON );
    err.Result.LastPosition   = intParam( ERR_MAIN_POS );
    err.Result.LastErrorType  = intParam( ERR_MAIN_TYPE );
    GET_STR_PARAM( ERR_MAIN_LOC, err.Result.LastLocation );
  }

  // process sub-result
  if  ( err.Status & KXCERRSTAT_SUB )
  {
    err.SubResult.LastCompCode    = intParam( ERR_SUB_CC );
    err.SubResult.LastReturnCode  = intParam( ERR_SUB_RC );
    err.SubResult.LastOpType      = intParam( ERR_SUB_OP );
    err.SubResult.LastReason      = intParam( ERR_SUB_REASON );
    err.SubResult.LastPosition    = intParam( ERR_SUB_POS );
    err.SubResult.LastErrorType   = intParam( ERR_SUB_TYPE );
    GET_STR_PARAM( ERR_SUB_LOC, err.SubResult.LastLocation );
  }
}

//=====================================================================
// Method name : RomaJxOper::setBuffer
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::setBuffer(const KXCBUFFER& buf, 
                           int dataId, 
                           int lenId)
{
  JByteArray jb;
  newByteArrayParam( dataId, buf.BufLen, jb );
  
  jb.copyFrom( buf.pBuffer, buf.BufLen );
  intParam( lenId ) = buf.BufLen;
}

//=====================================================================
// Method name : RomaJxOper::getBuffer
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::getBuffer(JByteArray& jItem, 
                           KXCBUFFER& buf, 
                           int dataId, 
                           int lenId)
{
  buf.BufLen = intParam( lenId );
  byteArrayParam( dataId, jItem );
  buf.Status = KXCBUFSTAT_USER;
  buf.pBuffer = jItem.mem();
}

//=====================================================================
// Method name : RomaJxOper::getStrParam
// Description : 
// Return type : void 
//---------------------------------------------------------------------
void RomaJxOper::getStrParam(int id, 
                             KXCCHAR* dest, 
                             int destLen )
{
  JString str;
  strParam( id, str );
  
  if (str.operator const char*() != NULL)
  {
    int cnt;
    cnt = min( destLen-1, (int)strlen(str) );
    str.copy( (char*)dest, cnt );
    dest[cnt] = '\0';
  } 
  else
    *dest = '\0';
}


#if defined(AIX)
  //=====================================================================
  // Method name : RomaJxOper::getObjName
  // Description : 
  // Return type : void 
  //---------------------------------------------------------------------
  void RomaJxOper::getObjName(int id, 
                              KXCOBJNAME &name)
  {
    JString str;
    strParam( id, str );
  
    if (str.operator const char*() != NULL)
      str.copy( (char*)name, KXCOBJECT_NAME_LENGTH );
    else
      *name = '\0';
  }

  //=====================================================================
  // Method name : RomaJxOper::setObjName
  // Description : 
  // Return type : void 
  //---------------------------------------------------------------------
  void RomaJxOper::setObjName( int id, 
                               const KXCOBJNAME &name )
  {
    KXCOBJNAMESZ buf;
    strncpy( (char*)buf, (const char*)name, KXCOBJECT_NAME_LENGTH );
    buf[KXCOBJECT_NAME_LENGTH] = '\0';
    setStrParam( id, (char*)buf );
  }

#else
  //=====================================================================
  // Method name : RomaJxOper::getObjName
  // Description : 
  // Return type : void 
  //---------------------------------------------------------------------
  void RomaJxOper::getObjName(int id, 
                              KXCOBJNAME * pName)
  {
    JString str;
    strParam( id, str );
  
    if (str.operator const char*() != NULL)
      str.copy( (char*)pName, KXCOBJECT_NAME_LENGTH );
    else
      *((char *)pName) = '\0';
  }

  //=====================================================================
  // Method name : RomaJxOper::setObjName
  // Description : 
  // Return type : void 
  //---------------------------------------------------------------------
  void RomaJxOper::setObjName(int id, 
                              const KXCOBJNAME * pName)
  {
    KXCOBJNAMESZ buf;
    strncpy( (char*)buf, (const char*)pName, KXCOBJECT_NAME_LENGTH );
    buf[KXCOBJECT_NAME_LENGTH] = '\0';
    setStrParam( id, (char*)buf );
  }
#endif
