//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcjbnxa.hpp
//
// Description  : KXC Java common native library definitions
//
// Defines      : class RomaJxOper
//
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes.
//--------------------------------------------------------------------

#ifndef __KXCJBNX_HPP
#define __KXCJBNX_HPP

#include <kxc.h>
#include <kxcointa.hpp>

#include "kxcjnia.hpp"

// Operation const intants shared from Java code
const int JXOP_CLJXJXT                  = 0;
const int JXOP_SVRIJXT                  = 1;
const int JXOP_TERMINATE                = 2;
const int JXOP_PUTREQ                   = 3;
const int JXOP_GETREQ                   = 4;
const int JXOP_PUTRESP                  = 5;
const int JXOP_GETRESP                  = 6;
const int JXOP_COMMIT                   = 7;
const int JXOP_ROLLBACK                 = 8;
const int JXOP_GET_LAST_ERROR           = 9;
const int JXOP_FREEBUF                  = 10;
const int JXOP_CLOSE_HANDLE             = 11;
const int JXOP_FORMAT_ERROR             = 12;
const int JXOP_OPEN_STORAGEQ            = 13;
const int JXOP_CLOSE_STORAGEQ           = 14;
const int JXOP_GET_STORAGEQ             = 15;
const int JXOP_PUT_STORAGEQ             = 16;
const int JXOP_DELETEQ                  = 17;
const int JXOP_INTERPRETMSG             = 18;
const int JXOP_PUTEXCEPTION             = 19;
const int JXOP_REPORTERROR              = 20;
const int JXOP_WRITELOG                 = 21;
const int JXOP_SENDALERT                = 22;
const int JXOP_GETEXCEPTION             = 23;
const int JXOP_PUBLISH                  = 24;
const int JXOP_SUBSCRIBE                = 25;
const int JXOP_UNSUBSCRIBE              = 26;

// directory access operations
const int JXOP_DIR_LOOKUP               = 27;
const int JXOP_DIR_GETITEM              = 28;
const int JXOP_DIR_GETDNAME             = 29;
const int JXOP_DIR_CNTATTRIB            = 30;
const int JXOP_DIR_GETATTRAT            = 31;
const int JXOP_DIR_GETVALUE             = 32;
const int JXOP_DIR_GETMODCNT_FROM_ITEM  = 33;
const int JXOP_DIR_GETMODCNT_FROM_DIR   = 34;
const int JXOP_DIR_UPDATE               = 35;
const int JXOP_DIR_DELETE               = 36;
const int JXOP_DIR_SETDN                = 37;
const int JXOP_DIR_ADDATTR              = 38;
const int JXOP_SETSECURITYBUFFER        = 39;

const int JXOP_GETILSBUFFER_SPEC        = 40;
const int JXOP_GETILSBUFFER_NEXT        = 41;
const int JXOP_SETILSBUFFER             = 42;
const int JXOP_QUERY_CONTEXT            = 43;

// Native return codes
const int JXRC_OK                       = 0;
const int JXRC_KXC_ERROR                = 1;
const int JXRC_UNKNOWN_OP               = 2;
const int JXRC_GEN_FAIL                 = 3;

// Parameter identifiers shared from Java code
// common
const int IP_COMM                       = 0;
const int OP_CODE                       = IP_COMM+0;
const int OPT                           = IP_COMM+4;
const int TR_TYPE                       = IP_COMM+5;
const int TR_CTX                        = IP_COMM+7;
const int WAIT_LEN                      = IP_COMM+8;
const int DEFAULT_ACTIONS               = IP_COMM+9;
const int ACTIONS                       = IP_COMM+10;
const int EXC_FLAGS                     = IP_COMM+11;
const int EXC_SECS                      = IP_COMM+12;
const int EXC_MSECS                     = IP_COMM+13;
const int EXC_USERDATA_LEN              = IP_COMM+14;
const int EXC_ERRCALL_TYPE              = IP_COMM+15;
const int EXC_SYSENV_TYPE               = IP_COMM+16;
const int EXC_SYSPROCESS_ID             = IP_COMM+17;
const int EXC_SYSTHREAD_ID              = IP_COMM+18;
const int EXC_ROMA_ELTYPE               = IP_COMM+19;
const int REPORT_MSG                    = IP_COMM+20;

// message
const int IP_MSG                        = 20;
const int MSG_TYPE                      = IP_MSG+1;
const int PRIORITY                      = IP_MSG+2;
const int PERSISTENCE                   = IP_MSG+3;
const int FORMAT                        = IP_MSG+4;
const int FEEDBACK                      = IP_MSG+5;
const int EXPIRY                        = IP_MSG+6;
const int CODEPAGE                      = IP_MSG+7;
const int ENCODING                      = IP_MSG+8;
const int MSGBUF_HANDLE                 = IP_MSG+11;
const int MSGBUF_SIZE                   = IP_MSG+12;
const int MSG_LENGTH                    = IP_MSG+13;
const int TAGBUF_HANDLE                 = IP_MSG+14;
const int TAGBUF_SIZE                   = IP_MSG+15;
const int TAG_LENGTH                    = IP_MSG+16;
const int SEC_STATUS                    = IP_MSG+17;
const int BACKOUT_COUNT                 = IP_MSG+18;
const int TIMESTAMP_SECS                = IP_MSG+19;
const int TIMESTAMP_MSECS               = IP_MSG+20;

// error
const int IP_ERR                        = 40;
const int ERR_STATUS                    = IP_ERR+1;
const int ERR_MAIN_CC                   = IP_ERR+2;
const int ERR_SUB_CC                    = IP_ERR+3;
const int ERR_MAIN_RC                   = IP_ERR+4;
const int ERR_SUB_RC                    = IP_ERR+5;
const int ERR_MAIN_OP                   = IP_ERR+6;
const int ERR_SUB_OP                    = IP_ERR+7;
const int ERR_MAIN_REASON               = IP_ERR+8;
const int ERR_SUB_REASON                = IP_ERR+9;
const int ERR_MAIN_POS                  = IP_ERR+10;
const int ERR_SUB_POS                   = IP_ERR+11;
const int ERR_MAIN_TYPE                 = IP_ERR+12;
const int ERR_SUB_TYPE                  = IP_ERR+13;

const int MSGINFO_STATUS                = IP_ERR+14;
const int WORKFLOW_LENGTH               = IP_ERR+15;
const int INTERPRET_OPT                 = IP_ERR+16;
const int LOGDATA_LEN                   = IP_ERR+17;
const int LOG_SEVERITY                  = IP_ERR+18;
const int LOG_FORMAT                    = IP_ERR+19;

// subject
const int IP_SUB                        = 60;
const int STYLE                         = IP_SUB+1;

// directory access  
const int IP_DIR                        = 61;
const int DIR_SRC_OPT                   = 62;
const int DIR_SRC_TYPE                  = 63;
const int DIR_SRC_MATCHES               = 64;
const int DIR_SRC_COUNT                 = 65;
const int DIR_ITEM_DETAIL_LENGTH        = 66;
const int DIR_ITEM_NBR                  = 67;
const int ILS_IDENTITY                  = 69;
const int ILS_TYPE                      = 70;
const int ILS_OPTION                    = 71;

const int PUTGET_STATUS                 = 72;
const int SUBJECT_STYLE                 = 73;

// strings and byte arrays
const int NAME                          = 0;
const int SECBUF                        = 1;
const int MSGID                         = 3;
const int CORRELID                      = 4;
const int MSGBUF                        = 5;
const int TAGBUF                        = 6;
const int FORMAT_NAME                   = 7;
const int ERR_MAIN_LOC                  = 8;
const int ERR_SUB_LOC                   = 9;
const int ERR_MAIN_FULL_TEXT            = 10;
const int ERR_SUB_FULL_TEXT             = 11;
const int ERR_MAIN_SHORT_TEXT           = 12;
const int ERR_SUB_SHORT_TEXT            = 13;
const int PROFILE_NAME                  = 14;
const int EXC_USERDATA                  = 15;
const int EXC_SYSDOMAIN_NAME            = 16;
const int EXC_SYSUSER_ID                = 17;
const int EXC_SYSPROG_NAME              = 18;
const int EXC_ROMA_ELNAME               = 19;
const int EXC_ROMA_QNAME                = 20;
const int EXC_ROMA_COMPNAME             = 21;
const int EXC_ROMA_SVCNAME              = 22;
const int EXC_ROMA_NODENAME             = 23;
const int WORKFLOW_DATA                 = 24;
const int LOGDATA                       = 25;
const int LOG_USERTEXT                  = 26;
const int API_PROFILE                   = 27;
const int ERROR_PROFILE                 = 28;
const int SUBJECT                       = 29;
const int MESSAGE_NAME                  = 30;

// Directory access
const int DIR_START                     = 31;
const int DIR_NAME                      = 32;
const int DIR_ITEM_DETAIL               = 33;
const int DIR_DIST_NAME                 = 34;
const int DIR_ATTR_NAME                 = 35;
const int DIR_ATTR_VALUE                = 36;
const int DIR_IDENTITY                  = 37;
const int DIR_PASSWORD                  = 38;
const int ILS_BUFFER                    = 39; 

const int MODEL_QUEUE                   = 40;

// Indexes for array of long parameters m_lp[]
const int LP_BASE                       = 0;          // base id
const int HCTX                          = LP_BASE+0;  // Roma context handle
const int HQUEUE                        = LP_BASE+1;  // queue handle
const int HOBJ                          = LP_BASE+2;  // theme handle
const int MSGINFO_HTHEME                = HOBJ;       // uses same slot as theme handle
const int ILS_DATA_HANDLE               = LP_BASE+3;  // inline service data handle


//--------------------------------------------------------------------
// Class: RomaJxOper
//
// Native library operation.
// Is const intructed and initialized on every native call.
// Has no thread context dependent semantics.
// Has an implementation derivation from JxEnv, thus achieving
// VM independency.
//--------------------------------------------------------------------
class RomaJxOper: private RomaJxEnv
{
public:
  
  // performs requested native operation
  int doOper();
  
  // returns native platform encoding
  static int encoding();
  
  // opens JxEnv init semantics (which are different across VMs)
  void init(JNIEnv*, jintArray, jobjectArray, jlongArray);
  
private:
  
  enum CtxType {PUBLISH,SUBSCRIBE,UNSUBSCRIBE,CLIENT,SERVER,STORAGEQ,EXCEPT};
  
  int init( CtxType );
  int put( CtxType );
  int get( CtxType );
  
  int closeHandle();
  
  int txControl( int opCode );
  
  int openStorageQ();
  int closeStorageQ();
  int deleteStorageQ();
  
  int terminate();
  
  int processKxcError();
  
  int interpretMsg();
  int reportError();
  
  int writeLog( bool sendAlert );
  
  int dirLookup();
  int dirGetItem();
  int dirGetModCountFromDir();
  int dirGetModCountFromItem();
  int dirGetDN();
  int dirCountAttribs();
  int dirGetAttributeAt();
  int dirDirGetValue();
  int dirUpdateItem();
  int dirDeleteItem();
  int dirSetDN();
  int dirAddAttr();
  int setSecurityBuffer();
  int getILSBufferSpec();
  int getILSBufferNext();
  int setILSBuffer();
  int queryContext();
  
  void getMsg( KXCMSG& msg );
  void setMsg( HKXC hKxc, const KXCMSG& msg, bool bAutoAlloc = false);
  
  
  void getMsgId( KXCMSGID& msgId, int i );
  void setMsgId( const KXCMSGID& msgId, int i );
  
  void getError( KXCERROR& err );
  void setError( const KXCERROR& err );
  
  void getException( KXCEXCEPTION& exc );
  void setException( const KXCEXCEPTION& exc, bool setUserInfo );
  
  void setBuffer( const KXCBUFFER& buf, int bufId, int lenId );
  void getBuffer( JByteArray& jItem, KXCBUFFER& buf, int dataId, int lenId );
  
#if defined(AIX)
  void getObjName( int id, KXCOBJNAME &name );
  void setObjName( int id, const KXCOBJNAME &name );
#else
  void getObjName( int id, KXCOBJNAME * pName );
  void setObjName( int id, const KXCOBJNAME * pName );
#endif
  
  PKXCCHAR getStrParamPtr( int id );
  
  void getStrParam( int id, KXCCHAR* dest, int destLen );
  
  PKXCSERVICE getService( KXCSERVICE& svc );
  
  JByteArray m_msgBuf, m_tagBuf;
  JByteArray m_userData;
};

//--------------------------------------------------------------------
// End protect against multiple includes.
//--------------------------------------------------------------------
#endif
