//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// KXC EMS API JNI routines
//
//--------------------------------------------------------------------
// $Id: kxcjems.cpp,v 1.1.2.5 2003/10/17 14:04:37 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/Attic/kxcjems.cpp,v $
//--------------------------------------------------------------------


#include "kxcjems.hpp"
#include "kxcjobj.hpp"


TRACE_MODULE_VERSION($Revision: 1.1.2.5 $);

#define TRACE_JBOOL(x) \
    #x << "=" << ((x) == JNI_TRUE ? "TRUE" : "FALSE") << "; "

#define TRACE_JLONG( x ) \
    #x << "=" << HEX << KXCULONG(x) << DEC << "; "

#define TRACE_BUF_DUMP( x, y ) \
    TRACE_DUMP(x,y,<< "Buffer");


// Flags indicating which parts of the theme information to use
// for the theme build
#define KXCJ_THEMEINFO_THEMEID       1
#define KXCJ_THEMEINFO_RETRYCOUNT    2
#define KXCJ_THEMEINFO_WORKFLOW      4
#define KXCJ_THEMEINFO_ETEINFO       8
#define KXCJ_THEMEINFO_TRAILERS      16
#define KXCJ_THEMEINFO_CONFIRMFLAGS  32

// Flags indicating which additional parts of the message information
// to extract
#define KXCJ_INTMSG_THEME            1


#define EH_TRY try {

#define EH_CATCH \
} catch (KxcJavaException) { TRACE_WARNING( << "Java exception occurred" ); \
} catch (...) { TRACE_WARNING( << "Caught unexpected exception" ); \
                KxcJEnv(*env).reportUnexpected(); }


static void reportRomaError(JNIEnv& env)
{
    TRACE_ENTRY(reportRomaError, << "");

    KXCERROR error;
    kxcGetLastError(&error);

    KxcJError je(env, error);

    if (je.throwToJava())
    {
        TRACE_EVENT( << "throwToJava() failed" );
    }

    TRACE_VOID_RETURN;
}


JNIEXPORT jobject JNICALL
    Java_com_candle_roma_kxc_RomaContext_jniQueryTheme(
        JNIEnv* env, jobject, jlong context, jlong theme, jboolean client)
{
    TRACE_ENTRY(jniQueryTheme, << TRACE_JLONG(context) 
                               << TRACE_JLONG(theme)
                               << TRACE_JBOOL(client));

    KXCBUFFER buf = { KXCBUFFER_DEFAULT };
    jobject res = NULL;

    EH_TRY

        if (kxcQueryTheme(context, theme,
            (client == JNI_TRUE ? KXCTID_REQWFLOW : KXCTID_RSPWFLOW) |
            KXCTID_THEMEID | KXCTID_RETRY | KXCTID_PTRAILER |
            KXCTID_ETEINFO | KXCTID_CONFIRM | KXCTID_VERSION_CURR, &buf))
        {
            KxcJThemeInfo info(*env, buf, client);
            res = info.getHandle();
        }
        else
        {
            reportRomaError(*env);
        }

    EH_CATCH

    kxcFreeBuffer(NULL_HKXC, &buf);
    TRACE_RETURN(res);
}


JNIEXPORT jlong JNICALL
    Java_com_candle_roma_kxc_RomaContext_jniBuildTheme(
        JNIEnv* env, jobject, jlong context, jlong theme,
        jobject joThemeInfo, jint options, 
        jboolean close, jboolean client)
{
    TRACE_ENTRY(jniBuildTheme, << TRACE_JLONG(context)
                               << TRACE_JLONG(theme) 
                               << TRACE_JLONG(options) 
                               << TRACE_JBOOL(close)
                               << TRACE_JBOOL(client));

    HKXCTHEME res = NULL_HKXCTHEME;
    KXCBUFFER buf = { KXCBUFFER_DEFAULT };

    EH_TRY

        // copy theme if the original must be preserved

        if (close == JNI_FALSE &&
            !kxcBuildTheme(context, theme, KXCTOPT_COPYTHEME, NULL, &res))
        {
            reportRomaError(*env);
            TRACE_RETURN(0);
        }

        if (joThemeInfo == NULL)
        {
            TRACE_RETURN(res);
        }

        KxcJThemeInfo info(*env, joThemeInfo);

        if (res == NULL_HKXCTHEME)
        {
            res = theme;
        }

        // set retry count

        if ((options & KXCJ_THEMEINFO_RETRYCOUNT) != 0)
        {
            TRACE_EVENT( << "Changing retry count" );

            KXCULONG retry = info.getRetryCount();
            buf.pBuffer = &retry;
            buf.BufLen = sizeof(retry);

            TRACE_EVENT( << TRACE(retry) );

            if (!kxcBuildTheme(context, res,
                KXCTOPT_SETRETRY | KXCTOPT_CLOSETHEME, &buf, &res))
            {
                reportRomaError(*env);
                TRACE_RETURN(0);
            }
        }

        // set confirmation flags

        if ((options & KXCJ_THEMEINFO_CONFIRMFLAGS) != 0)
        {
            TRACE_EVENT( << "Changing confirmation flags" );

            jint cf = info.getConfirmFlags();
            KXCCONFIRMSTATUS conf;
            conf.Version = KXCCONFIRMSTATVERS;
            conf.CoA = ((cf & KXCPOPT_COA) != 0);
            conf.CoD = ((cf & KXCPOPT_COD) != 0);

            buf.pBuffer = &conf;
            buf.BufLen = sizeof(conf);

            TRACE_EVENT( << TRACE(conf.CoA) << TRACE(conf.CoD) );

            if (!kxcBuildTheme(context, res,
                KXCTOPT_SETCONFIRM | KXCTOPT_CLOSETHEME, &buf, &res))
            {
                reportRomaError(*env);
                TRACE_RETURN(0);
            }
        }

        // set theme id

        if ((options & KXCJ_THEMEINFO_THEMEID) != 0)
        {
            TRACE_EVENT( << "Changing theme ID" );

            KxcJByteArray ba = info.getThemeId();

            if (!ba.isNull())
            {
                KXCMSGID themeID;
                ba.getData(themeID, sizeof(themeID));

                buf.pBuffer = themeID;
                buf.BufLen = sizeof(themeID);

                TRACE_BUF_DUMP( buf.pBuffer, buf.BufLen );

                if (!kxcBuildTheme(context, res,
                    KXCTOPT_SETTHEMEID | KXCTOPT_CLOSETHEME, &buf, &res))
                {
                    reportRomaError(*env);
                    TRACE_RETURN(0);
                }
            }
        }

        // set workflow

        if ((options & KXCJ_THEMEINFO_WORKFLOW) != 0)
        {
            TRACE_EVENT( << "Changing workflow" );

            KxcJWorkflow wf = info.getWorkflow();

            if (!wf.isNull())
            {
                KxcJObjectArray routes = wf.getRoutesArray();
                jsize num = routes.getLength();

                TRACE_EVENT( << "Number of routes: " << num );

                if (num)
                {
                    size_t bsize = sizeof(KXCWROUTE) * num + sizeof(KXCWFLOW);

                    TRACE_EVENT( << "Required buffer size: " << bsize );

                    KxcSimplePtr sp(bsize);
                    PKXCWFLOW pFlow = (PKXCWFLOW)sp.get();
                    pFlow->Version = KXCWFLOW_VERSION;
                    pFlow->NumRoutes = num;

                    for (jsize i = 0; i < num; ++i)
                    {
                        KxcJRoute rt(*env, routes.getElement(i));
                        rt.getData(pFlow->Routes[i]);
                    }

                    KxcJWorkflowService serv = wf.getWorkflowService();

                    if (!serv.isNull())
                    {
                        serv.getData(pFlow->Service);
                    }

                    buf.pBuffer = sp.get();
                    buf.BufLen = bsize;

                    TRACE_BUF_DUMP( buf.pBuffer, buf.BufLen );

                    if (!kxcBuildTheme(context, res, KXCTOPT_CLOSETHEME |
                        (client == JNI_TRUE ? KXCTOPT_SETREQWFLOW : KXCTOPT_SETRSPWFLOW),
                        &buf, &res))
                    {
                        reportRomaError(*env);
                        TRACE_RETURN(0);
                    }
                }
            }
        }

        // set trailers

        if ((options & KXCJ_THEMEINFO_TRAILERS) != 0)
        {
            TRACE_EVENT( << "Changing trailers" );

            KxcJObjectArray trailers = info.getTrailersArray();
            jsize num = trailers.getLength();

            TRACE_EVENT( << "Number of trailers: " << num );

            if (num)
            {
                // calculate required buffer size
                jsize i = 0;
                size_t bsize = sizeof(KXCTHEMEPTRLHDR);

                for (; i < num; ++i)
                {
                    KxcJTrailer tr(*env, trailers.getElement(i));
                    bsize += sizeof(KXCTHEMEPTRLDATA) +
                        kxcRoundLength(tr.getData().getLength());
                }

                TRACE_EVENT( << "Required buffer size: " << bsize );

                KxcSimplePtr sp(bsize);
                PKXCTHEMEPTRLHDR pHdr = (PKXCTHEMEPTRLHDR)sp.get();
                pHdr->Version = KXCTHEMEPTRLHDRVERS;
                pHdr->Number = num;

                char* ptr = REINTERPRET_CAST(char*, pHdr + 1);

                for (i = 0; i < num; ++i)
                {
                    KxcJTrailer tr(*env, trailers.getElement(i));
                    PKXCTHEMEPTRLDATA pData = 
                        REINTERPRET_CAST(PKXCTHEMEPTRLDATA, ptr);
                    pData->Version = KXCTHEMEPTRLDATAVERS;
                    pData->Type = tr.getType();
                    pData->Method = tr.getMethod();
                    pData->Status = 0;

                    KxcJByteArray ba = tr.getData();
                    jsize sz = ba.getLength();
                    pData->BufLen = sz;
                    ptr += sizeof(KXCTHEMEPTRLDATA);
                    ba.getData(ptr, sz);
                    ptr += kxcRoundLength(sz);
                }

                buf.pBuffer = sp.get();
                buf.BufLen = bsize;

                TRACE_BUF_DUMP( buf.pBuffer, buf.BufLen );

                if (!kxcBuildTheme(context, res,
                    KXCTOPT_CLOSETHEME | KXCTOPT_SETPTRAILER, &buf, &res))
                {
                    reportRomaError(*env);
                    TRACE_RETURN(0);
                }
            }
        }

        // set ETE info

        if ((options & KXCJ_THEMEINFO_ETEINFO) != 0)
        {
            TRACE_EVENT( << "Changing ETE info" );

            KxcJByteArray ete = info.getEteData();
            jsize sz = ete.getLength();

            TRACE_EVENT( << "Data length: " << sz );

            if (sz)
            {
                buf.pBuffer = ete.getData();
                buf.BufLen = sz;

                TRACE_BUF_DUMP( buf.pBuffer, buf.BufLen );

                if (!kxcBuildTheme(context, res,
                    KXCTOPT_CLOSETHEME | KXCTOPT_SETETEINFO, &buf, &res))
                {
                    reportRomaError(*env);
                    TRACE_RETURN(0);
                }
            }
        }

    EH_CATCH

    TRACE_RETURN(res);
}

JNIEXPORT jobject JNICALL 
    Java_com_candle_roma_kxc_RomaContext_jniInterpretMsg(
        JNIEnv* env, jobject, jlong context, jint options,
        jint type, jint prio, jint pers,
        jint format, jint feedback, jint expiry, jint cp, jint enc,
        jbyteArray mid, jbyteArray cid, jint status, jbyteArray mdata,
        jbyteArray tdata, jint backout, jlong sec, jlong msec,
        jstring fname, jstring sname, jstring mname)
{
    TRACE_ENTRY(jniInterpretMsg, << TRACE_JLONG(context)
                                 << TRACE_JLONG(options));

    jobject res = NULL;

    EH_TRY

        KXCMSG msg = { KXCMSG_DEFAULT };

        msg.MsgType = type;
        msg.Priority = prio;
        msg.Persistence = pers;
        msg.Format = format;
        msg.Feedback = feedback;
        msg.Expiry = expiry;
        msg.CodePage = cp;
        msg.Encoding = enc;
        msg.Status = status;
        msg.BackoutCount = backout;
        msg.TimeStamp.Seconds = sec;
        msg.TimeStamp.Millisecs = msec;

        KxcJByteArray(*env, mid).getData(msg.MsgId, sizeof(msg.MsgId));
        KxcJByteArray(*env, cid).getData(msg.CorrelId, sizeof(msg.CorrelId));

        KxcJByteArray jbaMsg(*env, mdata);
        msg.MsgBuffer.pBuffer = jbaMsg.getData();
        msg.MsgBuffer.BufLen = msg.MsgLen = jbaMsg.getLength();

        KxcJByteArray jbaTag(*env, tdata);
        msg.TagBuffer.pBuffer = jbaTag.getData();
        msg.TagBuffer.BufLen = msg.TagLen = jbaTag.getLength();

        KxcJString(*env, fname).getData(
            msg.FormatName, sizeof(msg.FormatName));

        KxcJString(*env, sname).getData(
            msg.SubjectName, sizeof(msg.SubjectName));

        KxcJString(*env, mname).getData(
            msg.MessageName, sizeof(msg.MessageName));

        TRACE_BUF_DUMP( &msg, sizeof(msg) );

        KXCMSGINFO info = { KXCMSGINFO_DEFAULT };

        if (kxcInterpretMessage(context, options, &msg, &info))
        {
            res = KxcJMsgInfo(*env, info).getHandle();
        }
        else
        {
            reportRomaError(*env);
        }

    EH_CATCH

    TRACE_RETURN(res);
}

JNIEXPORT jlong JNICALL
    Java_com_candle_roma_kxc_RomaContext_jniInsertWorkflow(
        JNIEnv* env, jobject, jlong context, jlong theme,
        jstring str, jboolean close, jboolean replace)
{
    TRACE_ENTRY(jniInsertWorkflow, << TRACE_JLONG(context)
                                   << TRACE_JLONG(theme)
                                   << TRACE_JBOOL(close)
                                   << TRACE_JBOOL(replace));

    HKXCTHEME res = NULL_HKXCTHEME;
    KXCBUFFER buf = { KXCBUFFER_DEFAULT };
    KXCSERVICE serv = { KXCSERVICE_DEFAULT };

    EH_TRY

        KxcJString(*env, str).getData(
            serv.ServiceName, sizeof(serv.ServiceName));

        TRACE_EVENT( << TRACE(serv.ServiceName) );

        buf.pBuffer = &serv;
        buf.BufLen = sizeof(serv);

        if (!kxcBuildTheme(context, theme,
            (close == JNI_TRUE ? KXCTOPT_CLOSETHEME : 0) |
            (replace == JNI_TRUE ? KXCTOPT_REPLACEWFLOW : KXCTOPT_INSERTWFLOW),
            &buf, &res))
        {
            reportRomaError(*env);
        }

    EH_CATCH

    TRACE_RETURN(res);
}

JNIEXPORT jlong JNICALL
    Java_com_candle_roma_kxc_RomaContext_jniCopyTheme(
        JNIEnv* env, jobject, jlong context, jlong theme)
{
    TRACE_ENTRY(jniCopyTheme, << TRACE_JLONG(context) 
                              << TRACE_JLONG(theme));

    HKXCTHEME res = NULL_HKXCTHEME;

    EH_TRY

        if (!kxcBuildTheme(context, theme, KXCTOPT_COPYTHEME, NULL, &res))
        {
            reportRomaError(*env);
        }

    EH_CATCH

    TRACE_RETURN(res);
}

JNIEXPORT jlong JNICALL
    Java_com_candle_roma_kxc_RomaContext_jniMergeEteInfo(
        JNIEnv* env, jobject, jlong context, jlong theme, jbyteArray ete,
        jboolean close)
{
    TRACE_ENTRY(jniMergeEteInfo, << TRACE_JLONG(context)
                                 << TRACE_JLONG(theme)
                                 << TRACE_JLONG(ete)
                                 << TRACE_JBOOL(close));

    HKXCTHEME res = NULL_HKXCTHEME;
    KXCBUFFER buf = { KXCBUFFER_DEFAULT };

    EH_TRY

        KxcJByteArray ba(*env, ete);
        buf.pBuffer = ba.getData();
        buf.BufLen = ba.getLength();

        TRACE_EVENT( << "Data length: " << buf.BufLen );
        TRACE_BUF_DUMP( buf.pBuffer, buf.BufLen );

        if (!kxcBuildTheme(context, theme, KXCTOPT_MERGEETEINFO |
            (close == JNI_TRUE ? KXCTOPT_CLOSETHEME : 0),
            &buf, &res))
        {
            reportRomaError(*env);
        }

    EH_CATCH

    TRACE_RETURN(res);
}

JNIEXPORT void JNICALL
    Java_com_candle_roma_kxc_RomaContext_jniSetBusFlow(
        JNIEnv* env, jobject, jlong context, jstring bflow, jstring oper)
{
    TRACE_ENTRY(jniSetBusFlow, << TRACE_JLONG(context));

    EH_TRY

        KXCBUSFLOW busflow;
        busflow.Version = KXCBUSFLOW_VERSION_1;

        KxcJString(*env, bflow).getData(
            busflow.BusinessFlowName, sizeof(busflow.BusinessFlowName));

        KxcJString(*env, oper).getData(
            busflow.OperationName, sizeof(busflow.OperationName));

        TRACE_EVENT( << TRACE(busflow.BusinessFlowName) );
        TRACE_EVENT( << TRACE(busflow.OperationName) );

        HKXCTHEME res;
        KXCBUFFER buf = { KXCBUFFER_DEFAULT };
        buf.pBuffer = &busflow;
        buf.BufLen = sizeof(busflow);

        if (!kxcBuildTheme(context, NULL_HKXCTHEME,
                KXCTOPT_SETBUSFLOW, &buf, &res))
        {
            reportRomaError(*env);
        }

    EH_CATCH

    TRACE_VOID_RETURN;
}

