//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2001
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 201 North Douglas, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcjni.hpp
//
// Description  : KXC Java native library (JNI)
//
// Defines      : class JString, JByteArray, RomaJxEnv
//
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes.
//--------------------------------------------------------------------
#ifndef __KXCJNI_HPP
#define __KXCJNI_HPP

#include <jni.h>
#include <string.h>

#ifdef __MVS__
#include <jni_convert.h>
#endif

//--------------------------------------------------------------------
// Class: JByteArray
//
// JNI adapter for java byte array object (byte[]).
//--------------------------------------------------------------------
class JByteArray
{
public:
  // JNI specific initilization semantics
  JByteArray();
  
  ~JByteArray();
  
  jbyte* mem();
  
  void copyTo( void* dest, jsize cnt ) const;
  void copyFrom( const void* src, jsize cnt );
  
  jsize size() const;
  
private:
  friend class RomaJxEnv;
  
  void retain( JNIEnv* env, jbyteArray jarr );
  
  // common semantics
  JByteArray( const JByteArray& );
  JByteArray& operator = ( const JByteArray& );
  
  void release();
  
  JNIEnv*     m_env;
  jbyteArray  m_jarr;
  jbyte*      m_arr;
  jsize       m_arrSize;
};

//--------------------------------------------------------------------
// Class: JString
//
// JNI adapter for java String object.
//--------------------------------------------------------------------
class JString
{
public:
  // JNI specific initilization semantics
  JString();
  
  ~JString();
  
  operator const char* () const { return m_str; }
  
  const char * operator () () const { return m_str; }
  
  operator int () const { return m_str != 0; }
  
  jsize length() const;
  
  void copy( char* dest, jsize cnt ) const;
  
private:
  friend class RomaJxEnv;
  
  void retain( JNIEnv* env, jstring jstr );
  
  // common semantics
  JString( const JString& );
  JString& operator = ( const JString& );
  
  void release();
  
  JNIEnv*     m_env;
  jstring     m_jstr;

#ifndef __MVS__
  const
#endif
  char* m_str;
};

//--------------------------------------------------------------------
// Class: RomaJxEnv
//
// RNI adapter for common Java native interface environment used.
// Encapsulates all VM specific details of Java native interface
// programming. This includes initialization , array and object access.
//--------------------------------------------------------------------
class RomaJxEnv
{
public:
  RomaJxEnv();
  ~RomaJxEnv();
  
  void byteArrayParam( jsize i, JByteArray& );
  
  void newByteArrayParam( jsize i, jsize size, JByteArray& );
  
  void strParam( jsize i, JString& );
  
  void setStrParam( jsize i, const char* str );
  
  jint& intParam( jsize i );
  jlong& longParam( jsize i );
  
  void init( JNIEnv *env, jintArray jip, jobjectArray jop, jlongArray jlp );
  
  int hasObjParam( jsize i );
  
private:
  jobject objParam( jsize i );
  
  void setObjParam( jsize i, jobject obj );
  
  JNIEnv*       m_env;
  jintArray     m_jip;
  jobjectArray  m_jop;
  jlongArray    m_jlp;
  
  jint*         m_ip;
  jlong*        m_lp;
  
public:
  jsize         m_ipSize;
  jsize         m_lpSize;
};

//--------------------------------------------------------------------
// End protect against multiple includes.
//--------------------------------------------------------------------
#endif
