/*--------------------------------------------------------------------
 *
 * Copyright 1999, Candle Corporation, A California corporation.
 * All rights reserved.  International copyright secured.

 * This material is proprietary to Candle Corporation and is not to be
 * reproduced, used, or disclosed except in accordance with program
 * licenses or upon written authorization of Candle Corporation.
 *
 *--------------------------------------------------------------------
 * $Id: krrerror.h,v 1.15.12.1 2002/10/03 14:16:44 e123479 Exp $
 * $Source: /romacvs/root/roma/src/krr/krrutil/krrerror.h,v $
 * $Revision: 1.15.12.1 $
 *--------------------------------------------------------------------
 * KRR C/C++ Platform header
 *------------------------------------------------------------------*/

#ifndef __KRRERROR_H
#define __KRRERROR_H

#define KRROP_BASE                      101
#define KRROP_REPOSITORY                KRROP_BASE+0
#define KRROP_DOM                       KRROP_BASE+1
#define KRROP_STL                       KRROP_BASE+2

#define KRRRC_BASE                      21000
#define KRRRC_OBJ_BASE                  (KRRRC_BASE+50)
#define KRRRC_STANZA_BASE               (KRRRC_BASE+100)
#define KRRRC_RDF_BASE                  (KRRRC_BASE+200)
#define KRRRC_DOM_BASE                  (KRRRC_BASE+300)
#define KRRRC_STL_BASE                  (KRRRC_BASE+400)

#define KRRPOS_0                        0L
#define KRRPOS_1                        1L
#define KRRPOS_2                        2L
#define KRRPOS_3                        3L
#define KRRPOS_4                        4L
#define KRRPOS_5                        5L
#define KRRPOS_6                        6L
#define KRRPOS_7                        7L
#define KRRPOS_8                        8L
#define KRRPOS_9                        9L
#define KRRPOS_10                       10L
#define KRRPOS_11                       11L
#define KRRPOS_12                       12L
#define KRRPOS_13                       13L
#define KRRPOS_14                       14L

/* LDAP directory is not accessible */
/* is followed with LDAP error as secondary info */
#define KRRRC_NOT_ACCESSIBLE                (KRRRC_BASE+1)

/********************************************************************
* This error is raised when an object is accessed from the LDAP and
* the access fails due to the following reasons include:
* 1. LDAP stanza is invalid (required attribute not found).
* 2. Object definition is not valid.
* The error Can be followed with secondary error (LDAP, DOM or RDF)
* that describes the exact reason for the failure.
********************************************************************/
#define KRRRC_INVALID_OBJECT                (KRRRC_BASE+2)

/* Object is not found in the LDAP.                                */
/* Can be followed with a secondary LDAP error.                    */
#define KRRRC_UNKNOWN_OBJECT                (KRRRC_BASE+3)

/* This error is raised when an object is added to LDAP but        */
/* another object with the same name already exists.               */
#define KRRRC_OBJECT_ALREADY_EXISTS         (KRRRC_BASE+4)

/* This error is raised when some failure occurs for open LDAP     */
/* connection.  A secondary error describes the details of the     */
/* failure.                                                        */
#define KRRRC_DIRECTORY_UNAVAILABLE         (KRRRC_BASE+5)

/* The error is raised on an attempt to add a duplicated item in   */
/* a group.                                                        */
#define KRRRC_GROUPITEM_DUPLICATED          (KRRRC_BASE+7)

/* The error is raised on an attempt to access an item from a      */
/* group where no such item exists with that name.                 */
#define KRRRC_GROUPITEM_UNKNOWN             (KRRRC_BASE+8)

/* Missing environment variable                                    */
#define KRRRC_ENV_MISSING                   (KRRRC_BASE+9)

/* Invalid environment variable                                    */
#define KRRRC_ENV_INVALID                   (KRRRC_BASE+10)

/* Out of memory error                                             */
#define KRRRC_OUT_OF_MEMORY                 (KRRRC_BASE+11)

/* Invalid wide character error                                    */
#define KRRRC_INVALID_WCHAR                 (KRRRC_BASE+12)

/* Invalid muliti-byte character error                             */
#define KRRRC_INVALID_MBCHAR                (KRRRC_BASE+13)

/* Invalid identifier string condition.                            */
#define KRRRC_INVALID_IDSTR                 (KRRRC_BASE+14)

/* This error is raised when the KrrObject::repImpl() is requested */
/* for an object that was not loaded from repository (e.g. created */
/* by new).  This is internal logic error.                         */
#define KRRRC_DETACHED_OBJECT               (KRRRC_BASE+15)

/* Internal error condition.                                       */
#define KRRRC_INTERNAL_ERROR                (KRRRC_BASE+20)

/* API call input parameters invariant violation                   */
#define KRRRC_API_ERROR                     (KRRRC_BASE+21)

/* Not implemented functionality                                   */
#define KRRRC_NOT_IMPLEMENTED               (KRRRC_BASE+22)

/* RDF parser error: RDF syntax is not valid.                      */
#define KRRRC_RDFPARSER_ERROR               (KRRRC_BASE+23)

/* RDF interpreter error: RDF semantics are not valid.             */
#define KRRRC_RDFLOADER_ERROR               (KRRRC_BASE+24)

/* Attempt to get unbound reference (KrrRef::get() method).        */
#define KRRRC_UNBOUND_REF                   (KRRRC_BASE+25)

/* This error is raised on an attempt to access a method from na   */
/* object definition that has no method by that name exists.       */
#define KRRRC_GROUPITEM_INVINDEX            (KRRRC_BASE+26)

/* Is thrown in the bad cast condition. See KrrAccessor::cast method */
#define KRRRC_BAD_CAST                      (KRRRC_BASE+27)

/* CANDLE_ROMA_HOME is not defined */
#define KRRRC_MISSING_ROMA_HOME             (KRRRC_BASE+28)

/* LDAP/LDIF does not support extended ASCII characters            */
#define KRRRC_NLS_ERROR                     (KRRRC_BASE+29)

/* Shared library error code                                       */
#define KRRRC_SHLIB_ERROR                   (KRRRC_BASE+30)

/* The error is raised on an attempt to add a duplicated method in */
/* the object definition                                           */
#define KRRRC_OBJMETHOD_DUPLICATED          (KRRRC_OBJ_BASE+1)

/* The error is raised on an attempt to access a method from an    */
/* object definition that no method exists with that name.         */
#define KRRRC_OBJMETHOD_UNKNOWN             (KRRRC_OBJ_BASE+2)

/* The error is raised on an attempt to access a method from an    */
/* object definition that no method exists with that index.        */
#define KRRRC_OBJMETHOD_INVINDEX            (KRRRC_OBJ_BASE+3)

#define KRRRC_STL_ERROR                     (KRRRC_STL_BASE+1)

/* This error is raised on an internal failure of assertions in    */
/* the STL code.                                                   */
#define KRRRC_STL_ASSERT                    (KRRRC_STL_BASE+2)

/* The specified feature is not supported in the current version   */
#define KRRRC_STANZA_NOT_SUPPORTED          (KRRRC_STANZA_BASE+1)

/* Invalid LDAP url was specified                                  */
#define KRRRC_STANZA_INVALID_LDAP           (KRRRC_STANZA_BASE+2)

/* Invalid port was specified                                      */
#define KRRRC_STANZA_INVALID_PORT           (KRRRC_STANZA_BASE+3)

/* This error is raised when the entry already exists in LDAP      */
#define KRRRC_STANZA_ALREADY_EXISTS         (KRRRC_STANZA_BASE+4)

/* This error is raised when the stanza's base dn was not found    */
#define KRRRC_STANZA_BASE_DN_NOT_FOUND      (KRRRC_STANZA_BASE+5)

/* This error is raised when the LDAP is not populated             */
/* @issue sdp: this error code is no longer supported.             */
#define KRRRC_NOT_CONFIGURED                (KRRRC_STANZA_BASE+6)

/* Stanza general exception error                                  */
#define KRRRC_STANZA_GENERAL_ERROR          (KRRRC_STANZA_BASE+7)

/* Invalid context record specified                                */
#define KRRRC_STANZA_INVCONTEXT             (KRRRC_STANZA_BASE+8)

/* Invalid (not-existing) LDIF file specified                      */
#define KRRRC_STANZA_INVALID_LDIF           (KRRRC_STANZA_BASE+9)

/* This error is raised when the stanza's base dn was not found    */
#define KRRRC_STANZA_SEARCH_ERROR           (KRRRC_STANZA_BASE+10)

/* DOM exception has been thrown                                   */
#define KRRRC_DOM_API_ERROR                 (KRRRC_DOM_BASE+1)

/* This error is raised when the DOM root element is expected, but */
/* is not found.                                                   */
#define KRRRC_DOM_ROOT_ELEMENT_NOT_EXISTS   (KRRRC_DOM_BASE+2)

/* This error is raised when the attempt to write not ASCII symbol */
/* is detected.                                                    */
#define KRRRC_DOM_NOT_ASCII_SYMBOL          (KRRRC_DOM_BASE+3)

/* The XML Dcoument being parsed is invalid and does meet the W3C  */
/* XML Recommendation.                                             */
#define KRRRC_DOM_INVALID_XML_DOCUMENT      (KRRRC_DOM_BASE+4)

/* Codepage id should be specified in the first column */
#define KRRRC_DOM_WRONG_CPID_POSITION       (KRRRC_DOM_BASE+5)

/* Wrong codepage id */
#define KRRRC_DOM_WRONG_CPID                (KRRRC_DOM_BASE+6)


#endif
