/********************************************************************/
/*                                                                  */
/* Module      : KXC.H                                              */
/*                                                                  */
/* Description : KXC External definitions                           */
/*                                                                  */
/* See end of module for CTVC change history                        */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998,2000                           */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Protect against multiple includes                                */
/********************************************************************/
#ifndef KXC_INCLUDED
#define KXC_INCLUDED

/********************************************************************/
/* Include environment definition                                   */
/********************************************************************/
#include <kxcenv.h>

/********************************************************************/
/* Enable C++ support                                               */
/********************************************************************/
#ifndef LINKAGE_TYPE
  #define LINKAGE_TYPE "C"
#endif

#if defined(__cplusplus)
  extern LINKAGE_TYPE {
#endif

/********************************************************************/
/*                                                                  */
/*                      B A S I C   T Y P E S                       */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Define some basic types                                          */
/********************************************************************/
#if defined(__alpha)
  /* Tru64 Unix has sizeof(long) == sizeof(void*) == 8 !!! */
  typedef unsigned int  KXCBOOL,   *PKXCBOOL;
#else
  typedef unsigned long KXCBOOL,   *PKXCBOOL;
#endif /* defined(__alpha) */
typedef void           KXCVOID,   *PKXCVOID;
typedef unsigned char  KXCCHAR,   *PKXCCHAR;
#if defined(__alpha)
  /* Tru64 Unix has sizeof(long) == sizeof(void*) == 8 !!! */
  typedef unsigned int  KXCULONG,  *PKXCULONG;
#else
  typedef unsigned long KXCULONG,  *PKXCULONG;
#endif /* defined(__alpha) */
typedef unsigned short KXCUSHORT, *PKXCUSHORT;


#ifndef FALSE
#define FALSE (0!=0)
#endif

#ifndef TRUE
#define TRUE  (0==0)
#endif

/********************************************************************/
/* Now define the handles                                           */
/********************************************************************/
#if defined(__alpha)
typedef unsigned long KXCHANDLE,  *PKXCHANDLE;
#define NULL_HKXC       0L
#define NULL_HKXCTHEME  0L
#define NULL_HKXCILS    0L
#else
typedef KXCULONG  KXCHANDLE,  *PKXCHANDLE;
#define NULL_HKXC       (KXCULONG)0L
#define NULL_HKXCTHEME  (KXCULONG)0L
#define NULL_HKXCILS    (KXCULONG)0L
#endif
typedef KXCHANDLE HKXC,       *PHKXC;
typedef KXCHANDLE HKXCTHEME,  *PHKXCTHEME;
typedef KXCHANDLE HKXCILS,    *PHKXCILS;

/********************************************************************/
/* Now define various string types                                  */
/********************************************************************/
#define KXCOBJECT_NAME_LENGTH  132
typedef KXCCHAR KXCOBJNAME[KXCOBJECT_NAME_LENGTH];
typedef KXCOBJNAME *PKXCOBJNAME;
typedef KXCCHAR KXCOBJNAMESZ[KXCOBJECT_NAME_LENGTH+1];
typedef KXCOBJNAMESZ *PKXCOBJNAMESZ;

#define KXCSUBNET_NAME_LENGTH 20
typedef KXCCHAR KXCSUBNETNAME[KXCSUBNET_NAME_LENGTH];

#define KXCSUBJECT_NAME_LENGTH 500
typedef KXCCHAR KXCSUBJNAME[KXCSUBJECT_NAME_LENGTH+1];
typedef KXCSUBJNAME *PKXCSUBJNAME;

#define KXCQCOMP_NAME_LENGTH  48

#define KXCMSG_ID_LENGTH 24
typedef KXCCHAR KXCMSGID[KXCMSG_ID_LENGTH];
typedef KXCMSGID *PKXCMSGID;

#define KXCMSGID_NULL "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"

/********************************************************************/
/* Define the KXC Nodename representation                           */
/********************************************************************/
#define KXCNODENAME_LENGTH_1       16
typedef KXCCHAR KXCNODENAME_1[KXCNODENAME_LENGTH_1];
#define KXCNODENAME_LENGTH_2       64
typedef KXCCHAR KXCNODENAME_2[KXCNODENAME_LENGTH_2];

/********************************************************************/
/* Declare the current version                                      */
/********************************************************************/
typedef KXCNODENAME_2 KXCNODENAME, *PKXCNODENAME;
#define KXCNODENAME_LENGTH KXCNODENAME_LENGTH_2

/********************************************************************/
/* Define the KXC Timestamp definitions                             */
/********************************************************************/
typedef struct
{
  KXCULONG Seconds;
  KXCULONG Millisecs;
} KXCTIMESTAMP, *PKXCTIMESTAMP;

#define KXCTS_INVALID   (0xFFFFFFFF)

/********************************************************************/
/* Now define the string types                                      */
/********************************************************************/
#define KXC_CHARSET_ASCII    0
#define KXC_CHARSET_EBCDIC   1

/********************************************************************/
/* Define the KXC Call Types                                        */
/********************************************************************/
#define KXCCALLTYPE_APPLICATION          0
#define KXCCALLTYPE_PUTREQUEST           1
#define KXCCALLTYPE_GETREQUEST           2
#define KXCCALLTYPE_PUTRESPONSE          3
#define KXCCALLTYPE_GETRESPONSE          4
#define KXCCALLTYPE_BRIDGEPUT            5
#define KXCCALLTYPE_BRIDGEGET            6
#define KXCCALLTYPE_PUTEXCEPTION         7
#define KXCCALLTYPE_INITIALIZE           8
#define KXCCALLTYPE_TERMINATE            9
#define KXCCALLTYPE_COMMIT              10
#define KXCCALLTYPE_ROLLBACK            11
#define KXCCALLTYPE_FREEBUFFER          12
#define KXCCALLTYPE_CLOSEHANDLE         13
#define KXCCALLTYPE_INTERPRETMSG        14
#define KXCCALLTYPE_WRITELOG            15
#define KXCCALLTYPE_SENDALERT           16
#define KXCCALLTYPE_OPENSTORAGEQ        17
#define KXCCALLTYPE_CLOSESTORAGEQ       18
#define KXCCALLTYPE_DELETESTORAGEQ      19
#define KXCCALLTYPE_PUTTOSTORAGEQ       20
#define KXCCALLTYPE_GETFROMSTORAGEQ     21
#define KXCCALLTYPE_QUERYTHEME          22
#define KXCCALLTYPE_BUILDTHEME          23
#define KXCCALLTYPE_PURGEQTOEMS         24
#define KXCCALLTYPE_REPORTERROR         25
#define KXCCALLTYPE_QUERYCONTEXT        26
#define KXCCALLTYPE_SETLDAPSERVICE      27
#define KXCCALLTYPE_SETILSDATA          28
#define KXCCALLTYPE_GETILSDATA          29
#define KXCCALLTYPE_DIRLOOKUP           30
#define KXCCALLTYPE_DIRGETDN            31
#define KXCCALLTYPE_DIRUPDATEITEM       32
#define KXCCALLTYPE_DIRDELETEITEM       33
#define KXCCALLTYPE_DIRGETMODCOUNT      34
#define KXCCALLTYPE_GETBUFFER           35
#define KXCCALLTYPE_XPORTDATA           36
#define KXCCALLTYPE_QMGRCONNECT         37

#define KXCCALLTYPE_NUMCALLS            38

#define KXCCALLTYPE_OMBASE             500

/********************************************************************/
/*                                                                  */
/*                           B U F F E R                            */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Buffer structure                              */
/********************************************************************/
typedef struct
{
  KXCULONG   Status;
  KXCULONG   BufLen;
  PKXCVOID   pBuffer;
} KXCBUFFER, *PKXCBUFFER;

#define KXCBUFSTAT_USER  0
#define KXCBUFSTAT_KXC   1

#define KXCBUFFER_DEFAULT     \
  KXCBUFSTAT_USER, 0, NULL

/********************************************************************/
/*                                                                  */
/*                S E R V I C E   S T R U C T U R E                 */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Service structure                             */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCOBJNAME ServiceName;
  KXCULONG   Options;
} KXCSERVICE, *PKXCSERVICE;

#define KXCSIG_KXCSERVICE  0x42585356

#define KXCSERVOPT_NONE  0

#define KXCVER_SERVICE_1     1
#define KXCVER_SERVICE_CURR  KXCVER_SERVICE_1

#define KXCSERVICE_DEFAULT     \
  KXCSIG_KXCSERVICE, KXCVER_SERVICE_CURR, "", KXCSERVOPT_NONE

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetServiceDefault(pService)           \
{                                                \
  KXCSERVICE mSV = { KXCSERVICE_DEFAULT };       \
  *(pService) = mSV;                             \
}

/********************************************************************/
/*                                                                  */
/*              S E C U R I T Y   S T R U C T U R E                 */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Security definition structures                */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   NumStrucs;
  KXCULONG   TotalLen;
} KXCSECDEF, *PKXCSECDEF;

#define KXCSIG_KXCSECDEF  0x42585344

#define KXCVER_SECDEF_1     1
#define KXCVER_SECDEF_CURR  KXCVER_SECDEF_1

#define KXCSECDEF_DEFAULT     \
  KXCSIG_KXCSECDEF, KXCVER_SECDEF_CURR, 0, sizeof(KXCSECDEF)

typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   Method;
  KXCULONG   DataLen;
} KXCSECHDR, *PKXCSECHDR;

#define KXCSIG_KXCSECHDR  0x42585348

#define KXCVER_SECHDR_1     1
#define KXCVER_SECHDR_CURR  KXCVER_SECHDR_1

#define KXCSECHDR_DEFAULT     \
  KXCSIG_KXCSECHDR, KXCVER_SECHDR_CURR, 0, sizeof(KXCSECHDR)

/********************************************************************/
/* Define the rounding macro                                        */
/********************************************************************/
#define kxcRoundLength(mLen)                                         \
 (((mLen) + (sizeof(KXCULONG)-1))                                    \
     & ~(sizeof(KXCULONG)-1))

/********************************************************************/
/* Now define the KXC SecDef Access macros                          */
/********************************************************************/
#define kxcInitSecDef(mpSecDef, mNumStrcs, mTotLen) \
{                                                   \
PKXCSECDEF mpSecCtl = (PKXCSECDEF) (mpSecDef);      \
  (mpSecCtl)->Signature = KXCSIG_KXCSECDEF;         \
  (mpSecCtl)->Version   = KXCVER_SECDEF_CURR;       \
  (mpSecCtl)->NumStrucs = (mNumStrcs);              \
  (mpSecCtl)->TotalLen  = (mTotLen);                \
}

#define kxcInitSecHdr(mpSecDef, mNumStrc, mMethod,           \
                      mDataLen, mpData)                      \
{                                                            \
PKXCSECDEF mpSecCtl = (PKXCSECDEF) (mpSecDef);               \
PKXCSECHDR mpSecHdr = (PKXCSECHDR) (((PKXCCHAR)(mpSecDef)) + \
                                    sizeof(KXCSECDEF));      \
KXCULONG   i;                                                \
  for(i=0;i<(mNumStrc);i++)                                  \
    mpSecHdr = (PKXCSECHDR) (((PKXCCHAR) (mpSecHdr+1))       \
                  + kxcRoundLength(mpSecHdr->DataLen));      \
  mpSecHdr->Signature = KXCSIG_KXCSECHDR;                    \
  mpSecHdr->Version   = KXCVER_SECHDR_CURR;                  \
  mpSecHdr->Method    = (mMethod);                           \
  mpSecHdr->DataLen   = (mDataLen);                          \
  memcpy(mpSecHdr+1, (mpData), (mDataLen));                  \
}

/********************************************************************/
/*                                                                  */
/*          A P P L I C A T I O N   D E F I N I T I O N             */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Application Definition structure              */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCOBJNAME ElementName;
  PKXCSECDEF pSecurity;
  KXCULONG   Options;
} KXCAPPDEF_1, *PKXCAPPDEF_1;

#define KXCVER_APPDEF_1     1

typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCOBJNAME ElementName;
  PKXCSECDEF pSecurity;
  KXCULONG   Options;
  KXCOBJNAME ModelQName;
  KXCOBJNAME InitProfile;
} KXCAPPDEF_2, *PKXCAPPDEF_2;

#define KXCVER_APPDEF_2     2

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCAPPDEF  0x42584144

#define KXCAPPOPT_NONE              0x00000000
#define KXCAPPOPT_EXCLUSIVE         0x00000001
#define KXCAPPOPT_NON_XACTIONAL     0x00000002
#define KXCAPPOPT_MQSERIES_XACTION  0x00000004
#define KXCAPPOPT_ROLLBK_OS_XACT    0x00000010

#define KXCAPPOPT_USEDEFAULT        0x40000000

#define KXCAPPDEF_DEFAULT     \
  KXCSIG_KXCAPPDEF, KXCVER_APPDEF_CURR, "", NULL, \
  KXCAPPOPT_NONE, "", ""

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCAPPDEF_2         KXCAPPDEF, *PKXCAPPDEF;
#define KXCVER_APPDEF_CURR  KXCVER_APPDEF_2

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetAppDefDefault(pAppDef)           \
{                                              \
  KXCAPPDEF mAD = { KXCAPPDEF_DEFAULT };       \
  *(pAppDef) = mAD;                            \
}

/********************************************************************/
/*                                                                  */
/*              P U B / S U B   D E F I N I T I O N S               */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Publish Subscribe Subject structure           */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   Style;
  KXCULONG   DataLen;
  PKXCCHAR   pSubject;
} KXCSUBJECT, *PKXCSUBJECT;

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCSUBJECT  0x5843534A

#define KXCSUBJECTSTYLE_NONE           0
#define KXCSUBJECTSTYLE_TIBRENDEZVOUS  1
#define KXCSUBJECTSTYLE_MQSERIES       2
#define KXCSUBJECTSTYLE_ROMA           3

#define KXCVER_SUBJECT_1     1
#define KXCVER_SUBJECT_CURR  KXCVER_SUBJECT_1

#define KXCSUBJECT_DEFAULT                \
  KXCSIG_KXCSUBJECT, KXCVER_SUBJECT_CURR, \
  KXCSUBJECTSTYLE_NONE, 0, NULL

/********************************************************************/
/* Now define the KXC Publish Subscribe List structure              */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   NumSubjects;
  KXCULONG   TotalLen;
  KXCSUBJECT aSubject;
} KXCPSLIST, *PKXCPSLIST;

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCPSLIST  0x5843504C

#define KXCVER_PSLIST_1     1
#define KXCVER_PSLIST_CURR  KXCVER_PSLIST_1

#define KXCPSLIST_DEFAULT                                     \
  KXCSIG_KXCPSLIST, KXCVER_PSLIST_CURR, 0,                   \
  (sizeof(KXCPSLIST) - sizeof(KXCSUBJECT)),                   \
  { KXCSUBJECT_DEFAULT }

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetPSListDefault(pPSList)           \
{                                              \
  KXCPSLIST mPSL = { KXCPSLIST_DEFAULT };      \
  *(pPSList) = mPSL;                           \
}

/********************************************************************/
/*                                                                  */
/*                  M E S S A G E   C O N T R O L                   */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Message Control structure                     */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   MsgType;
  KXCULONG   Priority;
  KXCULONG   Persistence;
  KXCULONG   Format;
  KXCULONG   Feedback;
  KXCULONG   Expiry;
  KXCULONG   CodePage;
  KXCULONG   Encoding;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
  KXCULONG   Status;
  KXCULONG   MsgLen;
  KXCBUFFER  MsgBuffer;
  KXCULONG   TagLen;
  KXCBUFFER  TagBuffer;
} KXCMSG_1, *PKXCMSG_1;

#define KXCVER_MSG_1     1

typedef struct
{
  KXCULONG     Signature;
  KXCULONG     Version;
  KXCULONG     MsgType;
  KXCULONG     Priority;
  KXCULONG     Persistence;
  KXCULONG     Format;
  KXCULONG     Feedback;
  KXCULONG     Expiry;
  KXCULONG     CodePage;
  KXCULONG     Encoding;
  KXCMSGID     MsgId;
  KXCMSGID     CorrelId;
  KXCULONG     Status;
  KXCULONG     MsgLen;
  KXCBUFFER    MsgBuffer;
  KXCULONG     TagLen;
  KXCBUFFER    TagBuffer;
  KXCULONG     BackoutCount;
  KXCTIMESTAMP TimeStamp;
  KXCOBJNAME   FormatName;
} KXCMSG_2, *PKXCMSG_2;

#define KXCVER_MSG_2     2

typedef struct
{
  KXCULONG     Signature;
  KXCULONG     Version;
  KXCULONG     MsgType;
  KXCULONG     Priority;
  KXCULONG     Persistence;
  KXCULONG     Format;
  KXCULONG     Feedback;
  KXCULONG     Expiry;
  KXCULONG     CodePage;
  KXCULONG     Encoding;
  KXCMSGID     MsgId;
  KXCMSGID     CorrelId;
  KXCULONG     Status;
  KXCULONG     MsgLen;
  KXCBUFFER    MsgBuffer;
  KXCULONG     TagLen;
  KXCBUFFER    TagBuffer;
  KXCULONG     BackoutCount;
  KXCTIMESTAMP TimeStamp;
  KXCOBJNAME   FormatName;
  KXCSUBJNAME  SubjectName;
  KXCOBJNAME   MessageName;
} KXCMSG_3, *PKXCMSG_3;

#define KXCVER_MSG_3     3

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCMSG     0x42584D44

#define KXCMSG_USEDEFAULT_VALUE  ((KXCULONG)-2)

#define KXCMSGTYPE_DATAGRAM        0x00000000
#define KXCMSGTYPE_REQUEST         0x00000001
#define KXCMSGTYPE_REPLY           0x00000002
#define KXCMSGTYPE_NOTIFY          0x00000004
#define KXCMSGTYPE_POSITIVE        0x00000008
#define KXCMSGTYPE_NEGATIVE        0x00000000
#define KXCMSGTYPE_COA             0x00000010
#define KXCMSGTYPE_COD             0x00000000
#define KXCMSGTYPE_CONFIRMATION    0x00000020
#define KXCMSGTYPE_MASK            0x0000003F
#define KXCMSGTYPE_PUBLICATION     0x00000040
#define KXCMSGTYPE_POSITIVE_PUBSUB 0x00000080
#define KXCMSGTYPE_NEGATIVE_PUBSUB 0x00000100
#define KXCMSGTYPE_ADVISORY        0x00000200

#define KXCMSGTYPE_POSITIVE_COA	 KXCMSGTYPE_CONFIRMATION | \
                                 KXCMSGTYPE_POSITIVE |     \
                                 KXCMSGTYPE_COA
#define KXCMSGTYPE_NEGATIVE_COA	 KXCMSGTYPE_CONFIRMATION | \
                                 KXCMSGTYPE_NEGATIVE |     \
                                 KXCMSGTYPE_COA
#define KXCMSGTYPE_POSITIVE_COD	 KXCMSGTYPE_CONFIRMATION | \
                                 KXCMSGTYPE_POSITIVE |     \
                                 KXCMSGTYPE_COD
#define KXCMSGTYPE_NEGATIVE_COD	 KXCMSGTYPE_CONFIRMATION | \
                                 KXCMSGTYPE_NEGATIVE |     \
                                 KXCMSGTYPE_COD

#define KXCMSGPRI_DEFAULT      ((KXCULONG)-1)
#define KXCMSGPRI_USEDEFAULT   KXCMSG_USEDEFAULT_VALUE

#define KXCMSGPERSIST_YES          1
#define KXCMSGPERSIST_NO           0
#define KXCMSGPERSIST_USEDEFAULT   KXCMSG_USEDEFAULT_VALUE

#define KXCMSGFORMAT_BINARY           1
#define KXCMSGFORMAT_STRING           2
#define KXCMSGFORMAT_BINARY_SIGN      3
#define KXCMSGFORMAT_STRING_SIGN      4
#define KXCMSGFORMAT_BINARY_ENC       5
#define KXCMSGFORMAT_STRING_ENC       6
#define KXCMSGFORMAT_BINARY_SIGN_ENC  7
#define KXCMSGFORMAT_STRING_SIGN_ENC  8

#define KXCMSGFORMAT_USER             65536
#define KXCMSGFORMAT_USENAME          ((KXCULONG)-1)
#define KXCMSGFORMAT_USEDEFAULT       KXCMSG_USEDEFAULT_VALUE

#define KXCMSGFBCK_NONE         0
#define KXCMSGFBCK_ACK          1
#define KXCMSGFBCK_NACK        64
#define KXCMSGFBCK_BAD_DEST    65
#define KXCMSGFBCK_Q_FULL      66
#define KXCMSGFBCK_EXPIRED     67
#define KXCMSGFBCK_ALLOC_ERR   68
#define KXCMSGFBCK_USEDEFAULT  KXCMSG_USEDEFAULT_VALUE

#define KXCMSGEXP_NEVER          ((KXCULONG)-1)
#define KXCMSGEXP_USEDEFAULT     KXCMSG_USEDEFAULT_VALUE

#define KXCMSGCODEPAGE_USER        ((KXCULONG)-1)
#define KXCMSGCODEPAGE_USEDEFAULT  KXCMSG_USEDEFAULT_VALUE

#define KXCMSGENC_DEFAULT        KXCMSGENC_PLATFORM
#define KXCMSGENC_NORMAL         0
#define KXCMSGENC_REVERSED       1
#define KXCMSGENC_USEDEFAULT     KXCMSG_USEDEFAULT_VALUE

#define KXCMSGSTAT_NONE          0x00000000
#define KXCMSGSTAT_ANONYMOUS     KXCMSGSTAT_NONE
#define KXCMSGSTAT_AUTHENTICATED 0x00000001
#define KXCMSGSTAT_NATIVE        0x00000002
#define KXCMSGSTAT_OK_TO_RESPOND 0x00000004
#define KXCMSGSTAT_OK_TO_NOTIFY  0x00000008

#define KXCMSGBACKOUT_UNSUPPORTED  ((KXCULONG)-1)

#define KXCMSGMAX_LEN       0x7FFFFFFF
#define KXCMSGMAX_TAGLEN    0xFFFF

#define KXCMSGLEN_UNKNOWN   ((KXCULONG) -1)

#define KXCMSG_DEFAULT                                       \
  KXCSIG_KXCMSG, KXCVER_MSG_3, KXCMSGTYPE_DATAGRAM,          \
  KXCMSGPRI_USEDEFAULT, KXCMSGPERSIST_USEDEFAULT,            \
  KXCMSGFORMAT_USEDEFAULT, KXCMSGFBCK_USEDEFAULT,            \
  KXCMSGEXP_USEDEFAULT,                                      \
  KXCMSGCODEPAGE_USEDEFAULT, KXCMSGENC_USEDEFAULT,           \
  KXCMSGID_NULL, KXCMSGID_NULL, 0,                           \
  0, { KXCBUFFER_DEFAULT }, 0, { KXCBUFFER_DEFAULT },        \
  KXCMSGBACKOUT_UNSUPPORTED, { 0, 0 }, "", "", ""

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCMSG_3          KXCMSG, *PKXCMSG;
#define KXCVER_MSG_CURR   KXCVER_MSG_3

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetMsgDefault(pMsg)                \
{                                             \
  KXCMSG mMC = { KXCMSG_DEFAULT };            \
  *(pMsg) = mMC;                              \
}

#define kxcSetMsgBuffer(pMsgCtl, mBufLen, mpBuff)\
  if ((pMsgCtl)->MsgBuffer.Status                \
                 == KXCBUFSTAT_KXC)              \
    { kxcFreeBuffer(NULL_HKXC,                   \
                    &((pMsgCtl)->MsgBuffer)); }; \
  (pMsgCtl)->MsgBuffer.pBuffer = (mpBuff);       \
  (pMsgCtl)->MsgBuffer.BufLen  = (mBufLen);      \
  (pMsgCtl)->MsgBuffer.Status  = KXCBUFSTAT_USER;

#define kxcSetTagBuffer(pMsgCtl, mBufLen, mpBuff)\
  if ((pMsgCtl)->TagBuffer.Status                \
                 == KXCBUFSTAT_KXC)              \
    { kxcFreeBuffer(NULL_HKXC,                   \
                    &((pMsgCtl)->TagBuffer)); }; \
  (pMsgCtl)->TagBuffer.pBuffer = (mpBuff);       \
  (pMsgCtl)->TagBuffer.BufLen  = (mBufLen);      \
  (pMsgCtl)->TagBuffer.Status  = KXCBUFSTAT_USER;

#define kxcSetMsgFormatName(pMsgCtl, mName)      \
        (pMsgCtl)->Format =KXCMSGFORMAT_USENAME; \
        strncpy((char *)(pMsgCtl)->FormatName,   \
                ((mName) == NULL)                \
                   ? "" : (char *)(mName),       \
                KXCOBJECT_NAME_LENGTH)

/********************************************************************/
/*                                                                  */
/*                  E R R O R   D E F I N I T I O N                 */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the error structure                                   */
/********************************************************************/
#define KXCFORMAT_BUFFER_LEN 200

typedef struct
{
  KXCULONG   LastCompCode;
  KXCULONG   LastReturnCode;
  KXCULONG   LastOpType;
  KXCULONG   LastReason;
} KXCERRDTL, *PKXCERRDTL;

typedef struct
{
  KXCULONG   LastCompCode;
  KXCULONG   LastReturnCode;
  KXCULONG   LastOpType;
  KXCULONG   LastReason;
  KXCULONG   LastPosition;
  KXCULONG   LastErrorType;
  KXCCHAR    LastLocation[KXCFORMAT_BUFFER_LEN];
} KXCSUBERROR, *PKXCSUBERROR;

#define KXCSUBERROR_DEFAULT      \
  0, 0, 0, 0, 0, 0, ""

typedef struct
{
  KXCULONG    Status;
  KXCSUBERROR Result;
  KXCSUBERROR SubResult;
} KXCERROR, *PKXCERROR;

#define KXCERRSTAT_NONE    0
#define KXCERRSTAT_MAIN    1
#define KXCERRSTAT_SUB     2

/********************************************************************/
/* Now define the flag values for formatting an error               */
/********************************************************************/
#define KXCERROR_COMPACT     0
#define KXCERROR_FULL        1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define KXCERROR_DEFAULT                     \
  KXCERRSTAT_NONE, { KXCSUBERROR_DEFAULT },  \
  { KXCSUBERROR_DEFAULT }

/********************************************************************/
/*                                                                  */
/*                      I L S   D E T A I L S                       */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC ILSData details structure                     */
/********************************************************************/
typedef struct
{
  HKXCILS hToILS;
  HKXCILS hFromILS;
} KXCILSDTL, *PKXCILSDTL;

#define KXCILSDTL_DEFAULT      \
  NULL_HKXCILS, NULL_HKXCILS

/********************************************************************/
/*                                                                  */
/*                      G E T   O P T I O N S                       */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Get options structure                         */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   ReceiveOptions;
  KXCULONG   WaitLen;
  PKXCSECDEF pSecurity;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
  KXCULONG   Signal;
} KXCGETOPT_1, *PKXCGETOPT_1;

#define KXCVER_GETOPT_1     1

typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   ReceiveOptions;
  KXCULONG   WaitLen;
  PKXCSECDEF pSecurity;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
  KXCULONG   Signal;
  KXCERRDTL  Error;
  KXCOBJNAME APIProfile;
  KXCOBJNAME ErrorProfile;
} KXCGETOPT_2, *PKXCGETOPT_2;

#define KXCVER_GETOPT_2     2

typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   ReceiveOptions;
  KXCULONG   WaitLen;
  PKXCSECDEF pSecurity;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
  KXCULONG   Signal;
  KXCERRDTL  Error;
  KXCOBJNAME APIProfile;
  KXCOBJNAME ErrorProfile;
  KXCULONG   SubjectStyle;
  KXCILSDTL  ILSData;
  KXCULONG   Status;
  KXCULONG   ForceOn;
  KXCULONG   ForceOff;
} KXCGETOPT_3, *PKXCGETOPT_3;

#define KXCVER_GETOPT_3     3

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCGETOPT  0x4258474F

#define KXCGOPT_NONE           0x00000000
#define KXCGOPT_USEMSGID       0x00000001
#define KXCGOPT_USECORRELID    0x00000002
#define KXCGOPT_WAIT           0x00000008
#define KXCGOPT_USETHEME       0x00000010
#define KXCGOPT_ANYTHEME       0x00000020
#define KXCGOPT_RETAINTHEME    0x00000040
#define KXCGOPT_CLOSETHEME     0x00000080
#define KXCGOPT_AUTO_ALLOC     0x00000100
#define KXCGOPT_PEEK_FIRST     0x00000200
#define KXCGOPT_PEEK_NEXT      0x00000400
#define KXCGOPT_PEEK_SAME      0x00000800
#define KXCGOPT_GET_PEEKED     0x00001000
#define KXCGOPT_ALLOWNATIVE    0x00002000
#define KXCGOPT_USESIGNAL      0x00004000
#define KXCGOPT_NOEMS          0x00008000
#define KXCGOPT_NONOTIFY       0x00010000
#define KXCGOPT_ALLOWTERMINATE 0x00020000
#define KXCGOPT_NOERRORPROF    0x00040000

#define KXCGOPT_USEDEFAULT    0x40000000

#define KXCGOWAIT_INDEFINITE  ((KXCULONG)(-1))
#define KXCGOWAIT_USEDEFAULT  ((KXCULONG)(-2))

#define KXCGOPTSTAT_NONE      0x00000000
#define KXCGOPTSTAT_TERMINATE 0x00000001

#define KXCGETOPT_DEFAULT                   \
  KXCSIG_KXCGETOPT, KXCVER_GETOPT_3,        \
  KXCGOPT_USEDEFAULT, KXCGOWAIT_USEDEFAULT, \
  NULL, KXCMSGID_NULL, KXCMSGID_NULL, 0,    \
  { 0, 0, 0, 0}, "", "", 0,                 \
  { KXCILSDTL_DEFAULT }, KXCGOPTSTAT_NONE,  \
  0, 0

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCGETOPT_3          KXCGETOPT, *PKXCGETOPT;
#define KXCVER_GETOPT_CURR   KXCVER_GETOPT_3

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetGetOptDefault(pGetOpt)          \
{                                             \
  KXCGETOPT mGO = { KXCGETOPT_DEFAULT };      \
  *(pGetOpt) = mGO;                           \
}

/********************************************************************/
/*                                                                  */
/*                      P U T   O P T I O N S                       */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Put options structure                         */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   RequestOptions;
  PKXCSECDEF pSecurity;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
} KXCPUTOPT_1, *PKXCPUTOPT_1;

#define KXCVER_PUTOPT_1     1

typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   RequestOptions;
  PKXCSECDEF pSecurity;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
  KXCERRDTL  Error;
  KXCOBJNAME APIProfile;
  KXCOBJNAME ErrorProfile;
} KXCPUTOPT_2, *PKXCPUTOPT_2;

#define KXCVER_PUTOPT_2     2

typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   RequestOptions;
  PKXCSECDEF pSecurity;
  KXCMSGID   MsgId;
  KXCMSGID   CorrelId;
  KXCERRDTL  Error;
  KXCOBJNAME APIProfile;
  KXCOBJNAME ErrorProfile;
  KXCULONG   PubSubOptions;
  KXCBUFFER  PubSubSubject;
  KXCILSDTL  ILSData;
  KXCULONG   Status;
  KXCULONG   ForceOn;
  KXCULONG   ForceOff;
} KXCPUTOPT_3, *PKXCPUTOPT_3;

#define KXCVER_PUTOPT_3     3

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCPUTOPT  0x4258504F

#define KXCPOPT_NONE           0x00000000
#define KXCPOPT_USEMSGID       0x00000001
#define KXCPOPT_USECORRELID    0x00000002
#define KXCPOPT_USETHEME       0x00000010
#define KXCPOPT_NEWTHEME       0x00000020
#define KXCPOPT_RETAINTHEME    0x00000040
#define KXCPOPT_CLOSETHEME     0x00000080
#define KXCPOPT_FREEBUFFER     0x00000100
#define KXCPOPT_DATAGRAM       0x00000200
#define KXCPOPT_ALLOWNOSYNC    0x00000400
#define KXCPOPT_NOTIFY         0x00000800
#define KXCPOPT_EXCTIMESTAMP   0x00001000
#define KXCPOPT_EXCROMADATA    0x00002000
#define KXCPOPT_EXCSYSDATA     0x00004000
#define KXCPOPT_EXCERRDATA     0x00008000
#define KXCPOPT_EXCUSERDATA    0x00010000
#define KXCPOPT_INCRETRYCOUNT  0x00020000
#define KXCPOPT_NOEMS          0x00040000
#define KXCPOPT_NONOTIFY       0x00080000
#define KXCPOPT_NOXFORM        0x00100000
#define KXCPOPT_ALLOWTERMINATE 0x00200000
#define KXCPOPT_COA            0x00400000
#define KXCPOPT_COD            0x00800000
#define KXCPOPT_NOERRORPROF    0x01000000

#define KXCPOPT_USEDEFAULT     0x40000000

#define KXCPSOPT_NONE          0x00000000
#define KXCPSOPT_PUBLISH       0x00000001
#define KXCPSOPT_SUBSCRIBE     0x00000002
#define KXCPSOPT_UNSUBSCRIBE   0x00000004

#define KXCPOPT_EXCALL                          \
  (KXCPOPT_EXCTIMESTAMP | KXCPOPT_EXCROMADATA | \
   KXCPOPT_EXCSYSDATA   | KXCPOPT_EXCERRDATA  | \
   KXCPOPT_EXCUSERDATA )

#define KXCPOPTSTAT_NONE      0x00000000
#define KXCPOPTSTAT_TERMINATE 0x00000001

#define KXCPUTOPT_DEFAULT                         \
  KXCSIG_KXCPUTOPT, KXCVER_PUTOPT_3,              \
  KXCPOPT_USEDEFAULT, NULL,                       \
  KXCMSGID_NULL, KXCMSGID_NULL,                   \
  { 0, 0, 0, 0}, "", "",                          \
  0,  { KXCBUFFER_DEFAULT },                      \
  { KXCILSDTL_DEFAULT }, KXCPOPT_ALLOWTERMINATE,  \
  0, 0

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCPUTOPT_3          KXCPUTOPT, *PKXCPUTOPT;
#define KXCVER_PUTOPT_CURR   KXCVER_PUTOPT_3

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetPutOptDefault(pPutOpt)       \
{                                          \
  KXCPUTOPT mPO = { KXCPUTOPT_DEFAULT };   \
  *(pPutOpt) = mPO;                        \
}

/********************************************************************/
/*                                                                  */
/*                         S Y N C P O I N T                        */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the KXC Syncpoint structure                           */
/********************************************************************/
typedef struct
{
  KXCULONG  Signature;
  KXCULONG  Version;
  PKXCVOID  TransactionContext;
  KXCULONG  TransactionType;
} KXCSYNC, *PKXCSYNC;

#define KXCSIG_KXCSYNC  0x42585359

#define KXCVER_SYNC_1     1
#define KXCVER_SYNC_CURR  KXCVER_SYNC_1

#define KXCSYNC_NONE          0
#define KXCSYNC_LOCAL         1
#define KXCSYNC_GLOBAL        2

#define KXCSYNC_USEDEFAULT    ((KXCULONG)(-2))

#define KXCSYNC_DEFAULT                  \
  KXCSIG_KXCSYNC, KXCVER_SYNC_CURR, NULL, KXCSYNC_USEDEFAULT

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetSyncDefault(pSync)       \
{                                      \
  KXCSYNC mS = { KXCSYNC_DEFAULT };    \
  *(pSync) = mS;                       \
}

/********************************************************************/
/*                                                                  */
/*                         E X C E P T I O N                        */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Define the KXC Roma config exception substructure                */
/********************************************************************/
typedef struct
{
  KXCULONG    Version;
  KXCOBJNAME  ElementName;
  KXCULONG    ElementType;
  KXCOBJNAME  QueueName;
  KXCOBJNAME  ComponentName;
  KXCOBJNAME  ServiceName;
  KXCNODENAME NodeName;
} KXCEXCROMACONF_1, *PKXCEXCROMACONF_1;

#define KXCVER_EXCROMACONF_1     1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCEXCROMACONF_DEFAULT                    \
  KXCVER_EXCROMACONF_1, "", 0, "", "", "", ""

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCEXCROMACONF_1         KXCEXCROMACONF, *PKXCEXCROMACONF;
#define KXCVER_EXCROMACONF_CURR  KXCVER_EXCROMACONF_1

/********************************************************************/
/* Define the KXC System config exception substructure              */
/********************************************************************/
typedef struct
{
  KXCULONG   Version;
  KXCULONG   EnvironmentType;
  KXCOBJNAME DomainName;
  KXCOBJNAME UserId;
  KXCOBJNAME ProgramName;
  KXCULONG   ProcessId;
  KXCULONG   ThreadId;
} KXCEXCSYSCONF_1, *PKXCEXCSYSCONF_1;

#define KXCVER_EXCSYSCONF_1      1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCENVTYPE_MASK     0xFF000000
#define KXCENVTYPE_INTEL    0x01000000
#define KXCENVTYPE_UNIX     0x02000000
#define KXCENVTYPE_MVS      0x04000000
#define KXCENVTYPE_OS400    0x08000000
#define KXCENVTYPE_VMS      0x10000000

#define KXCENVOS_MASK       0x00FF0000
#define KXCENVOS_WINNT     (0x00010000 | KXCENVTYPE_INTEL)
#define KXCENVOS_WIN95     (0x00020000 | KXCENVTYPE_INTEL)
#define KXCENVOS_AIX       (0x00010000 | KXCENVTYPE_UNIX)
#define KXCENVOS_HPUX11    (0x00020000 | KXCENVTYPE_UNIX)
#define KXCENVOS_SUN       (0x00040000 | KXCENVTYPE_UNIX)
#define KXCENVOS_HPUX102   (0x00080000 | KXCENVTYPE_UNIX)
#define KXCENVOS_DUX       (0x00100000 | KXCENVTYPE_UNIX)
#define KXCENVOS_MVSCICS   (0x00010000 | KXCENVTYPE_MVS)
#define KXCENVOS_MVSBATCH  (0x00020000 | KXCENVTYPE_MVS)
#define KXCENVOS_OS400     (0x00100000 | KXCENVTYPE_OS400)

#define KXCEXCSYSCONF_DEFAULT                     \
  KXCVER_EXCSYSCONF_1, 0, "", "", "", 0, 0

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCEXCSYSCONF_1          KXCEXCSYSCONF, *PKXCEXCSYSCONF;
#define KXCVER_EXCSYSCONF_CURR   KXCVER_EXCSYSCONF_1

/********************************************************************/
/* Define the KXC Error data exception substructure                 */
/********************************************************************/
typedef struct
{
  KXCULONG    Version;
  KXCERROR    Error;
  KXCULONG    CallType;
  KXCOBJNAME  QueueName;
} KXCEXCERRDATA_1, *PKXCEXCERRDATA_1;

#define KXCVER_EXCERRDATA_1      1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCEXCERRDATA_DEFAULT                     \
  KXCVER_EXCERRDATA_1, { KXCERROR_DEFAULT }, 0, ""

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCEXCERRDATA_1          KXCEXCERRDATA, *PKXCEXCERRDATA;
#define KXCVER_EXCERRDATA_CURR   KXCVER_EXCERRDATA_1

/********************************************************************/
/* Define the KXC User data exception substructure                  */
/********************************************************************/
typedef struct
{
  KXCULONG  Version;
  KXCBUFFER UserBuf;
  KXCULONG  UserBufLen;
} KXCEXCUSRDATA_1, *PKXCEXCUSRDATA_1;

#define KXCVER_EXCUSRDATA_1      1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCEXCUSRDATA_DEFAULT                     \
  KXCVER_EXCUSRDATA_1, { KXCBUFFER_DEFAULT }, 0

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCEXCUSRDATA_1          KXCEXCUSRDATA, *PKXCEXCUSRDATA;
#define KXCVER_EXCUSRDATA_CURR   KXCVER_EXCUSRDATA_1

/********************************************************************/
/* Define the KXC exception structure                               */
/********************************************************************/
typedef struct
{
  KXCTIMESTAMP TimeStamp;
  KXCERROR     Error;
  KXCNODENAME  NodeName;
  KXCULONG     CallType;
  KXCULONG     ElementType;
  KXCOBJNAMESZ ElementName;
  KXCOBJNAMESZ ComponentName;
  KXCOBJNAMESZ QueueName;
} KXCEXCEPTION_1, *PKXCEXCEPTION_1;

#define KXCVER_EXCEPTION_1       1

typedef struct
{
  KXCULONG         Signature;
  KXCULONG         Version;
  KXCULONG         ExceptionFlags;
  KXCTIMESTAMP     TimeStamp;
  KXCEXCROMACONF_1 RomaConfData;
  KXCEXCSYSCONF_1  SystemConfData;
  KXCEXCERRDATA_1  ErrorData;
  KXCEXCUSRDATA_1  UserData;
} KXCEXCEPTION_2, *PKXCEXCEPTION_2;

#define KXCVER_EXCEPTION_2       2

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCEXCEPTION      0x58434558

#define KXCEXC_NONE              0x00000000
#define KXCEXC_TIMESTAMP         0x00000001
#define KXCEXC_ROMADATA          0x00000002
#define KXCEXC_SYSTEMDATA        0x00000004
#define KXCEXC_ERRORDATA         0x00000008
#define KXCEXC_USERDATA          0x00000010

#define KXCEXCEPTION_DEFAULT                                \
  KXCSIG_KXCEXCEPTION, KXCVER_EXCEPTION_2,                  \
  KXCEXC_NONE, { 0, 0 }, { KXCEXCROMACONF_DEFAULT },      \
  { KXCEXCSYSCONF_DEFAULT }, { KXCEXCERRDATA_DEFAULT }, \
  { KXCEXCUSRDATA_DEFAULT }

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCEXCEPTION_2           KXCEXCEPTION, *PKXCEXCEPTION;
#define KXCVER_EXCEPTION_CURR    KXCVER_EXCEPTION_2

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetExceptionDefault(mpExc)            \
{                                                \
  KXCEXCEPTION mEX = { KXCEXCEPTION_DEFAULT };   \
  *(mpExc) = mEX;                                \
}

/********************************************************************/
/* Define the KXC Error Profiling options structure                 */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   ErrorOptions;
  KXCOBJNAME ErrorProfile;
  KXCULONG   DefaultAction;
  KXCERRDTL  Error;
  KXCULONG   Actions;
} KXCEPOPT_1, *PKXCEPOPT_1;

#define KXCVER_EPOPT_1           1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCEPOPT          0x58434550

#define KXCEPOPT_NONE            0x00000000
#define KXCEPOPT_USEEPNAME       0x00000001
#define KXCEPOPT_USEDEFAULT      0x00000002
#define KXCEPOPT_NONOTIFY        0x00000004
#define KXCEPOPT_NOEMS           0x00000008
#define KXCEPOPT_NOALERT         0x00000010
#define KXCEPOPT_NOXFORM         0x00000020

#define KXCEP_ACTION_DEFAULT     0x00000000
#define KXCEP_ACTION_NOTIFY      0x00000001
#define KXCEP_ACTION_EMS         0x00000002
#define KXCEP_ACTION_ALERT       0x00000004
#define KXCEP_ACTION_LOG         0x00000008
#define KXCEP_ACTION_RESET       0x00000010
#define KXCEP_ACTION_NOACTION    0x00000020
#define KXCEP_ACTION_EMSRAW      0x00000040

#define KXCEPOPT_DEFAULT                   \
  KXCSIG_KXCEPOPT, KXCVER_EPOPT_1,         \
  KXCEPOPT_NONE, "", KXCEP_ACTION_DEFAULT, \
  { 0, 0, 0, 0}, KXCEP_ACTION_DEFAULT

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCEPOPT_1               KXCEPOPT, *PKXCEPOPT;
#define KXCVER_EPOPT_CURR        KXCVER_EPOPT_1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetErrOptDefault(pEPOpt)     \
{                                       \
  KXCEPOPT mEPO = { KXCEPOPT_DEFAULT }; \
  *(pEPOpt) = mEPO;                     \
}

#define kxcSetExceptionUserData(mpExc, mBufLen, mpBuff)\
  if ((mpExc)->UserData.UserBuf.Status           \
                 == KXCBUFSTAT_KXC)              \
  { kxcFreeBuffer(NULL_HKXC,                     \
            &((mpExc)->UserData.UserBuf)); };    \
  (mpExc)->UserData.UserBuf.pBuffer = (mpBuff);  \
  (mpExc)->UserData.UserBuf.BufLen = (mBufLen);  \
  (mpExc)->UserData.UserBuf.Status = KXCBUFSTAT_USER;

/********************************************************************/
/* Define the KXC logging data structure                            */
/********************************************************************/
typedef struct
{
  KXCULONG  Signature;
  KXCULONG  Version;
  KXCULONG  Severity;
  KXCULONG  Format;
  PKXCCHAR  pUserText;
  KXCBUFFER DataBuf;
  KXCULONG  DataLen;
} KXCLOGDATA_1, *PKXCLOGDATA_1;

#define KXCVER_LOGDATA_1         1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCLOGDATA        0x58434C44

#define KXCLOGSEVERITY_INFO      0
#define KXCLOGSEVERITY_WARNING   1
#define KXCLOGSEVERITY_ERROR     2
#define KXCLOGSEVERITY_SEVERE    3

#define KXCLOGFORMAT_NONE        0x00000000
#define KXCLOGFORMAT_NOTIMESTAMP 0x00000001
#define KXCLOGFORMAT_NOSEVERITY  0x00000002
#define KXCLOGFORMAT_HEXDUMP     0x00000004
#define KXCLOGFORMAT_MASK        0x00000007

#define KXCLOGDATA_DEFAULT                    \
  KXCSIG_KXCLOGDATA, KXCVER_LOGDATA_1,        \
  KXCLOGSEVERITY_INFO, KXCLOGFORMAT_HEXDUMP,  \
  NULL, { KXCBUFFER_DEFAULT }, 0

#define KXCLOGDATA_DEFUSRTXT     ("DEFAULT LOGDATA ENTRY")
#define KXCLOGDATA_WFLOWHDR      ("WORKFLOW:")
#define KXCLOGDATA_MSGHDR        ("MESSAGE:")
#define KXCLOGDATA_DATASEP       (":")

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCLOGDATA_1             KXCLOGDATA, *PKXCLOGDATA;
#define KXCVER_LOGDATA_CURR      KXCVER_LOGDATA_1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetLogDefault(mpUserLog)              \
{                                                \
  KXCLOGDATA mUL = { KXCLOGDATA_DEFAULT };       \
  *(mpUserLog) = mUL;                            \
}

#define kxcSetLogBuffer(mpUserLog, mBufLen, mpBuff)\
  if ((mpUserLog)->DataBuf.Status                \
                 == KXCBUFSTAT_KXC)              \
  { kxcFreeBuffer(NULL_HKXC,                     \
                  &((mpUserLog)->DataBuf)); };   \
  (mpUserLog)->DataBuf.pBuffer = (mpBuff);       \
  (mpUserLog)->DataBuf.BufLen  = (mBufLen);      \
  (mpUserLog)->DataBuf.Status  = KXCBUFSTAT_USER;

/********************************************************************/
/*                                                                  */
/*                    S T O R A G E   Q U E U E                     */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Define the storageQ definition structure                         */
/********************************************************************/
typedef struct
{
  KXCULONG      Signature;
  KXCULONG      Version;
  KXCULONG      Options;
  KXCOBJNAME    QueueName;
} KXCSTORQUEUEDEF_1, *PKXCSTORQUEUEDEF_1;

#define KXCVER_STORQ_1     1

typedef struct
{
  KXCULONG      Signature;
  KXCULONG      Version;
  KXCULONG      Options;
  KXCOBJNAME    QueueName;
  KXCOBJNAME    ModelQName;
} KXCSTORQUEUEDEF_2, *PKXCSTORQUEUEDEF_2;

#define KXCVER_STORQ_2     2

#define KXCSIG_KXCSTORQ 0x58435351
#define KXCVER_STORQ_CURR  KXCVER_STORQ_2

#define KXCSTORQUEUEDEF1_DEFAULT           \
  KXCSIG_KXCSTORQ, KXCVER_STORQ_CURR, 0, ""

#define KXCSTORQUEUEDEF2_DEFAULT           \
  KXCSIG_KXCSTORQ, KXCVER_STORQ_CURR, 0, "", ""

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCSTORQUEUEDEF_2        KXCSTORQUEUEDEF, *PKXCSTORQUEUEDEF;
#define KXCSTORQUEUEDEF_DEFAULT  KXCSTORQUEUEDEF2_DEFAULT

/********************************************************************/
/* Define the values for the storage queue open options             */
/********************************************************************/
#define KXCOOPT_NONE               0x00000000
#define KXCOOPT_OUTPUT             0x00000001
#define KXCOOPT_INPUT              0x00000002
#define KXCOOPT_PEEK               0x00000004
#define KXCOOPT_NON_XACTIONAL      0x00000008
#define KXCOOPT_EXCLUSIVE          0x00000010
#define KXCOOPT_DELETE_ON_CLOSE    0x00000020
#define KXCOOPT_MUST_EXIST         0x00000040

/********************************************************************/
/* Now define the KXC Storage Queue handle                          */
/********************************************************************/
typedef KXCHANDLE HKXCSTORQ,  *PHKXCSTORQ;
#define NULL_HKXCSTORQ   0

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetStoreQDefault(pStorQ)                   \
{                                                    \
  KXCSTORQUEUEDEF StQ = { KXCSTORQUEUEDEF_DEFAULT }; \
  *(pStorQ) = StQ;                                   \
}

/********************************************************************/
/*                                                                  */
/*                   I L S   D E F I N I T I O N                    */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Define the KXC ILSData options structure                         */
/********************************************************************/
typedef struct
{
  KXCULONG   Signature;
  KXCULONG   Version;
  KXCULONG   Options;
  KXCULONG   Type;
  KXCULONG   Method;
  KXCBUFFER  DataBuf;
  KXCULONG   DataLen;
} KXCILSOPT_1, *PKXCILSOPT_1;

#define KXCVER_ILSOPT_1          1

/********************************************************************/
/* And the field contents                                           */
/********************************************************************/
#define KXCSIG_KXCILSOPT         0x4258494C

#define KXCIOPT_NONE             0x00000000
#define KXCIOPT_REPLACE          0x00000001
#define KXCIOPT_GETNEXT          0x00000002

#define KXCILSOPT_DEFAULT                   \
  KXCSIG_KXCILSOPT, KXCVER_ILSOPT_1,        \
  KXCIOPT_NONE, 0, 0, { KXCBUFFER_DEFAULT }

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCILSOPT_1              KXCILSOPT, *PKXCILSOPT;
#define KXCVER_ILSOPT_CURR       KXCVER_ILSOPT_1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetILSDefault(mpILSOpt)               \
{                                                \
  KXCILSOPT mIOpt = { KXCILSOPT_DEFAULT };       \
  *(mpILSOpt) = mIOpt;                           \
}

/********************************************************************/
/*                                                                  */
/*                 D I R E C T O R Y   A C C E S S                  */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Define the Directory search requirements structure               */
/********************************************************************/
typedef struct {
  KXCULONG        Signature;
  KXCULONG        Version;
  KXCULONG        Options;
  PKXCCHAR        StartingPoint;
  KXCULONG        SearchType;
  PKXCCHAR        SearchName;
} KXCDIRSEARCHREQ_1, *PKXCDIRSEARCHREQ_1;

#define KXCVER_KXCDIRSEARCHREQ_1    1
#define KXCSIG_KXCDIRSEARCHREQ      0x58445251

/********************************************************************/
/* Define the search types for the directory search request         */
/********************************************************************/
#define KXCDOPT_SEARCH_EXACT          0
#define KXCDOPT_SEARCH_ONELEVEL       1
#define KXCDOPT_SEARCH_SUBTREE        2

/********************************************************************/
/* Define the search options                                        */
/********************************************************************/
#define KXCDOPT_NONE                  0x00000000
#define KXCDOPT_USE_ROMA_BASE         0x00000001

/********************************************************************/
/* And the access macros                                            */
/********************************************************************/
#define KXCDIRSEARCHREQ_DEFAULT                         \
  KXCSIG_KXCDIRSEARCHREQ, KXCVER_KXCDIRSEARCHREQ_1,     \
  KXCDOPT_NONE, NULL, KXCDOPT_SEARCH_EXACT, NULL

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRSEARCHREQ_1            KXCDIRSEARCHREQ, *PKXCDIRSEARCHREQ;
#define KXCVER_KXCDIRSEARCHREQ_CURR  KXCVER_KXCDIRSEARCHREQ_1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetDirSearchReqDefault(mpSearch)             \
{                                                       \
  KXCDIRSEARCHREQ mISearch = {KXCDIRSEARCHREQ_DEFAULT}; \
  *(mpSearch) = mISearch;                               \
}

/********************************************************************/
/* Define the Directory search results structure                    */
/********************************************************************/
typedef struct {
  KXCULONG        Signature;
  KXCULONG        Version;
  KXCULONG        Matches;
  KXCULONG        ModCount;
} KXCDIRSEARCHRES_1, *PKXCDIRSEARCHRES_1;

#define KXCVER_KXCDIRSEARCHRES_1    1
#define KXCSIG_KXCDIRSEARCHRES      0x58445253

#define KXCDIRSEARCHRES_DEFAULT     \
  KXCSIG_KXCDIRSEARCHRES, KXCVER_KXCDIRSEARCHRES_1, 0, 0

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRSEARCHRES_1            KXCDIRSEARCHRES, *PKXCDIRSEARCHRES;
#define KXCVER_KXCDIRSEARCHRES_CURR  KXCVER_KXCDIRSEARCHRES_1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetDirSearchResDefault(mpResults)             \
{                                                        \
  KXCDIRSEARCHRES mIResults = {KXCDIRSEARCHRES_DEFAULT}; \
  *(mpResults) = mIResults;                              \
}

/********************************************************************/
/* Define the Directory Get Item structure                          */
/********************************************************************/
typedef struct {
  KXCULONG        Signature;
  KXCULONG        Version;
  KXCULONG        EntryNumber;
} KXCDIRITEMREQ_1, *PKXCDIRITEMREQ_1;

#define KXCVER_KXCDIRITEMREQ_1    1
#define KXCSIG_KXCDIRITEMREQ      0x58444951

#define KXCDIRITEMREQ_DEFAULT     \
  KXCSIG_KXCDIRITEMREQ, KXCVER_KXCDIRITEMREQ_1, 0

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRITEMREQ_1            KXCDIRITEMREQ, *PKXCDIRITEMREQ;
#define KXCVER_KXCDIRITEMREQ_CURR  KXCVER_KXCDIRITEMREQ_1

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetDirItemReqDefault(mpItem)                  \
{                                                        \
  KXCDIRITEMREQ mIItem = {KXCDIRITEMREQ_DEFAULT};        \
  *(mpItem) = mIItem;                                    \
}

/********************************************************************/
/* Define the Directory Update Item structure                       */
/********************************************************************/
typedef struct {
  KXCULONG        Signature;
  KXCULONG        Version;
  KXCULONG        Options;
  KXCOBJNAME      Id;
  KXCOBJNAME      Password;
} KXCDIRUPDATEOPT_1, *PKXCDIRUPDATEOPT_1;

#define KXCVER_KXCDIRUPDATEOPT_1  1
#define KXCSIG_KXCDIRUPDATEOPT    0x58445550

#define KXCDIRUPDATEOPT_DEFAULT                       \
  KXCSIG_KXCDIRUPDATEOPT, KXCVER_KXCDIRUPDATEOPT_1,   \
  KXCUPDOPT_NONE, "", ""

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRUPDATEOPT_1           KXCDIRUPDATEOPT, *PKXCDIRUPDATEOPT;
#define KXCVER_KXCDIRUPDATEOPT_CURR KXCVER_KXCDIRUPDATEOPT_1

/********************************************************************/
/* Define the options for the update directory request              */
/********************************************************************/
#define KXCUPDOPT_NONE               0x00000000
#define KXCUPDOPT_MUSTEXIST          0x00000001
#define KXCUPDOPT_MUSTNOTEXIST       0x00000002

#define KXCUPDOPT_MASK               0x00000003

/********************************************************************/
/* Define the Directory Item header structure which is returned     */
/* from a kxcDirGetItem call and prefixes the DN and list of name   */
/* value pairs.                                                     */
/********************************************************************/
typedef struct {
  KXCULONG        Signature;
  KXCULONG        Version;
  KXCULONG        ModCount;
  KXCULONG        nNVPairs;
  KXCULONG        Flags;
  KXCULONG        TotalLen;
} KXCDIRITEMMAINHDR_1, *PKXCDIRITEMMAINHDR_1;

#define KXCVER_KXCDIRITEMMAINHDR_1  1
#define KXCSIG_KXCDIRITEMMAINHDR    0x58444948

#define KXCDIRITEMMAINHDR_DEFAULT                                 \
  KXCSIG_KXCDIRITEMMAINHDR, KXCVER_KXCDIRITEMMAINHDR_1, 0, 0,   \
  KXCDIRITEMFLAG_NONE, 0
/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRITEMMAINHDR_1   KXCDIRITEMMAINHDR, *PKXCDIRITEMMAINHDR;
#define KXCVER_KXCDIRITEMMAINHDR_CURR KXCVER_KXCDIRITEMMAINHDR_1

#define KXCDIRITEMFLAG_NONE       0x00000000
#define KXCDIRITEMFLAG_DN_EXISTS  0x00000001

/********************************************************************/
/* Define the Directory Item Main header structure returned         */
/* from a kxcDirGetItem call and prefixes the DN and list of name   */
/* value pairs.                                                     */
/********************************************************************/
typedef struct {
  KXCULONG        AttribLen;
  KXCULONG        ValueLen;
} KXCDIRITEMHDR_1, *PKXCDIRITEMHDR_1;

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRITEMHDR_1           KXCDIRITEMHDR, *PKXCDIRITEMHDR;

/********************************************************************/
/* Define the Directory Item Main header structure returned         */
/* from a kxcDirGetItem call and prefixes the DN and list of name   */
/* value pairs.                                                     */
/********************************************************************/
typedef struct {
  KXCULONG        DnLen;
} KXCDIRDNHDR_1, *PKXCDIRDNHDR_1;

/********************************************************************/
/* And declare the current version                                  */
/********************************************************************/
typedef KXCDIRDNHDR_1           KXCDIRDNHDR, *PKXCDIRDNHDR;

/********************************************************************/
/* Macro to set the structure to default value                      */
/********************************************************************/
#define kxcSetDirUpdateOptDefault(mpUpdate)              \
{                                                        \
  KXCDIRUPDATEOPT mIUpdate = {KXCDIRUPDATEOPT_DEFAULT};  \
  *(mpUpdate) = mIUpdate;                                \
}

/********************************************************************/
/* define the macros to access the items within the buffer that was */
/* returned by the function kxcDirGetItem                           */
/********************************************************************/
#define kxcDirGetDn(mpBuff, mpDn, mResult)                          \
{                                                                   \
  mResult = kxcDirGetDnEx((PKXCBUFFER)(mpBuff),                     \
                          (PKXCCHAR *)&(mpDn));                     \
}

#define kxcDirGetAttributeAt(mpBuff, mNum, mpName, mpVal, mResult)  \
{                                                                   \
  mResult = kxcDirGetAttributeAtEx((PKXCBUFFER)(mpBuff),            \
                                   (KXCULONG)(mNum),                \
                                   (PKXCCHAR *)&(mpName),           \
                                   (PKXCCHAR *)&(mpVal));           \
}

#define kxcDirGetValue(mpBuff, mpName, mNum, mpVal, mResult)        \
{                                                                   \
  mResult = kxcDirGetValueEx((PKXCBUFFER)(mpBuff),                  \
                             (PKXCCHAR)(mpName),                    \
                             (KXCULONG)(mNum),                      \
                             (PKXCCHAR *)&(mpVal));                 \
}

#define kxcDirCountAttributes(mpBuff, mnAttribs, mResult)           \
{                                                                   \
  mResult = kxcDirCountAttributesEx((PKXCBUFFER)(mpBuff),           \
                                    (PKXCULONG)&(mnAttribs));       \
}


/********************************************************************/
/* Now define the KXC Directory Item Access write macros which will */
/* initialize the header and distinguished name of a buffer that    */
/* will be passed to the kxcDirUpdateItem and kxcDirDeleteItem API  */
/* calls.                                                           */
/********************************************************************/
#define kxcSetDirDn(mpBuff, mpDn, mResult)                          \
{                                                                   \
  mResult = kxcDirSetDnEx((PKXCBUFFER)(mpBuff),                     \
                          (PKXCCHAR)(mpDn));                        \
}

#define kxcSetDirItem(mpBuff, mpName, mpVal, mResult)               \
{                                                                   \
  mResult = kxcDirSetItemEx((PKXCBUFFER)(mpBuff),                   \
                            (PKXCCHAR)(mpName),                     \
                            (PKXCCHAR)(mpVal));                     \
}

/********************************************************************/
/* Now define the BMQ NativeQ message Class and Type                */
/* Access macros                                                    */
/********************************************************************/
#define kxcGetBMQClass(pMsgId)                        \
          (short)((((short)(pMsgId)[0]) << 8) |       \
          ((short)(pMsgId)[1] & 0x00FF))

#define kxcGetBMQType(pMsgId)                         \
          (short)((((short)(pMsgId)[2]) << 8) |       \
          ((short)(pMsgId)[3] & 0x00FF))

#define kxcSetBMQClass(pMsgId,c)                      \
          (pMsgId)[0] = (((short)(c) & 0xFF00) >> 8); \
          (pMsgId)[1] = ((short)(c) & 0x00FF);

#define kxcSetBMQType(pMsgId, t)                      \
          (pMsgId)[2] = (((short)(t) & 0xFF00) >> 8); \
          (pMsgId)[3] = ((short)(t) & 0x00FF);


/********************************************************************/
/*                                                                  */
/*                      E R R O R   C O D E S                       */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/* Now define the completion codes                                  */
/********************************************************************/
#define KXCCC_OK                       0
#define KXCCC_WARNING                  1
#define KXCCC_FAILED                   2

/********************************************************************/
/* Now define the Operating layer types generating the error        */
/*                                                                  */
/* NB User OpTypes must be within the range [KXCOP_USER..KXCOP_MAX] */
/*    inclusive.                                                    */
/********************************************************************/
#define KXCOP_KXC                      0
#define KXCOP_MQSERIES                 1
#define KXCOP_MSMQ                     2
#define KXCOP_RPC                      3
#define KXCOP_OS                       4
#define KXCOP_LIB                      5
#define KXCOP_LDAP                     6
#define KXCOP_LDIF                     7
#define KXCOP_CICS                     8
#define KXCOP_TIB                      9
#define KXCOP_CTLIB                   10
#define KXCOP_MQSI                    11              /* SR1-2EE9H */
#define KXCOP_BMQ                     12
#define KXCOP_KXC_OM                  50
#define KXCOP_USER                   100
#define KXCOP_MAX                    254
#define KXCOP_INVALID                255

/********************************************************************/
/* Now define the return codes                                      */
/********************************************************************/
#define KXCRC_OK                       0

#define KXCRC_INVALID_CONTEXT          1
#define KXCRC_INVALID_OPTIONS          2
#define KXCRC_INVALID_ROUTE            3
#define KXCRC_INVALID_THEME            4
#define KXCRC_INVALID_MESSAGE          5
#define KXCRC_INVALID_RESPONSE         6
#define KXCRC_INVALID_PARAMETER        7
#define KXCRC_INVALID_APPDEF           8
#define KXCRC_INVALID_SERVICE          9
#define KXCRC_INVALID_HANDLE          10
#define KXCRC_CONTEXT_MISMATCH        11
#define KXCRC_NONXACTIONAL            12
#define KXCRC_NON_LOCALQ              13
#define KXCRC_INVALID_SYNCTYPE        14
#define KXCRC_INVALID_SECURITY        15
#define KXCRC_CALL_IN_PROGRESS        16
#define KXCRC_BUFFER_EXISTS           17
#define KXCRC_NO_DATA_AVAILABLE       18

#define KXCRC_NO_MSG_AVAILABLE        20
#define KXCRC_BUFFER_TOO_SMALL        21
#define KXCRC_BUFFER_OVERFLOW         22
#define KXCRC_NOT_SUPPORTED           23
#define KXCRC_VALIDATION_FAILED       24
#define KXCRC_INVALID_TRAILER         25
#define KXCRC_MESSAGE_SUPPRESSED      26
#define KXCRC_BACKED_OUT              27
#define KXCRC_ACCESS_DENIED           28
#define KXCRC_BUFFER_ERROR            29
#define KXCRC_Q_DELETED               30
#define KXCRC_Q_FULL                  31
#define KXCRC_MSG_UNAVAILABLE         32
#define KXCRC_SIGNAL_ACCEPTED         33
#define KXCRC_SIGNAL_INVALID          34
#define KXCRC_SIGNAL_OUTSTANDING      35

#define KXCRC_ROUTING_FAILURE         40
#define KXCRC_STORAGE_FAILURE         41
#define KXCRC_TRANSPORT_FAILURE       42
#define KXCRC_EXIT_FAILURE            43
#define KXCRC_EXCEPTION_CAUGHT        44
#define KXCRC_NOT_ACCESSIBLE          45
#define KXCRC_RESOURCE_PROBLEM        46
#define KXCRC_TRANSPORT_UNAVAILABLE   47
#define KXCRC_MUTEX_FAILURE           48
#define KXCRC_INVALID_MSGTYPE         49
#define KXCRC_CACHE_FAILURE           50
#define KXCRC_HANDLE_BROKEN           51
#define KXCRC_SYSCONFIG_FAILURE       52
#define KXCRC_TRANSFORM_DISALLOWED    53

#define KXCRC_UNKNOWN_INITPROFILE     57
#define KXCRC_UPDATE_FAILED           58
#define KXCRC_DELETE_FAILED           59
#define KXCRC_UNKNOWN_SERVICE         60
#define KXCRC_UNKNOWN_MSGFORMAT       61
#define KXCRC_UNKNOWN_CLIENT          62
#define KXCRC_UNKNOWN_SERVER          63
#define KXCRC_UNKNOWN_BRIDGE          64
#define KXCRC_INVALID_QNAME           65
#define KXCRC_CONFIG_ERROR            66
#define KXCRC_DIRECTORY_UNAVAILABLE   67
#define KXCRC_BRIDGE_UNAVAILABLE      68
#define KXCRC_INVALID_COMPONENT       69
#define KXCRC_INVALID_ELEMENT         70
#define KXCRC_INVALID_SERVERNAME      71
#define KXCRC_INVALID_FILENAME        72
#define KXCRC_WAIT_CANCELLED          73
#define KXCRC_UNKNOWN_ERRPROFILE      74
#define KXCRC_UNKNOWN_PUTPROFILE      75
#define KXCRC_UNKNOWN_GETPROFILE      76
#define KXCRC_UNKNOWN_OBJECT          77
#define KXCRC_GET_ATTRIBUTE_FAILURE   78
#define KXCRC_UNKNOWN_NODE            79


#define KXCRC_LDAP_GET_FAILURE             80
#define KXCRC_LDAP_OPEN_FAILURE            81
#define KXCRC_LDAP_BIND_FAILURE            82
#define KXCRC_LDAP_SEARCH_FAILURE          83
#define KXCRC_LDAP_FIRST_ENTRY_FAILURE     84
#define KXCRC_LDAP_GET_DN_FAILURE          85
#define KXCRC_LDAP_NEXT_ENTRY_FAILURE      86
#define KXCRC_LDAP_COUNT_ENTRIES_FAILURE   87
#define KXCRC_LDAP_MODIFY_FAILURE          88
#define KXCRC_LDAP_DELETE_FAILURE          89
#define KXCRC_LDAP_ADD_FAILURE             90
#define KXCRC_LDAP_UNBIND_FAILURE          92
#define KXCRC_LDAP_CONNECTION_LOST         95
#define KXCRC_LDAP_INVALID_GATEWAY         96
#define KXCRC_LDAP_MODRDN_FAILURE          97
#define KXCRC_LDAP_IMPORT_FAILURE          98
#define KXCRC_LDAP_EXPORT_FAILURE          99
#define KXCRC_LDAP_MODCOUNT_FAILURE       100

#define KXCRC_LDIF_GET_FAILURE             120
#define KXCRC_LDIF_OPEN_FAILURE            121
#define KXCRC_LDIF_SEARCH_FAILURE          123
#define KXCRC_LDIF_GET_DN_FAILURE          124
#define KXCRC_LDIF_NEXT_ENTRY_FAILURE      125
#define KXCRC_LDIF_COUNT_ENTRIES_FAILURE   126
#define KXCRC_LDIF_MODIFY_FAILURE          127
#define KXCRC_LDIF_DELETE_FAILURE          128
#define KXCRC_LDIF_ADD_FAILURE             129
#define KXCRC_LDIF_INVALID_GATEWAY         131
#define KXCRC_LDIF_RENAME_FAILURE          132
#define KXCRC_LDIF_INVALID_ATTR_FAILURE    133
#define KXCRC_LDIF_READ_FAILURE            134
#define KXCRC_LDIF_WRITE_FAILURE           135

#define KXCRC_INVALID_PS_OPTIONS           140
#define KXCRC_INVALID_SUBJECT              141
#define KXCRC_THREAD_INIT_FAILURE          142
#define KXCRC_PERSISTENCE_NOT_SUPPORTED    143
#define KXCRC_MSGID_NOT_SUPPORTED          144
#define KXCRC_CORRELID_NOT_SUPPORTED       145
#define KXCRC_PEEK_NOT_SUPPORTED           146

#define KXCRC_NO_AVAILABLE_ELEMENT         149

#define KXCRC_CICS_GETMAIN_FAILURE           1
#define KXCRC_CICS_COMMIT_ROLLBACK           2
#define KXCRC_CICS_NO_TCTLS                  3
#define KXCRC_CICS_QMGR_QUIESCE              4
#define KXCRC_CICS_CONN_QUIESCE              5
#define KXCRC_CICS_WAITFAIL                  6
#define KXCRC_CICS_COMMAND_TIMEOUT           7
#define KXCRC_CICS_INVALID_POSTCODE          8
#define KXCRC_CICS_EXEC_WAIT_ERROR           9
#define KXCRC_CICS_GETSIG_FAIL              10
#define KXCRC_CICS_CALL_ABANDONED           11
#define KXCRC_CICS_KXCCTPI_UNAVAILABLE      12
#define KXCRC_CICS_KXCCCDIR_UNAVAILABLE     13
#define KXCRC_CICS_TRUE_INACTIVE            14
#define KXCRC_CMQI_CALL_ABANDONED         5000

#define KXCPOS_0                       0
#define KXCPOS_1                       1
#define KXCPOS_2                       2
#define KXCPOS_3                       3
#define KXCPOS_4                       4
#define KXCPOS_5                       5
#define KXCPOS_6                       6
#define KXCPOS_7                       7
#define KXCPOS_8                       8
#define KXCPOS_9                       9
#define KXCPOS_10                     10
#define KXCPOS_11                     11
#define KXCPOS_12                     12
#define KXCPOS_13                     13
#define KXCPOS_14                     14
#define KXCPOS_15                     15
#define KXCPOS_16                     16
#define KXCPOS_17                     17
#define KXCPOS_18                     18
#define KXCPOS_19                     19
#define KXCPOS_20                     20

#define KXCETYPE_CRITICAL   0x00000001
#define KXCETYPE_ERROR      0x00000002
#define KXCETYPE_TRANSIENT  0x00000004
#define KXCETYPE_API        0x00000008

#ifndef    KXCNOSIGNATURES
/********************************************************************/
/* Now define the function calls                                    */
/********************************************************************/
extern KXCBOOL KXCENTRY kxcGetLastError(PKXCERROR pError);

extern KXCBOOL KXCENTRY kxcClientInitialize(PKXCAPPDEF  pAppDef,
                                            PHKXC       phKXC);
extern KXCBOOL KXCENTRY kxcServerInitialize(PKXCAPPDEF  pAppDef,
                                            PHKXC       phKXC);
extern KXCBOOL KXCENTRY kxcTerminate(PHKXC phKXC);

extern KXCBOOL KXCENTRY kxcFreeBuffer(HKXC       hKXC,
                                      PKXCBUFFER pBuffer);

extern KXCBOOL KXCENTRY kxcPutRequest(HKXC        hKXC,
                                      PKXCSERVICE pService,
                                      PKXCMSG     pMessage,
                                      PKXCPUTOPT  pOptions,
                                      PKXCSYNC    pSyncPt,
                                      PHKXCTHEME  phTheme);

extern KXCBOOL KXCENTRY kxcPutResponse(HKXC        hKXC,
                                       PKXCMSG     pMessage,
                                       PKXCPUTOPT  pOptions,
                                       PKXCSYNC    pSyncPt,
                                       PHKXCTHEME  phTheme);

extern KXCBOOL KXCENTRY kxcGetRequest(HKXC       hKXC,
                                      PKXCMSG    pMessage,
                                      PKXCGETOPT pOptions,
                                      PKXCSYNC   pSyncPt,
                                      PHKXCTHEME phTheme);

extern KXCBOOL KXCENTRY kxcGetResponse(HKXC       hKXC,
                                       PKXCMSG    pMessage,
                                       PKXCGETOPT pOptions,
                                       PKXCSYNC   pSyncPt,
                                       PHKXCTHEME phTheme);

extern KXCBOOL KXCENTRY kxcCloseHandle(HKXC       hKXC,
                                       PKXCHANDLE pHandle);

extern KXCBOOL KXCENTRY kxcCommit(HKXC     hKXC,
                                  PKXCSYNC pSyncPt);
extern KXCBOOL KXCENTRY kxcRollback(HKXC     hKXC,
                                    PKXCSYNC pSyncPt);

extern KXCVOID KXCENTRY kxcFormatError(PKXCSUBERROR  pSubError,
                                       KXCULONG      Flags,
                                       char         *Buffer,
                                       PKXCULONG     pBufSize);

extern KXCBOOL KXCENTRY kxcWriteLog(PKXCLOGDATA  pLog);
extern KXCBOOL KXCENTRY kxcSendAlert(HKXC hKXC, PKXCLOGDATA  pLog);

extern KXCBOOL KXCENTRY kxcPutException(HKXC          hKXC,
                                        PKXCSERVICE   pService,
                                        PKXCMSG       pMessage,
                                        PKXCEXCEPTION pException,
                                        PKXCPUTOPT    pOptions,
                                        PKXCSYNC      pSyncPt,
                                        PHKXCTHEME    phTheme);

extern KXCBOOL KXCENTRY kxcReportError(HKXC          hKXC,
                                       PKXCEPOPT     pOptions,
                                       PKXCMSG       pMessage,
                                       PKXCEXCEPTION pException,
                                       PKXCSYNC      pSyncPt,
                                       PHKXCTHEME    phTheme);

/********************************************************************/
/* Define the StorageQ API functions                                */
/********************************************************************/
extern KXCBOOL KXCENTRY kxcOpenStorageQueue(HKXC        hKXC,
                                       PKXCSTORQUEUEDEF pQueueDef,
                                       PHKXCSTORQ       phStoreQ);
extern KXCBOOL KXCENTRY kxcPutToStorageQueue(HKXC   hKXC,
                                       HKXCSTORQ    hStoreQ,
                                       PKXCMSG      pMessage,
                                       PKXCPUTOPT   pOptions,
                                       PKXCSYNC     pSync,
                                       PHKXCTHEME   phTheme);
extern KXCBOOL KXCENTRY kxcGetFromStorageQueue(HKXC hKXC,
                                       HKXCSTORQ    hStoreQ,
                                       PKXCMSG      pMessage,
                                       PKXCGETOPT   pOptions,
                                       PKXCSYNC     pSync,
                                       PHKXCTHEME   phTheme);
extern KXCBOOL KXCENTRY kxcCloseStorageQueue(HKXC hKXC,
                                       PHKXCSTORQ phStoreQ);
extern KXCBOOL KXCENTRY kxcDeleteStorageQueue(HKXC      hKXC,
                                       PKXCSTORQUEUEDEF pQueueDef);

/********************************************************************/
/* Define the ILSData API functions                                 */
/********************************************************************/
extern KXCBOOL KXCENTRY kxcSetILSData(PKXCILSOPT pILSOpt,
                                      PHKXCILS   phILSData);
extern KXCBOOL KXCENTRY kxcGetILSData(PKXCILSOPT pILSOpt,
                                      HKXCILS    hILSData);

/********************************************************************/
/* Define the Directory Look Up functions                           */
/********************************************************************/
extern KXCBOOL KXCENTRY kxcDirLookUp(HKXC             hKxc,
                                     PKXCDIRSEARCHREQ pSearchReq,
                                     PKXCDIRSEARCHRES pSearchRes);
extern KXCBOOL KXCENTRY kxcDirGetItem(HKXC           hKxc,
                                      PKXCDIRITEMREQ pItemReq,
                                      PKXCBUFFER     pItemDtl);
extern KXCBOOL KXCENTRY kxcDirUpdateItem(HKXC             hKxc,
                                         PKXCDIRUPDATEOPT pOptions,
                                         PKXCBUFFER       pItemDtl);
extern KXCBOOL KXCENTRY kxcDirDeleteItem(HKXC             hKxc,
                                         PKXCDIRUPDATEOPT pOptions,
                                         PKXCBUFFER       pItemDtl);
extern KXCBOOL KXCENTRY kxcDirGetModCount(HKXC          hKxc,
                                          PKXCULONG     pModCount);


/********************************************************************/
/* Define the Directory API helper functions.                       */
/********************************************************************/
extern KXCBOOL KXCENTRY kxcDirGetDnEx(PKXCBUFFER  pBuffer,
                                      PKXCCHAR   *ppDn);
extern KXCBOOL KXCENTRY kxcDirGetAttributeAtEx(PKXCBUFFER  pBuffer,
                                               KXCULONG    Num,
                                               PKXCCHAR   *ppName,
                                               PKXCCHAR   *ppVal);
extern KXCBOOL KXCENTRY  kxcDirGetValueEx(PKXCBUFFER  pBuffer,
                                          PKXCCHAR    pName,
                                          KXCULONG    Num,
                                          PKXCCHAR   *ppVal);
extern KXCBOOL KXCENTRY kxcDirCountAttributesEx(PKXCBUFFER  pBuffer,
                                                PKXCULONG   pNum);
extern KXCBOOL KXCENTRY kxcDirSetDnEx(PKXCBUFFER  pBuffer,
                                      PKXCCHAR    pDn);
extern KXCBOOL KXCENTRY kxcDirSetItemEx(PKXCBUFFER  pBuffer,
                                        PKXCCHAR    pName,
                                        PKXCCHAR    pVal);
#endif  /* KXCNOSIGNATURES */

#ifndef   KXCNOAMAC
/********************************************************************/
/* Include environment definition                                   */
/********************************************************************/
  #include <kxcamac.h>
#endif /* KXCNOAMAC */

/********************************************************************/
/* Enable C++ support                                               */
/********************************************************************/
#if defined(__cplusplus)
  }
#endif

/********************************************************************/
/* End protection against multiple includes                         */
/********************************************************************/
#endif

/********************************************************************/
/* End of module kxc.h. Change History follows                      */
/********************************************************************/
/*
 * $Log: kxchkxca.h,v $
 * Revision 1.1.1.1.10.2  2002/04/16 17:26:11  mboag
 * 1-61JHE KXCRC_NO_AVAILABLE_ELEMENT if in service flag off for all BEs in a BC
 *
 * Revision 1.1.1.1.10.1  2002/04/11 17:44:50  mboag
 * 1-4HNZH XC310 Add KXCAPPOPT_ROLLBK_OS_XACT to allow rollback action
 *         on any outstanding transactions on kxcTerminate()
 *
 * Revision 1.1.1.1  2001/05/16 09:49:20  pross
 * Created from CTVC xcv310 010511-F1
 *
// 
//    Rev 1.38.2.0.1.1.1.2   20 Mar 2001 11:11:12   RHEND
// XCV310: 1-4O45T Changed max msg length
// 
//    Rev 1.38.2.0.1.1.1.1   02 Feb 2001 14:22:54   RHEND
// XCV310: I1-46VPU Included AS400 environment; changed VMS
// 
//    Rev 1.38.2.0.1.1.1.0   10 Jan 2001 12:29:16   PROSS
// XCV310: Merge from version XC300, revision 1.39
//
//    Rev 1.39   09 Jan 2001 13:20:48   PROSS
// XC300: IXCDEV v300 MVS Updates
//
//    Rev 1.38.2.0.1.1   20 Nov 2000 16:37:28   CWICK
// XC400: I1-28CFT Additional LDIF failure defines
//
//    Rev 1.38.2.0.1.0   17 Nov 2000 17:41:26   PROSS
// XC400: I1-2UJ1H HPUX compiler warnings, mainly macro casts
//
//    Rev 1.38.2.0   07 Nov 2000 17:45:38   CWICK
// XC310: Merge from version XC301, revision 1.38.1.0
//
//    Rev 1.38.1.0   02 Nov 2000 14:36:20   CWICK
// XC301: IXCDEV Tru/64 port
//
//    Rev 1.38   04 Sep 2000 08:40:54   RHEND
// XC300: I1-2EE9H Added KXCOP_MQSI
//
//    Rev 1.37   18 Apr 2000 10:58:58   MMALC
// XC300: I1-SK35 Separate the HP-UX 10 and 11 platform ID's
//
//    Rev 1.36   12 Apr 2000 14:38:02   MMALC
// XC300: I662403 Remove some unused LDAP/LDIF error codes
//
//    Rev 1.35   28 Feb 2000 15:30:20   RHEND
// XC300: I669016 Added PEEK_NOT_SUPPORTED error code
//
//    Rev 1.34   28 Feb 2000 14:39:04   MMALC
// XC300: I667870 Add new call type QMGRCONNECT
//
//    Rev 1.33   10 Feb 2000 13:47:48   MMALC
// XC300: I664668 Add new optype definitions
//
//    Rev 1.32   18 Jan 2000 11:38:08   RHENL
// XC300: I804.029 added CALLTYPE_OMBASE and KXCOP_KXC_OM
//
//    Rev 1.31   20 Dec 1999 13:54:22   MMALC
// XC300: I628747 Add CALLTYPE_GETBUFFER
//
//    Rev 1.30   15 Dec 1999 14:46:48   RHEND
// XC300: I804.037 Reinstaed SetDefault macros; removed old defaults
//
//    Rev 1.29   24 Nov 1999 15:27:40   CWICK
// XC300: (I804.030) Rounding for DnLen in dir access macros
//
//    Rev 1.28   23 Nov 1999 12:18:28   MMALC
// XC300: I804.035 Add new XPORTDATA call type
//
//    Rev 1.27   12 Nov 1999 14:18:08   MMALC
// XC300: (I804.036) Move Macros to kxchamca.h
//
//    Rev 1.26   09 Nov 1999 11:21:10   MMALC
// XC300: (I804.036) Add KXCAPPOPT_USEDEFAULT
//
//    Rev 1.25   05 Nov 1999 09:22:18   MMALC
// XC300: (I804.003) Fix Dir access macros
//
//    Rev 1.24   04 Nov 1999 15:15:00   MMALC
// XC300: I804.036 Add InitProfile to KXCAPPDEF
//
//    Rev 1.23   01 Nov 1999 15:21:36   CWICK
// XC300: I804.030 Allow for NULL char in TotalLen in kxcSetDirDn
//
//    Rev 1.22   28 Oct 1999 17:14:54   MMALC
// XC300: (I804.009) Remove FFDC Call Types
//
//    Rev 1.21   27 Oct 1999 11:27:02   MMALC
// XC300: (I804.003) Add return codes UPDATE/DELETE FAILED
//
//    Rev 1.20   26 Oct 1999 17:08:38   MMALC
// XC300: (I804.003) Remove mpStfrom kxcDirGetValue macro
//
//    Rev 1.19   18 Oct 1999 20:29:20   MMALC
// XC300: (I804.009) Add new CALLTYPES
//
//    Rev 1.18   18 Oct 1999 18:23:46   DREES
// XC300: I804.024 PubSub changes
//
//    Rev 1.17   11 Oct 1999 15:55:16   RHEND
// XC300: (I804.003) Added version in KXCSEARCHREQ
//
//    Rev 1.16   11 Oct 1999 15:31:54   MMALC
// XC300: I804.009 Add Write Entry calltype
//
//    Rev 1.15   29 Sep 1999 14:52:18   MMALC
// XC300: (I804.003) Update Dir access macros
//
//    Rev 1.14   10 Sep 1999 15:12:46   RHEND
// XC300: I804.032 New versions of KXCAPPDEF & KXCSTORQUEUEDEF
//
//    Rev 1.13   10 Sep 1999 11:43:44   CWICK
// XC300: (I804.003) Changed KXCDOPT_SEARCH_* literals
//
//    Rev 1.12   19 Aug 1999 17:11:34   MMALC
// XC300: (I804.003) Add MODCOUNT error code to list
//
//    Rev 1.11   12 Aug 1999 10:51:04   CWICK
// XC300: I804.020 Added KXCRC_UNKNOWN_NODE for failed LDAP Node lookup
//
//    Rev 1.10   11 Aug 1999 19:15:44   MMALC
// XC300: (I804.003) Add write macros for directory access API calls
//
//    Rev 1.9   11 Aug 1999 17:36:28   MMALC
// XC300: (I804.003) Add new directory access structures and macros
//
//    Rev 1.8   04 Aug 1999 11:49:28   KOGIL
// XC300: I804.019 Modified _NOEAR to NOERRORPROF
//
//    Rev 1.7   28 Jul 1999 10:17:10   CWICK
// XC300: I804.001 Added KXCOP_CTLIB
//
//    Rev 1.6   06 Jul 1999 15:10:30   RHEND
// XC300: I804.025 Added CoD and CoA Constants
//
//    Rev 1.5   21 Jun 1999 11:22:30   TTATA
// XC300: I804.004 Added Status to KXCPUTOPT/KXCGETOPT, Magname to KXCMSG
//
//    Rev 1.4   15 Jun 1999 17:17:04   MMALC
// XC300: (I804.003) Dir lookup attribute values returned as a pointer to a list of pointers
//
//    Rev 1.3   14 Jun 1999 10:06:52   MMALC
// XC300: I804.003 Add new API prototypes and search options for Directory look up
//
//    Rev 1.2   20 Apr 1999 11:24:14   GATAY
// XC300: (ID804.014) Add kxcSetILSDefault macro
//
//    Rev 1.1   19 Apr 1999 15:59:02   GATAY
// XC300: ID804.014 New externals for ILSData buffer support
//
//    Rev 1.0   19 Feb 1999 18:04:02   GATAY
// XC300: Initial Upload for BSP3.0
 */
/********************************************************************/
