/********************************************************************
 *                                                                    
 * Module      : kxmufful.c
 *                                                                    
 * Description : Full User Defined Functions C API Sample
 *                                                                    
 * This source is an example of the map conversion library,
 * which provides custom conversion functions (located in other modules):
 * "tolower"
 * "toupper"
 * "substring"
 * "pow"
 * "sum"
 * "makeGroup"
 * "addForEach"
 * "findGroup"
 * "glueElems"
 * "getColors"
 * "getShifts"
 * "reverseElems"
 * "negateBoolean"
 * "isTodayDate"
 * "getInitValue"
 *                                                                    
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 * 
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 * 
 ********************************************************************
 * $Id: kxmufful.c,v 1.12.2.1 2001/09/07 12:19:37 e123479 Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufful.c,v $
 * $Revision: 1.12.2.1 $                                                   
 ********************************************************************/

#include "kxmufful.h"

char* initValue = NULL;

/* Utility function to return error message */
void setError( const char* errorMsg, const char* file, int line )
{
    KXMERROR error = { KXMERROR_DEFAULT };
    strcpy( error.Primary.ErrorMessage, errorMsg );
    sprintf( error.Primary.Location, "File '%.64s' line %d.", file, line );
    kxmSetLastError( &error ); 
}

/* initialization function */
static KXMBOOL libInitialize( KXMFUNLIB* lib )
{
    char val[] = "Initialized value";
    initValue = malloc(sizeof(val));
    if (!initValue)
    {
        SET_ERROR("Memory allocation failed");
        return FALSE;
    }
    strcpy(initValue, val);
    return TRUE;
}

/* termination function */
static KXMBOOL libTerminate( KXMFUNLIB* lib )
{
    if (initValue)
    {
        free(initValue);
    }
    return TRUE;
}

/* list of functions defined by the library */
static KXMFUN* sampleFuncs[] = 
{
    &tolowerFunc, 
    &toupperFunc, 
    &substrFunc, 
    &powFunc,
    &sumFunc,
    &makeGroupFunc,
    &addForEachFunc,
    &findGroupFunc,
    &glueElemsFunc,
    &getColorsFunc,
    &getShiftsFunc,
    &reverseElemsFunc,
    &negateBooleanFunc,
    &isTodayDateFunc,
    &getInitValueFunc,
    /* last element must be NULL */
    NULL
};

/* library structure */  
static KXMFUNLIB sampleLib = 
{ 
    KXMFUNLIB_SIGVER,  
    KXMFUNAPIVER_CURRENT,
    "Sample Function Library",
    sampleFuncs,
    libInitialize,
    libTerminate
};

/* library entry function */
KXMFUN_DEFINEENTRY( sampleLib )

