/********************************************************************
 *                                                                    
 * Module      : kxmufprm.c
 *                                                                    
 * Description : User Defined Functions C API Sample
 *
 * This source is part of the full Function Library example (kxmufful).
 * This source shows how to handle a primitive data type and how to set 
 * a Boolean result.
 *
 * This file contains the following user defined function in it:
 * "isTodayDate"
 *                                                                    
 ********************************************************************
 *                                                                    
 * Copyright 2001, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************
 * 
 * Disclaimer:  This sample is provided for information               
 *              purposes only.  Candle neither makes any warranties   
 *              or representations regarding this software, or its    
 *              use.  This file is provided "as is" and Candle    
 *              expressly disclaims the warranties of                 
 *              merchantability and fitness for a particular          
 *              purpose.                                              
 * 
 ********************************************************************
 * $Id: kxmufprm.c,v 1.5.2.1 2001/09/07 12:19:38 e123479 Exp $           
 * $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmufun/kxmufprm.c,v $
 * $Revision: 1.5.2.1 $                                                   
 ********************************************************************/

#include "kxmufful.h"
#include <time.h>

/********************************************************************
 *  isTodayDate( date )
 *
 *  Returns true, if specified date is equal to current date
 ********************************************************************/
static KXMBOOL isTodayDate( const KXMFUNARGS* args, KXMFUNRES* result )
{
    KXMDATE date = { KXMDATE_DEFAULT };
    time_t xt;
    struct tm* ct;
    int ret_val;	/* Return value from function */

    /* Get the parameter value */
    if ( !kxmGetDate( *args->Args, NULL, &date, NULL, NULL ) ) 
    {
        SET_ERROR("*** ERROR: isTodayDate: Unable to get 'date' value");
        return FALSE;
    }

    /* Perform calculations of the main function */
    time( &xt );
    ct = gmtime( &xt );
    ret_val = ( date.Day == (KXMULONG)ct->tm_mday && 
		date.Month == (KXMULONG)ct->tm_mon + 1 && 
		date.Year == (KXMULONG)ct->tm_year + 1900 );

    /* Set the return value of the function */
    if ( !kxmSetBool( result->Value, NULL, ret_val, NULL, NULL ) ) 
    {
        SET_ERROR("*** ERROR: isTodayDate: Unable to set resulting value");
        return FALSE;
    }

    /* Indicate the function completed successfully */
    return TRUE;
}

/* Typed Arguement Definitions for isTodayDate */
static KXMFUNARG isTodayDateArgs[] = 
{
    { KXMFUNARG_SIGVER, "date", NULL, KXMVALTYPE_DATE | KXMVALTYPE_DATETIME }
};

/* Function Definition for isTodayDate */
KXMFUN isTodayDateFunc =
{ 
    KXMFUN_SIGVER, 
    "isTodayDate",
    "Returns true, if specified date is equal to current date.",
    isTodayDate, 
    1, 1, 1, 
    isTodayDateArgs, 
    KXMVALTYPE_BOOL
};

/********************************************************************
 *  getInitValue()
 *
 *  Returns initialization string
 ********************************************************************/
static KXMBOOL getInitValue( const KXMFUNARGS* args, KXMFUNRES* result )
{
    extern char* initValue;
    if ( !kxmSetString( result->Value, NULL, initValue, strlen(initValue),
        NULL, NULL ) )
    {
    	SET_ERROR("*** ERROR: getInitValue: Unable to set return value");
        return FALSE;
    }
    return TRUE;
}

/* Function Definition for getStartTime */
KXMFUN getInitValueFunc =
{ 
    KXMFUN_SIGVER, 
    "getInitValue",
    "Returns initialization value.",
    getInitValue, 
    0, 0, 0, 
    (KXMFUNARG*)NULL, 
    KXMVALTYPE_STRING
};

